Package: ato
Title: Download and Tidy Australian Taxation Office Data
Version: 0.1.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com", role = c("aut", "cre"))
Description: Fetch Australian Taxation Office ('ATO') Taxation
    Statistics and related datasets via the 'data.gov.au' Comprehensive
    Knowledge Archive Network ('CKAN') API
    <https://data.gov.au/data/api/3/>. Provides tidy access to
    individual, company, superannuation, goods and services tax
    ('GST'), fringe benefits tax ('FBT'), Voluntary Tax Transparency
    Code ('VTTC'), Pay As You Go ('PAYG') withholding, charity,
    excise, and Corporate Tax Transparency data, plus Division 293,
    Petroleum Resource Rent Tax, Medicare Levy Surcharge, fuel tax
    credits, compliance, and Working Holiday Maker aggregates.
    Includes reproducibility helpers (snapshot pinning, SHA-256
    cache integrity, session manifest, optional 'Zenodo' deposit),
    classification crosswalks ('ANZSIC' 2006 to 2020, 'ANZSCO' 2013
    to 2021), panel harmonisation, reconciliation against Final
    Budget Outcome totals, and real-terms and per-capita helpers
    backed by bundled Australian Bureau of Statistics ('ABS')
    Consumer Price Index and Estimated Resident Population series.
    Bridges to the 'taxstats' 2 per cent microdata sample via
    column-schema mapping. Data is published by the Australian
    Taxation Office under Creative Commons Attribution 2.5 Australia
    or 3.0 Australia licences (dataset-dependent).
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), httr2 (>= 1.0.0), jsonlite, readxl (>= 1.4.0),
        tools, utils
Suggests: digest, knitr, openssl, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://charlescoverdale.github.io/ato/,
        https://github.com/charlescoverdale/ato
BugReports: https://github.com/charlescoverdale/ato/issues
NeedsCompilation: no
Packaged: 2026-04-24 13:43:04 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 18:50:19 UTC
Built: R 4.6.0; ; 2026-05-01 21:27:21 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,ato_tbl)
export(ato_cache_info)
export(ato_catalog)
export(ato_charities)
export(ato_cite)
export(ato_clear_cache)
export(ato_companies)
export(ato_compliance)
export(ato_crosswalk)
export(ato_deflate)
export(ato_deposit_zenodo)
export(ato_division293)
export(ato_download)
export(ato_excise)
export(ato_fbt)
export(ato_fuel_tax_credits)
export(ato_gst)
export(ato_harmonise)
export(ato_help)
export(ato_individuals)
export(ato_individuals_age)
export(ato_individuals_occupation)
export(ato_individuals_postcode)
export(ato_individuals_sex)
export(ato_individuals_state)
export(ato_industry)
export(ato_international)
export(ato_irpd)
export(ato_manifest)
export(ato_manifest_clear)
export(ato_manifest_write)
export(ato_medicare_levy)
export(ato_meta)
export(ato_payg)
export(ato_per_capita)
export(ato_prrt)
export(ato_rba)
export(ato_rdti)
export(ato_reconcile)
export(ato_schema_map)
export(ato_sha256)
export(ato_sme_benchmarks)
export(ato_snapshot)
export(ato_state_tax)
export(ato_super_funds)
export(ato_tax_expenditures)
export(ato_tax_gaps)
export(ato_to_taxstats)
export(ato_top_taxpayers)
export(ato_vttc)
export(ato_whm)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)

## -----------------------------------------------------------------------------
# library(ato)
# ato_snapshot("2026-04-24")
# 
# pc_panel <- ato_individuals_postcode(
#   year = c("2015-16", "2016-17", "2017-18", "2018-19",
#            "2019-20", "2020-21", "2021-22", "2022-23")
# )
# pc_panel <- ato_harmonise(pc_panel)
# 
# # For each year, rank postcodes by mean taxable income per return,
# # take top 1% of returns, compute their share of total income.
# top1 <- function(df) {
#   df <- df[order(-df$taxable_income / df$number_of_individuals), ]
#   cum_returns <- cumsum(df$number_of_individuals)
#   total_returns <- sum(df$number_of_individuals, na.rm = TRUE)
#   cutoff <- which(cum_returns >= 0.01 * total_returns)[1]
#   sum(df$taxable_income[seq_len(cutoff)], na.rm = TRUE) /
#     sum(df$taxable_income, na.rm = TRUE)
# }
# 
# shares <- by(pc_panel, pc_panel$year, top1)
# shares

## -----------------------------------------------------------------------------
# ctt <- ato_top_taxpayers(year = "2022-23")
# 
# # Effective tax rate = tax payable / taxable income, for entities
# # with positive taxable income. Drop zero-taxable rows (they bias
# # the ratio; rely on loss-makers analysis separately).
# ctt <- ctt[!is.na(ctt$taxable_income) & ctt$taxable_income > 0, ]
# ctt$etr <- ctt$tax_payable / ctt$taxable_income
# 
# by_industry <- aggregate(etr ~ entity_type, data = ctt, FUN = median)
# by_industry[order(-by_industry$etr), ]

## -----------------------------------------------------------------------------
# tg <- ato_tax_gaps()
# 
# library(ggplot2)
# ggplot(tg, aes(x = year, y = tax_gap_estimate,
#                colour = tax_gap_type)) +
#   geom_line() +
#   labs(title = "ATO estimated tax gaps over time",
#        x = NULL, y = "Estimated tax gap (AUD million)",
#        colour = "Gap type",
#        caption = "Source: ATO Taxation Statistics. Retrieved via ato package.") +
#   theme_minimal()

## -----------------------------------------------------------------------------
# help_data <- ato_help()
# 
# # Bucketed by age range; real-terms deflation to 2022-23
# help_data$real <- ato_deflate(help_data$total_debt,
#                                year = help_data$year,
#                                base = "2022-23")
# head(help_data)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)

## -----------------------------------------------------------------------------
# library(ato)
# head(ato_schema_map(), 15)

## -----------------------------------------------------------------------------
# library(ato)
# 
# ato_snapshot("2026-04-24")
# 
# mls <- ato_medicare_levy(year = "2022-23", component = "surcharge")
# 
# # Reconcile the published MLS total against FBO where available
# # (MLS does not have a direct FBO line; rolled into individuals
# # income tax net).
# ind_total <- sum(ato_individuals(year = "2022-23")$tax_payable,
#                  na.rm = TRUE)
# ato_reconcile(ind_total, "2022-23", "individuals_income_tax_net")

## -----------------------------------------------------------------------------
# # Rename columns to taxstats schema
# mls_ts <- ato_to_taxstats(mls)
# 
# # Now these columns are consistent with what `taxstats::taxstats1819`
# # uses. You can write analysis code that works on both.
# 
# # Example: use grattan to compute the new regime's tax for the 2% sample
# # library(taxstats)
# # library(grattan)
# # sample_2pc <- taxstats1819
# # sample_2pc$new_tax <- income_tax(
# #   income = sample_2pc$Taxable_Income,
# #   fy.year = "2023-24",
# #   ...
# # )
# # reform_cost <- sum(sample_2pc$new_tax - sample_2pc$Tax_assessed_amt,
# #                    na.rm = TRUE) * 50  # 2% -> 100%

# Individuals demographic cuts: age, sex, state.

#' Individual tax data by age range
#'
#' Returns Taxation Statistics Individuals Table 2 (approximately):
#' counts, total income, taxable income, and tax payable by age
#' range and (usually) sex. Age ranges are 5-year bands for most
#' of working life plus wider bands at the tails.
#'
#' @param year `"YYYY-YY"`, `"latest"`, or a vector of years.
#' @param sex One of `"all"` (default), `"male"`, or `"female"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics Individuals.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   Individuals explanatory notes}. Age-range breakdowns use the
#'   taxpayer's reported date of birth at lodgement; sex is
#'   self-reported on the return.
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_individuals_age(year = "2022-23", sex = "female"))
#' options(op)
#' }
ato_individuals_age <- function(year = "latest",
                                 sex = c("all", "male", "female")) {
  sex <- match.arg(sex)
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    fn <- function(year) ato_individuals_age(year = year, sex = sex)
    return(ato_stack_years(fn, resolved,
                           title_prefix = "ATO individuals by age"))
  }
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "age|individual(s)?02|individual_02")
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  sex_col <- ato_find_col(df, "sex")
  if (sex != "all" && !is.na(sex_col)) {
    df <- df[tolower(df[[sex_col]]) == sex, , drop = FALSE]
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO individuals by age ", year))
}

#' Individual tax data by sex
#'
#' Returns counts and aggregates split by sex. Thin wrapper around
#' the ATO "Selected items by sex" table.
#'
#' @param year `"YYYY-YY"`, `"latest"`, or a vector of years.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics Individuals.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   Individuals explanatory notes}. Age-range breakdowns use the
#'   taxpayer's reported date of birth at lodgement; sex is
#'   self-reported on the return.
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_individuals_sex(year = "2022-23"))
#' options(op)
#' }
ato_individuals_sex <- function(year = "latest") {
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    fn <- function(year) ato_individuals_sex(year = year)
    return(ato_stack_years(fn, resolved,
                           title_prefix = "ATO individuals by sex"))
  }
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "by.sex|sex|individual(s)?03|individual_03")
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO individuals by sex ", year))
}

#' Individual tax data by state or territory
#'
#' Returns counts and aggregates by state. Thin wrapper around
#' the ATO "Selected items by state/territory" table.
#'
#' @param year `"YYYY-YY"`, `"latest"`, or a vector of years.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics Individuals.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   Individuals explanatory notes}. Age-range breakdowns use the
#'   taxpayer's reported date of birth at lodgement; sex is
#'   self-reported on the return.
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_individuals_state(year = "2022-23"))
#' options(op)
#' }
ato_individuals_state <- function(year = "latest") {
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    fn <- function(year) ato_individuals_state(year = year)
    return(ato_stack_years(fn, resolved,
                           title_prefix = "ATO individuals by state"))
  }
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "state|territory|individual(s)?04|individual_04")
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO individuals by state ", year))
}

# Charity tax concessions

#' Charity and deductible gift recipient data
#'
#' Returns the ATO's data on income tax-exempt entities and
#' Deductible Gift Recipients (DGRs): entity counts, income,
#' expenditure, and gift deductions by charity subtype and
#' state. Covers public benevolent institutions, health promotion
#' charities, environmental organisations, and other DGR
#' categories.
#'
#' Used by Treasury (charity tax expenditure estimates),
#' researchers studying the non-profit sector, and civil society
#' policy analysts.
#'
#' @param year Income year in `"YYYY-YY"` form (e.g. `"2021-22"`)
#'   or `"latest"`.
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD.
#'
#' @source Australian Taxation Office charity statistics on
#'   data.gov.au. Licensed CC BY 2.5 AU.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   ch <- ato_charities(year = "2021-22")
#'   head(ch)
#' })
#' options(op)
#' }
ato_charities <- function(year = "latest") {
  result    <- ato_ckan_search(q = "charity tax", rows = 10L)
  pkgs      <- result$results
  ids       <- vapply(pkgs, function(p) p$name %||% "", character(1))
  char_ids  <- ids[grepl("charit|deductible.gift|dgr|not.for.profit",
                           ids, ignore.case = TRUE)]

  if (length(char_ids) == 0L) {
    cli::cli_abort(c(
      "Could not find a charity / DGR package on data.gov.au.",
      "i" = "Browse: {.url https://data.gov.au/data/organization/australiantaxationoffice}"
    ))
  }
  pkg_id <- char_ids[1L]
  ato_check_staleness(pkg_id)

  if (identical(year, "latest")) {
    pkg  <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year  <- if (length(years) > 0L) max(years) else "2021-22"
  } else {
    year <- ato_resolve_year(year)
  }

  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO charity / DGR statistics ", year))
}

# Tier 4 new functions : live tests only

test_that("live: ato_fbt fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_fbt()
  expect_s3_class(x, "ato_tbl")
  expect_gt(nrow(x), 0L)
})

test_that("live: ato_payg fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_payg()
  expect_s3_class(x, "ato_tbl")
  expect_gt(nrow(x), 0L)
})

test_that("live: ato_charities fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_charities()
  expect_s3_class(x, "ato_tbl")
  expect_gt(nrow(x), 0L)
})

test_that("live: ato_vttc fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_vttc()
  expect_s3_class(x, "ato_tbl")
  expect_gt(nrow(x), 0L)
})

# ATO compliance and debt book: audit yield, debt under dispute,
# collectable debt. Key integrity indicators.

#' ATO compliance program outcomes
#'
#' Returns the ATO's annual compliance program outcomes: audit
#' yield (tax raised from audits), settled disputes, collectable
#' debt, and compliance cost recovery. These appear in the ATO
#' annual report and related data.gov.au releases.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param metric One of `"overview"` (default), `"debt"`
#'   (collectable vs insolvency vs disputed), or `"audit"`
#'   (liabilities raised by program area).
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office annual report data.
#'   Licensed CC BY 3.0 AU.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_compliance(year = "2022-23", metric = "debt"))
#' options(op)
#' }
ato_compliance <- function(year = "latest",
                            metric = c("overview", "debt", "audit")) {
  metric <- match.arg(metric)
  results <- tryCatch(
    ato_ckan_search(q = "compliance+OR+annual-report", rows = 50L),
    error = function(e) list(results = list())
  )
  pattern <- switch(metric,
    overview = "compliance.*overview|annual.report",
    debt     = "debt|collectable",
    audit    = "audit|liabilit"
  )
  hit <- NULL
  if (!is.null(results$results)) {
    for (p in results$results) {
      name <- p$name %||% ""
      if (grepl(pattern, name, ignore.case = TRUE)) {
        hit <- p
        break
      }
    }
  }
  if (is.null(hit)) {
    cli::cli_abort(c(
      "No compliance dataset found for metric {.val {metric}}.",
      "i" = "Try {.code ato_catalog()} to browse available datasets."
    ))
  }
  res <- hit$resources[[1L]]
  url <- res$url %||% ""
  df <- if (grepl("\\.csv$", url, ignore.case = TRUE)) {
    ato_fetch_csv(url)
  } else {
    ato_fetch_xlsx(url, sheet = 1)
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 3.0 AU",
              title = paste0("ATO compliance ", year, " (", metric, ")"))
}

# Classification crosswalks: bundled lookup tables for ANZSIC
# 2006 -> 2020, ANZSCO 2013 -> 2021, and a state-anchor postcode
# file. Full fine-grained crosswalks (4-digit ANZSIC, 6-digit
# ANZSCO, postcode to SA2/LGA/CED) should be fetched from ABS.

#' Load a bundled ATO crosswalk table
#'
#' Returns one of the bundled classification crosswalks. Used
#' internally by [ato_harmonise()] and available for user-level
#' panel work.
#'
#' Bundled crosswalks (at division/major-group level):
#' - `"anzsic"`: ANZSIC 2006 to 2020 (19 divisions, complete)
#' - `"anzsco"`: ANZSCO 2013 to 2021 (8 major groups, complete)
#' - `"postcode"`: postcode first-digit to state anchors
#' - `"cpi"`: ABS CPI annual, base 2011-12 = 1.0
#' - `"erp"`: ABS Estimated Resident Population, June 30 annual
#' - `"budget"`: Final Budget Outcome reference totals
#'
#' For 4-digit ANZSIC, 6-digit ANZSCO, or postcode-to-SA2/LGA/CED
#' crosswalks, fetch the full tables from ABS. The bundled
#' division/major-group level covers cross-year ATO Taxation
#' Statistics joins at the industry headings used in all ATO
#' tables.
#'
#' @param name One of `"anzsic"`, `"anzsco"`, `"postcode"`,
#'   `"cpi"`, `"erp"`, `"budget"`.
#'
#' @return A data frame.
#'
#' @references
#' Australian Bureau of Statistics (2006). \emph{Australian and
#'   New Zealand Standard Industrial Classification (ANZSIC)}.
#'   Catalogue 1292.0.
#'
#' Australian Bureau of Statistics (2020). \emph{ANZSIC 2006
#'   Update}, cat. 1292.0, divisional structure. Used by ATO
#'   Taxation Statistics from 2022-23.
#'
#' Australian Bureau of Statistics (2013). \emph{Australian and
#'   New Zealand Standard Classification of Occupations
#'   (ANZSCO)}. Catalogue 1220.0.
#'
#' Australian Bureau of Statistics (2022). \emph{ANZSCO Revised
#'   Edition}, cat. 1220.0. Used by ATO Taxation Statistics from
#'   2022-23 onward.
#'
#' @family harmonisation
#' @export
#' @examples
#' ato_crosswalk("anzsic")
#' ato_crosswalk("cpi")
ato_crosswalk <- function(name = c("anzsic", "anzsco", "postcode",
                                    "cpi", "erp", "budget")) {
  name <- match.arg(name)
  file <- switch(name,
    anzsic   = "anzsic_2006_2020_crosswalk.csv",
    anzsco   = "anzsco_2013_2021_crosswalk.csv",
    postcode = "postcode_state_crosswalk.csv",
    cpi      = "abs_cpi_annual.csv",
    erp      = "abs_erp_annual.csv",
    budget   = "budget_reference_totals.csv"
  )
  path <- system.file("extdata", file, package = "ato")
  if (!nzchar(path) || !file.exists(path)) {
    cli::cli_abort("Crosswalk file {.file {file}} not installed.")
  }
  utils::read.csv(path, stringsAsFactors = FALSE,
                  na.strings = c("", "NA"), check.names = FALSE)
}

test_that("ato_crosswalk returns a data frame for each type", {
  for (nm in c("anzsic", "anzsco", "postcode", "cpi", "erp", "budget")) {
    cw <- ato_crosswalk(nm)
    expect_s3_class(cw, "data.frame")
    expect_gt(nrow(cw), 0L)
  }
})

test_that("anzsic crosswalk has 19 divisions", {
  cw <- ato_crosswalk("anzsic")
  expect_equal(nrow(cw), 19L)
  expect_true(all(c("anzsic_2006_code", "anzsic_2020_code") %in% names(cw)))
})

test_that("cpi crosswalk has expected structure", {
  cw <- ato_crosswalk("cpi")
  expect_true(all(c("financial_year", "cpi_all_groups_australia") %in%
                  names(cw)))
  expect_true("2022-23" %in% cw$financial_year)
})

test_that("erp crosswalk values are reasonable", {
  cw <- ato_crosswalk("erp")
  erp_2022 <- cw$erp_june_australia_thousands[cw$financial_year == "2022-23"]
  # Australia's population around 26 million
  expect_gt(erp_2022, 25000)
  expect_lt(erp_2022, 28000)
})

# Medicare Levy and Medicare Levy Surcharge (MLS)

#' Medicare Levy and Medicare Levy Surcharge
#'
#' Returns aggregate Medicare Levy and MLS data from Taxation
#' Statistics Individuals. The 2% Medicare Levy is on most
#' taxable income; MLS is an additional 1.0 to 1.5% on high-income
#' earners without adequate private hospital cover. Used in
#' private health insurance reform analysis.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param component One of `"levy"` (default, standard Medicare
#'   Levy) or `"surcharge"` (MLS).
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics
#'   Individuals. Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Medicare Levy Act 1986};
#'   \emph{A New Tax System (Medicare Levy Surcharge -- Fringe
#'   Benefits) Act 1999}.
#'
#' Productivity Commission (2015). \emph{Efficiency in Health}.
#'   Analysis of Medicare Levy and MLS distributional effects.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_medicare_levy(year = "2022-23", component = "surcharge"))
#' options(op)
#' }
ato_medicare_levy <- function(year = "latest",
                               component = c("levy", "surcharge")) {
  component <- match.arg(component)
  id <- ato_ts_package_id(year)
  pattern <- if (component == "surcharge") {
    "medicare.levy.surcharge|mls|surcharge"
  } else {
    "medicare.levy|individual(s)?0[12]"
  }
  res <- ato_ckan_resolve(id, pattern)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Medicare ", component, " ", year))
}

# Research and Development Tax Incentive

#' R&D Tax Incentive claimants
#'
#' Returns the annual "Report of data about Research and
#' Development Tax Incentive entities": claimants, claimed
#' expenditure, refundable and non-refundable tax offsets by
#' industry and company size. Treasury and DISR use this series
#' to evaluate the R&D Tax Incentive programme, which is the
#' largest single element of Australia's business innovation
#' policy (AUD 2 billion+ per year).
#'
#' @param year Income year in `"YYYY-YY"` form (e.g.
#'   `"2022-23"`) or `"latest"`. Current releases cover 2021-22
#'   and 2022-23.
#'
#' @return An `ato_tbl` with one row per entity (or aggregated
#'   cell, depending on the release schema). Monetary values in
#'   nominal AUD.
#'
#' @source Australian Taxation Office Research and Development
#'   Tax Incentive report. Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Income Tax Assessment Act
#'   1997}, Division 355 (Research and Development Tax Incentive).
#'
#' Department of Industry, Science and Resources and Australian
#'   Taxation Office (annual). \emph{R&DTI Transparency Report}.
#'   Jointly administered programme methodology.
#'
#' Ferris, B., Finkel, A. and Fraser, J. (2016). \emph{Review of
#'   the R&D Tax Incentive}. Australian Government review
#'   (the "Three Fs review") informing subsequent programme design.
#'
#' Organisation for Economic Co-operation and Development (annual).
#'   \emph{R&D Tax Incentives Database}. International comparator
#'   data for R&D tax expenditures.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   rdti <- ato_rdti(year = "2022-23")
#'   head(rdti)
#' })
#' options(op)
#' }
ato_rdti <- function(year = "latest") {
  pkg_id <- ATO_PACKAGE_IDS$rdti
  if (identical(year, "latest")) {
    pkg <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year <- if (length(years) > 0L) max(years) else "2022-23"
  } else {
    year <- ato_resolve_year(year)
  }
  ato_check_staleness(pkg_id)
  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO R&D Tax Incentive ", year))
}

# Tier 2 function signature + validation tests.
# Live-network tests are in a separate file guarded by
# ATO_LIVE_TESTS=true.

test_that("ato_excise validates table arg", {
  expect_error(ato_excise(table = "wine"))
})

test_that("ato_help validates scheme arg", {
  expect_error(ato_help(scheme = "hecs"))
})

test_that("ato_irpd validates table 1-3", {
  expect_error(ato_irpd(table = 4L))
  expect_error(ato_irpd(table = 0L))
})

test_that("ato_companies validates table arg", {
  expect_error(ato_companies(table = "revenue"))
})

test_that("ato_companies accepts new table names", {
  # match.arg should succeed without hitting the network
  tbl_names <- c("industry", "snapshot", "key_items_by_size",
                 "entity_type", "industry_by_size", "sub_industry",
                 "taxable_status", "source", "expenses")
  for (t in tbl_names) {
    expect_silent(match.arg(t, tbl_names))
  }
})

test_that("ato_individuals_postcode multi-year routes to stack", {
  # We can't test the full stack without the network, but we can
  # verify the length-1 path still works via match.arg on year.
  expect_error(
    ato_individuals_postcode(year = c("not-a-year", "also-not")),
    "parse year"
  )
})

test_that("live: ato_tax_gaps fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_tax_gaps()
  expect_s3_class(x, "ato_tbl")
})

test_that("live: ato_rdti fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_rdti(year = "2022-23")
  expect_s3_class(x, "ato_tbl")
})

test_that("live: ato_irpd table 1 fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_irpd(year = "2023-24", table = 1L)
  expect_s3_class(x, "ato_tbl")
})

test_that("live: ato_excise fetches each table", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  for (t in c("excise_rates", "ftc_rates", "beer", "spirits")) {
    expect_s3_class(ato_excise(table = t), "ato_tbl")
  }
})

test_that("live: ato_sme_benchmarks fetches", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  x <- ato_sme_benchmarks()
  expect_s3_class(x, "ato_tbl")
})

test_that("live: ato_help fetches HELP and VSL", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  expect_s3_class(ato_help("help"), "ato_tbl")
  expect_s3_class(ato_help("vsl"), "ato_tbl")
})

test_that("live: ato_companies dispatches across Tables 1-9", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  # Test the snapshot and key_items_by_size tables that Grattan
  # Institute uses in corporate-tax reform work.
  expect_s3_class(ato_companies(year = "2022-23", table = "snapshot"),
                  "ato_tbl")
  expect_s3_class(ato_companies(year = "2022-23", table = "entity_type"),
                  "ato_tbl")
})

test_that("live: ato_individuals_postcode multi-year stacks", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  panel <- ato_individuals_postcode(year = c("2021-22", "2022-23"),
                                    state = "TAS")
  expect_s3_class(panel, "ato_tbl")
  expect_true("year" %in% names(panel))
  expect_equal(length(unique(panel$year)), 2L)
})

# GST + industry aggregates

#' GST and activity statement ratios
#'
#' Returns the Taxation Statistics GST tables (T1-T5) or the
#' Activity Statement Ratios (A1-A5) for the requested year.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param table One of `"overview"` (default, GST T1), `"state"`
#'   (GST by state), `"industry"` (GST by ANZSIC), or
#'   `"ratios"` (Activity Statement Ratios).
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   GST and Activity Statement Ratios explanatory notes}.
#'
#' Commonwealth of Australia. \emph{A New Tax System (Goods and
#'   Services Tax) Act 1999}. Enabling legislation for the 10 per
#'   cent value-added tax introduced 1 July 2000.
#'
#' Productivity Commission (2018). \emph{Horizontal Fiscal
#'   Equalisation}. Background reference on the GST distribution
#'   formula across states.
#'
#' @family gst
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   g <- ato_gst(year = "2022-23", table = "industry")
#'   head(g)
#' })
#' options(op)
#' }
ato_gst <- function(year = "latest",
                    table = c("overview", "state", "industry", "ratios")) {
  table <- match.arg(table)
  id <- ato_ts_package_id(year)
  pattern <- switch(table,
    overview = "gst0?1|gst_01",
    state    = "gst0?[23]|gst_02|gst_03",
    industry = "gst0?[45]|gst_04|gst_05",
    ratios   = "activity.statement|ratio"
  )
  res <- ato_ckan_resolve(id, pattern)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO GST ", year, " (", table, ")"))
}

#' Industry aggregates across entity types
#'
#' Derived helper that returns an ANZSIC industry breakdown based
#' on either individual, company, or all entities for the year.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param entity One of `"individual"`, `"company"` (default), or
#'   `"all"`.
#' @param anzsic Optional substring filter on industry name.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics.
#'   Licensed CC BY 2.5 AU.
#'
#' @family gst
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   i <- ato_industry(year = "2022-23", entity = "company",
#'                     anzsic = "manufacturing")
#'   head(i)
#' })
#' options(op)
#' }
ato_industry <- function(year = "latest",
                          entity = c("company", "individual", "all"),
                          anzsic = NULL) {
  entity <- match.arg(entity)
  id <- ato_ts_package_id(year)

  fetch_one <- function(p) {
    res <- ato_ckan_resolve(id, p)
    url <- res$url %||% ""
    df <- ato_fetch_xlsx(url, sheet = 1)
    df$entity <- entity
    df
  }

  df <- switch(entity,
    company    = fetch_one("company04|company_04"),
    individual = fetch_one("individual(s)?05|individual_05"),
    all        = {
      c1 <- fetch_one("company04|company_04")
      i1 <- fetch_one("individual(s)?05|individual_05")
      common <- intersect(names(c1), names(i1))
      rbind(c1[, common, drop = FALSE], i1[, common, drop = FALSE])
    }
  )

  if (!is.null(anzsic)) {
    ind_col <- ato_find_col(df, "industry")
    if (!is.na(ind_col)) {
      ind_pattern <- paste(tolower(anzsic), collapse = "|")
      df <- df[grepl(ind_pattern, tolower(df[[ind_col]])), , drop = FALSE]
    }
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = "https://data.gov.au/data/organization/australiantaxationoffice",
              licence = "CC BY 2.5 AU",
              title = paste0("ATO industry ", year, " (", entity, ")"))
}

# Error path tests (all offline)

test_that("new_ato_tbl rejects non-data-frame input", {
  expect_error(ato:::new_ato_tbl(list(a = 1)))
  expect_error(ato:::new_ato_tbl(1:3))
})

test_that("ato_irpd rejects table outside 1-3", {
  expect_error(ato_irpd(table = 4L))
  expect_error(ato_irpd(table = 0L))
  expect_error(ato_irpd(table = 5L))
})

test_that("ato_excise rejects unknown table", {
  expect_error(ato_excise(table = "wine"))
  expect_error(ato_excise(table = "gst"))
})

test_that("ato_help rejects unknown scheme", {
  expect_error(ato_help(scheme = "hecs"))
  expect_error(ato_help(scheme = "austudy"))
})

test_that("ato_companies rejects unknown table", {
  expect_error(ato_companies(table = "revenue"))
  expect_error(ato_companies(table = "profit"))
})

test_that("ato_super_funds rejects unknown type", {
  expect_error(ato_super_funds(type = "industry"))
})

test_that("ato_gst rejects unknown table", {
  expect_error(ato_gst(table = "income"))
})

test_that("ato_top_taxpayers rejects unknown entity_type", {
  expect_error(ato_top_taxpayers(entity_type = "partnership"))
})

test_that("ato_top_taxpayers rejects unknown sheet", {
  expect_error(ato_top_taxpayers(sheet = "gst"))
})

test_that("ato_individuals_occupation rejects bad sex argument", {
  expect_error(ato_individuals_occupation(sex = "nonbinary"))
})

test_that("ato_resolve_year errors on nonsense input", {
  expect_error(ato:::ato_resolve_year("not-a-year"))
  expect_error(ato:::ato_resolve_year("20"))
  expect_error(ato:::ato_resolve_year("abc"))
})

test_that("ato_cite rejects non-tbl non-string input", {
  expect_error(ato_cite(123))
  expect_error(ato_cite(list(a = 1)))
})

test_that("ato_meta rejects non-tbl non-string input", {
  expect_error(ato_meta(42))
  expect_error(ato_meta(list()))
})

test_that("ato_download errors on unknown parse value", {
  expect_error(ato_download("any-id", parse = "json"))
})

test_that("ato_gst validates table choice", {
  expect_error(ato_gst(table = "junk"))
})

test_that("ato_industry validates entity arg", {
  expect_error(ato_industry(entity = "trust"))
})

test_that("ato_gst live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  g <- ato_gst(year = "2022-23", table = "industry")
  expect_s3_class(g, "ato_tbl")
})

# Superannuation Taxation Statistics

#' Superannuation fund aggregates
#'
#' Returns Taxation Statistics Super Funds tables or Self-Managed
#' Superannuation Fund ('SMSF') aggregates, depending on `type`.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param type One of `"apra"` (APRA-regulated funds, default),
#'   `"smsf"` (SMSF statistical overview), or `"all"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics Super
#'   Funds tables + SMSF statistical overview. Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   Super funds and SMSF explanatory notes}. Distinguishes
#'   reporting populations: APRA-regulated large funds, SMSFs,
#'   and Pooled Superannuation Trusts.
#'
#' Australian Prudential Regulation Authority (annual).
#'   \emph{Annual Superannuation Bulletin}. Complementary
#'   APRA-regulated fund statistics.
#'
#' Commonwealth of Australia. \emph{Superannuation Industry
#'   (Supervision) Act 1993} (SIS Act); \emph{Superannuation
#'   Guarantee (Administration) Act 1992} (SGAA).
#'
#' Productivity Commission (2018). \emph{Superannuation:
#'   Assessing Efficiency and Competitiveness}. Inquiry report.
#'
#' @family super
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   s <- ato_super_funds(year = "2022-23", type = "apra")
#'   head(s)
#' })
#' options(op)
#' }
ato_super_funds <- function(year = "latest",
                             type = c("apra", "smsf", "all")) {
  type <- match.arg(type)
  if (type == "smsf") {
    res <- ato_ckan_resolve(ATO_PACKAGE_IDS$smsf,
                            "annual|overview|smsf")
    url <- res$url %||% ""
    df <- ato_fetch_xlsx(url, sheet = 1)
    return(new_ato_tbl(df,
                        source = url,
                        licence = "CC BY 2.5 AU",
                        title = "ATO SMSF annual overview"))
  }

  id <- ato_ts_package_id(year)
  pattern <- if (type == "apra") {
    # Real resource filenames are ts<YY>fund0[1-4]... (e.g. ts23fund01aprasbyyear.xlsx);
    # the display name is "SuperFunds - Table N".
    "fund0[1-4]|superfunds"
  } else {
    "super"
  }
  res <- ato_ckan_resolve(id, pattern)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO super funds ", year, " (", type, ")"))
}

test_that("ato_super_funds validates type arg", {
  expect_error(ato_super_funds(type = "corporate"))
})

test_that("ato_super_funds live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  s <- ato_super_funds(year = "2022-23", type = "apra")
  expect_s3_class(s, "ato_tbl")
})

# CKAN API wrapper for data.gov.au

#' @noRd
CKAN_BASE <- "https://data.gov.au/data/api/3/action"

#' @noRd
ATO_ORG <- "australiantaxationoffice"

#' Query the CKAN package_search endpoint for ATO packages
#'
#' @param q Additional query string (joined with `+` to the org filter).
#' @param rows Max rows to return per page.
#' @return Parsed list with `count` and `results` (list of packages).
#' @noRd
ato_ckan_search <- function(q = NULL, rows = 200L) {
  query <- paste0("organization:", ATO_ORG)
  if (!is.null(q) && nzchar(q)) query <- paste0(query, "+", q)
  url <- sprintf("%s/package_search?q=%s&rows=%d",
                 CKAN_BASE, utils::URLencode(query, reserved = TRUE), rows)
  resp <- ato_request(url) |>
    httr2::req_perform()
  if (httr2::resp_status(resp) != 200L) {
    cli::cli_abort("CKAN search failed (HTTP {httr2::resp_status(resp)}).")
  }
  body <- httr2::resp_body_string(resp)
  parsed <- jsonlite::fromJSON(body, simplifyVector = FALSE)
  if (!isTRUE(parsed$success)) {
    cli::cli_abort("CKAN search returned success = FALSE.")
  }
  parsed$result
}

#' Fetch a single CKAN package by id (slug).
#' @noRd
ato_ckan_package <- function(id) {
  url <- sprintf("%s/package_show?id=%s",
                 CKAN_BASE, utils::URLencode(id, reserved = TRUE))
  resp <- ato_request(url) |>
    httr2::req_perform()
  if (httr2::resp_status(resp) != 200L) {
    cli::cli_abort("Package not found: {.val {id}}.")
  }
  body <- httr2::resp_body_string(resp)
  parsed <- jsonlite::fromJSON(body, simplifyVector = FALSE)
  if (!isTRUE(parsed$success)) {
    cli::cli_abort("CKAN package_show returned success = FALSE.")
  }
  parsed$result
}

#' Resolve the first resource in a package that matches a filename regex.
#' @noRd
ato_ckan_resolve <- function(package_id, pattern) {
  pkg <- ato_ckan_package(package_id)
  resources <- pkg$resources
  if (is.null(resources) || length(resources) == 0L) {
    cli::cli_abort("Package {.val {package_id}} has no resources.")
  }
  urls <- vapply(resources, function(r) r$url %||% "", character(1))
  names_v <- vapply(resources, function(r) r$name %||% r$url %||% "", character(1))
  hit <- grepl(pattern, urls, ignore.case = TRUE) |
    grepl(pattern, names_v, ignore.case = TRUE)
  if (!any(hit)) {
    cli::cli_abort(c(
      "No resource in {.val {package_id}} matches {.val {pattern}}.",
      "i" = "Available: {.val {names_v}}"
    ))
  }
  resources[[which(hit)[1L]]]
}

test_that("ato_ckan_search hits the network when live tests enabled", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  r <- ato:::ato_ckan_search(rows = 5L)
  expect_type(r, "list")
  expect_gt(length(r$results), 0L)
})

test_that("ato_catalog returns an ato_tbl with expected columns", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  cat <- ato_catalog()
  expect_s3_class(cat, "ato_tbl")
  expect_true(all(c("id", "title", "licence") %in% names(cat)))
  expect_gt(nrow(cat), 20L)
})

# Fuel Tax Credits: rates and claimant detail by industry.

#' Fuel Tax Credits by industry and claim period
#'
#' Returns the Fuel Tax Credits scheme data: entitlement rates
#' by fuel type, claim totals by industry. FTC is a major
#' implicit fossil-fuel subsidy and is a key lens for
#' decarbonisation policy cost-benefit analysis.
#'
#' The ATO publishes FTC data as part of the Excise Data release
#' and in standalone FTC tables.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#' @param by One of `"industry"` (default, by ANZSIC division),
#'   `"fuel"` (by fuel type), or `"period"` (quarterly rates).
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Excise and Fuel Tax Credit
#'   data. Licensed CC BY 3.0 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Fuel Tax Act 2006}; \emph{Fuel
#'   Tax (Consequential and Transitional Provisions) Act 2006}.
#'
#' Denniss, R. and Grudnoff, M. (2021). \emph{Fossil fuel
#'   subsidies in Australia}. The Australia Institute. FTC-as-
#'   subsidy framing used in decarbonisation policy analysis.
#'
#' Intergovernmental Panel on Climate Change (2022). \emph{Climate
#'   Change 2022: Mitigation of Climate Change}. Chapter 13
#'   covers fossil-fuel subsidy reform.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(head(ato_fuel_tax_credits(year = "latest", by = "industry")))
#' options(op)
#' }
ato_fuel_tax_credits <- function(year = "latest",
                                  by = c("industry", "fuel", "period")) {
  by <- match.arg(by)
  pkg <- ato_ckan_package(ATO_PACKAGE_IDS$excise)
  pattern <- switch(by,
    industry = "fuel.*(credit|ftc).*industry|ftc.*industry",
    fuel     = "fuel.*(credit|ftc).*fuel|ftc.*fuel",
    period   = "fuel.*(credit|ftc).*(rate|period)|ftc.*rate"
  )
  res <- ato_ckan_resolve(ATO_PACKAGE_IDS$excise, pattern)
  url <- res$url %||% ""
  df <- if (grepl("\\.csv$", url, ignore.case = TRUE)) {
    ato_fetch_csv(url)
  } else {
    ato_fetch_xlsx(url, sheet = 1)
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 3.0 AU",
              title = paste0("ATO fuel tax credits ", year, " (", by, ")"))
}

# Corporate Tax Transparency

#' Corporate Tax Transparency
#'
#' Returns the ATO's annual Corporate Tax Transparency release,
#' mandated by Part 5-25 of the *Taxation Administration Act
#' 1953*. Covers every Australian public company, foreign-owned
#' company, or Australian-owned private company above the
#' AUD 100 million total-income threshold (the private-company
#' threshold was lowered from AUD 200 million to AUD 100 million
#' for the 2022-23 income year onwards, making all three
#' categories uniform). The 2023-24 release was published 1
#' October 2025 and covered 4,110 entities.
#'
#' The underlying XLSX has three sheets:
#' - **Information** (cover/metadata, ~7 rows).
#' - **Income tax details** (the headline dataset, ~4,000
#'   entities: total income, taxable income, tax payable).
#' - **PRRT details** (petroleum resource rent tax filers,
#'   typically 10-20 entities).
#'
#' Licensed under **CC BY 3.0 Australia** (the Corporate Tax
#' Transparency and Voluntary Tax Transparency Code releases use
#' CC BY 3.0 AU; most other Taxation Statistics use CC BY 2.5 AU).
#'
#' @param year `"YYYY-YY"` (e.g. `"2023-24"`) or `"latest"`.
#' @param entity_type One of `"all"` (default), `"public"`,
#'   `"private"`, or `"foreign"`. Matches the CTT `Entity type`
#'   column values `"Australian public"`, `"Australian private"`,
#'   `"Foreign-owned"`.
#' @param sheet One of `"income_tax"` (default, the ~4,000-entity
#'   income-tax sheet) or `"prrt"` (petroleum resource rent tax
#'   filers, typically 10-20 entities).
#'
#' @return An `ato_tbl` with one row per disclosed entity. All
#'   monetary values are nominal AUD of the reporting year.
#'
#' @source Australian Taxation Office Corporate Tax Transparency
#'   release. Licensed CC BY 3.0 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Taxation Administration Act
#'   1953}, Part 5-25 (Corporate Tax Transparency).
#'
#' Australian Taxation Office (annual). \emph{Report of entity
#'   tax information}. The statutory Corporate Tax Transparency
#'   release.
#'
#' Commonwealth Treasury (2013). \emph{Improving the transparency
#'   of Australia's business tax system: Exposure draft
#'   explanatory memorandum}. Rationale for the Part 5-25 regime.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   top <- ato_top_taxpayers(year = "2023-24")
#'   head(top)
#'   # Petroleum resource rent tax sheet
#'   prrt <- ato_top_taxpayers(year = "2023-24", sheet = "prrt")
#'   head(prrt)
#' })
#' options(op)
#' }
ato_top_taxpayers <- function(year = "latest",
                               entity_type = c("all", "public", "private", "foreign"),
                               sheet = c("income_tax", "prrt")) {
  entity_type <- match.arg(entity_type)
  sheet <- match.arg(sheet)

  pkg_id <- ATO_PACKAGE_IDS$ctt
  if (identical(year, "latest")) {
    pkg <- ato_ckan_package(pkg_id)
    resources <- pkg$resources %||% list()
    urls <- vapply(resources, function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year <- if (length(years) > 0L) max(years) else "2023-24"
  } else {
    year <- ato_resolve_year(year)
  }
  ato_check_staleness(pkg_id)
  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  ext <- tolower(tools::file_ext(url))
  sheet_name <- switch(sheet,
    income_tax = "Income tax details",
    prrt       = "PRRT details"
  )
  sheet_fallback <- switch(sheet, income_tax = 2, prrt = 3)

  df <- if (ext == "csv") {
    ato_fetch_csv(url)
  } else {
    tryCatch(
      ato_fetch_xlsx(url, sheet = sheet_name),
      error = function(e) ato_fetch_xlsx(url, sheet = sheet_fallback)
    )
  }

  type_col <- ato_find_col(df, "entity")
  if (entity_type != "all" && !is.na(type_col)) {
    # CTT column values: "Australian public", "Australian private",
    # "Foreign-owned". Match the substantive token only.
    etype_pattern <- switch(entity_type,
      public  = "public",
      private = "private",
      foreign = "foreign"
    )
    df <- df[grepl(etype_pattern, tolower(df[[type_col]])), , drop = FALSE]
  }

  ato_warn_suppression(df, context = "CTT entity cells")

  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 3.0 AU",
              title   = paste0("ATO Corporate Tax Transparency ", year,
                               " (", sheet, ")"))
}

test_that("ato_top_taxpayers validates entity_type", {
  expect_error(ato_top_taxpayers(entity_type = "other"))
})

test_that("ato_top_taxpayers live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  t <- ato_top_taxpayers(year = "2023-24")
  expect_s3_class(t, "ato_tbl")
  expect_gt(nrow(t), 1000L)
})

# Working Holiday Maker (WHM) tax aggregates

#' Working Holiday Maker tax data
#'
#' Returns aggregate Working Holiday Maker tax data: number of
#' backpackers, total earnings, tax paid. Relevant for migration
#' and labour-market policy analysis.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Migration Act 1958}, visa
#'   subclasses 417 and 462; \emph{Working Holiday Maker
#'   Reform Act 2016}. Establishes the 15 per cent flat
#'   tax rate from the first dollar of WHM earnings.
#'
#' Productivity Commission (2016). \emph{Migrant Intake into
#'   Australia}. Includes WHM labour-market analysis.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_whm(year = "2022-23"))
#' options(op)
#' }
ato_whm <- function(year = "latest") {
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "working.holiday.maker|whm|backpacker")
  url <- res$url %||% ""
  df <- if (grepl("\\.csv$", url, ignore.case = TRUE)) {
    ato_fetch_csv(url)
  } else {
    ato_fetch_xlsx(url, sheet = 1)
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Working Holiday Maker ", year))
}

# Division 293: extra 15% tax on concessional super contributions
# for high-income earners (over AUD 250k combined income).

#' Division 293 tax assessments (high-income super contributions)
#'
#' Returns Division 293 tax data: number of assessments, average
#' Division 293 liability, and distribution by income band. Division
#' 293 applies an extra 15% tax on concessional super contributions
#' for individuals with combined income plus low-tax super
#' contributions above AUD 250,000. Central to retirement-income
#' reform analysis (e.g. Grattan's "Better Super" proposals).
#'
#' Published as part of the Individuals Taxation Statistics
#' (Table 3b in recent releases).
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics
#'   Individuals. Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Income Tax Assessment Act
#'   1997}, Division 293. Extra 15 per cent tax on concessional
#'   super contributions for high-income earners.
#'
#' Daley, J., Coates, B. and Wood, D. (2018). \emph{Money in
#'   retirement: more than enough}. Grattan Institute. Uses
#'   Division 293 distributional data in reform analysis.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_division293(year = "2022-23"))
#' options(op)
#' }
ato_division293 <- function(year = "latest") {
  id <- ato_ts_package_id(year)
  res <- tryCatch(
    ato_ckan_resolve(id, "division.?293|div.?293"),
    error = function(e) NULL
  )
  if (is.null(res)) {
    # Fallback: Table 3b "high-income super" pattern.
    res <- ato_ckan_resolve(id, "individual(s)?03|individual_03")
  }
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Division 293 ", year))
}

# Company Taxation Statistics

#' Company Taxation Statistics
#'
#' Returns the annual Company Taxation Statistics tables. The
#' Company release ships tables covering entity type, turnover
#' band, industry, taxable status, source of income, and
#' expense deductions. Pick the table that matches your
#' question:
#'
#' - **snapshot** (T1): aggregate counts, total income, net tax
#'   across all companies (~1m entities)
#' - **key_items_by_size** (T2): net tax by company size band
#' - **entity_type** (T3): split by public/private/co-operative
#' - **industry** (T4, default): key items by 2-digit ANZSIC
#'   subdivision
#' - **industry_by_size** (T5): industry x turnover band
#' - **sub_industry** (T6): 4-digit ANZSIC class detail
#' - **taxable_status** (T7): items by taxable status
#' - **source** (T8): source of income
#' - **expenses** (T9): expense and deduction categories
#'
#' **Classification break.** Releases from 2022-23 onwards use
#' ANZSIC 2020; earlier releases use ANZSIC 2006. A warning is
#' emitted when the requested year(s) are at or after this boundary,
#' or when a multi-year request spans it.
#'
#' @param year `"YYYY-YY"`, `"latest"`, or a vector of years for
#'   a multi-year panel. Multi-year requests add a `year` column.
#' @param table One of `"snapshot"`, `"key_items_by_size"`,
#'   `"entity_type"`, `"industry"` (default), `"industry_by_size"`,
#'   `"sub_industry"`, `"taxable_status"`, `"source"`, or
#'   `"expenses"`.
#' @param industry Optional substring filter on industry name
#'   (applied only when the fetched table has an industry column).
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD of the
#'   reporting year.
#'
#' @source Australian Taxation Office Taxation Statistics Company
#'   Tables. Licensed CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Taxation Statistics:
#'   Company tables explanatory notes}. Methodology notes on
#'   lodgement cut-off, entity-type definitions, and turnover-band
#'   thresholds. Accessible from
#'   \url{https://www.ato.gov.au/about-ato/research-and-statistics/in-detail/taxation-statistics/}.
#'
#' Australian Bureau of Statistics (2020). \emph{Australian and
#'   New Zealand Standard Industrial Classification (ANZSIC)},
#'   2006 revision with 2020 update. Catalogue 1292.0.
#'
#' @family companies
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   s <- ato_companies(year = "2022-23", table = "snapshot")
#'   head(s)
#'   m <- ato_companies(year = "2022-23", industry = "mining")
#'   head(m)
#'   # Multi-year industry panel
#'   panel <- ato_companies(year = c("2021-22", "2022-23"))
#' })
#' options(op)
#' }
ato_companies <- function(year = "latest",
                           table = c("industry",
                                     "snapshot",
                                     "key_items_by_size",
                                     "entity_type",
                                     "industry_by_size",
                                     "sub_industry",
                                     "taxable_status",
                                     "source",
                                     "expenses"),
                           industry = NULL) {
  table <- match.arg(table)

  # Multi-year path
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    if (table %in% c("industry", "industry_by_size", "sub_industry")) {
      ato_warn_classification_span(resolved, "anzsic")
    }
    fn <- function(year) {
      ato_companies(year = year, table = table, industry = industry)
    }
    return(ato_stack_years(fn, resolved,
                           title_prefix = paste0("ATO companies (", table, ")")))
  }

  id <- ato_ts_package_id(year)
  resolved_year <- sub("taxation-statistics-", "", id)

  if (table %in% c("industry", "industry_by_size", "sub_industry")) {
    ato_warn_classification_break(resolved_year, "anzsic")
  }

  pattern <- switch(table,
    snapshot          = "company01|company_01",
    key_items_by_size = "company02|company_02",
    entity_type       = "company03|company_03",
    industry          = "company04|company_04|company.*industry",
    industry_by_size  = "company05|company_05",
    sub_industry      = "company06|company_06",
    taxable_status    = "company07|company_07",
    source            = "company08|company_08",
    expenses          = "company09|company_09"
  )
  res <- ato_ckan_resolve(id, pattern)
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)

  ind_col <- ato_find_col(df, "industry")
  if (!is.null(industry) && !is.na(ind_col)) {
    ind_pattern <- paste(tolower(industry), collapse = "|")
    df <- df[grepl(ind_pattern, tolower(df[[ind_col]])), , drop = FALSE]
  }
  if (!is.null(industry) && nrow(df) == 0L) {
    cli::cli_warn("No industry rows matched {.val {industry}}.")
  }

  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO companies ", year, " (", table, ")"))
}

test_that("ato_companies live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  c <- ato_companies(year = "2022-23")
  expect_s3_class(c, "ato_tbl")
  expect_gt(nrow(c), 10L)
})

# Cache management

#' @noRd
ato_cache_dir <- function() {
  d <- getOption("ato.cache_dir",
                 default = tools::R_user_dir("ato", "cache"))
  if (!dir.exists(d)) dir.create(d, recursive = TRUE)
  d
}

#' Clear the ato cache
#'
#' Deletes all locally cached files. The next call to any data
#' function will re-download.
#'
#' @return Invisibly returns `NULL`.
#' @family configuration
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' ato_clear_cache()
#' options(op)
#' }
ato_clear_cache <- function() {
  d <- ato_cache_dir()
  files <- list.files(d, full.names = TRUE)
  n <- length(files)
  if (n > 0L) unlink(files, recursive = TRUE)
  cli::cli_inform("Removed {n} cached file{?s} from {.path {d}}.")
  invisible(NULL)
}

#' Check whether a CKAN package has been updated since the cache was last written
#'
#' Only runs when `options(ato.check_staleness = TRUE)`. Silently no-ops on
#' network error so it never blocks data retrieval.
#' @noRd
ato_check_staleness <- function(package_id) {
  if (!isTRUE(getOption("ato.check_staleness", FALSE))) return(invisible(NULL))
  tryCatch({
    pkg <- ato_ckan_package(package_id)
    modified <- pkg$metadata_modified %||% ""
    if (!nzchar(modified)) return(invisible(NULL))
    modified_time <- as.POSIXct(modified, format = "%Y-%m-%dT%H:%M:%S",
                                 tz = "UTC")
    d <- ato_cache_dir()
    files <- list.files(d, full.names = TRUE)
    if (length(files) == 0L) return(invisible(NULL))
    oldest_mtime <- min(file.info(files)$mtime)
    if (!is.na(modified_time) && modified_time > oldest_mtime) {
      cli::cli_warn(c(
        "ATO has updated {.val {package_id}} since your cache was last refreshed.",
        "i" = "Run {.code ato_clear_cache()} and re-fetch to get the latest data."
      ))
    }
  }, error = function(e) invisible(NULL))
  invisible(NULL)
}

#' Inspect the local ato cache
#'
#' @return A list with `dir`, `n_files`, `size_bytes`,
#'   `size_human`, and `files`.
#' @family configuration
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' ato_cache_info()
#' options(op)
#' }
ato_cache_info <- function() {
  d <- ato_cache_dir()
  empty <- data.frame(
    name = character(0L),
    size_bytes = numeric(0L),
    modified = as.POSIXct(character(0L)),
    stringsAsFactors = FALSE
  )
  paths <- list.files(d, full.names = TRUE)
  if (length(paths) == 0L) {
    return(list(dir = d, n_files = 0L, size_bytes = 0,
                size_human = "0 B", files = empty))
  }
  info <- file.info(paths)
  files <- data.frame(
    name = basename(paths),
    size_bytes = info$size,
    modified = info$mtime,
    stringsAsFactors = FALSE
  )
  files <- files[order(-files$size_bytes), , drop = FALSE]
  rownames(files) <- NULL
  total <- sum(files$size_bytes)
  list(dir = d, n_files = nrow(files), size_bytes = total,
       size_human = ato_format_bytes(total), files = files)
}

test_that("ato_cache_dir honours the ato.cache_dir option", {
  tmp <- tempfile("ato_cache_")
  op <- options(ato.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  d <- ato:::ato_cache_dir()
  expect_equal(normalizePath(d), normalizePath(tmp))
  expect_true(dir.exists(d))
})

test_that("ato_cache_info returns empty structure", {
  tmp <- tempfile("ato_cache_")
  op <- options(ato.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  info <- ato_cache_info()
  expect_type(info, "list")
  expect_setequal(names(info),
                  c("dir", "n_files", "size_bytes", "size_human", "files"))
  expect_equal(info$n_files, 0L)
})

test_that("ato_cache_info counts files", {
  tmp <- tempfile("ato_cache_")
  op <- options(ato.cache_dir = tmp)
  on.exit(options(op), add = TRUE)
  dir.create(tmp, recursive = TRUE)
  writeLines("a", file.path(tmp, "a.csv"))
  writeLines("ab", file.path(tmp, "b.csv"))
  info <- ato_cache_info()
  expect_equal(info$n_files, 2L)
})

test_that("ato_clear_cache removes files", {
  tmp <- tempfile("ato_cache_")
  op <- options(ato.cache_dir = tmp)
  on.exit(options(op), add = TRUE)
  dir.create(tmp, recursive = TRUE)
  writeLines("x", file.path(tmp, "x.csv"))
  expect_invisible(ato_clear_cache())
  expect_false(file.exists(file.path(tmp, "x.csv")))
})

test_that("ato_format_bytes formats thresholds", {
  expect_equal(ato:::ato_format_bytes(0), "0 B")
  expect_match(ato:::ato_format_bytes(1500), "KB$")
  expect_match(ato:::ato_format_bytes(1500000), "MB$")
})

# Study and Training Support Loans

#' Study and Training Support Loan data (HELP, AASL, VSL)
#'
#' Returns aggregate statistics on Australia's three main
#' education-loan schemes:
#' - **HELP** (Higher Education Loan Program, ~3m borrowers,
#'   AUD 80bn+ outstanding debt)
#' - **AASL** (Australian Apprenticeship Support Loans,
#'   previously Trade Support Loans)
#' - **VSL** (VET Student Loans, vocational education loans)
#'
#' Headline covers: new loans by income range, outstanding debt
#' by age and gender, repayment rates, median debt on entry.
#' Used by Treasury (PBO costings of HELP indexation changes)
#' and education policy researchers.
#'
#' @param scheme One of `"help"` (default), `"aasl"`, or `"vsl"`.
#'
#' @return An `ato_tbl`. All dollar values in nominal AUD.
#'
#' @source Australian Taxation Office Study and Training Support
#'   Loans statistics. Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Higher Education Support Act
#'   2003}; \emph{VET Student Loans Act 2016}.
#'
#' Australian Department of Education (annual). \emph{Higher
#'   Education Statistics: HELP statistics collection}.
#'
#' Norton, A. and Cherastidtham, I. (2018). \emph{Mapping
#'   Australian higher education}. Grattan Institute.
#'   Methodology reference for HELP repayment projections.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   help <- ato_help(scheme = "help")
#'   head(help)
#' })
#' options(op)
#' }
ato_help <- function(scheme = c("help", "aasl", "vsl")) {
  scheme <- match.arg(scheme)
  pattern <- switch(scheme,
    help = "help-statistics",
    aasl = "aasl-statistics",
    vsl  = "vsl-statistics"
  )
  res <- ato_ckan_resolve(ATO_PACKAGE_IDS$help, pattern)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO ", toupper(scheme), " statistics"))
}

# International benchmarking via OECD Revenue Statistics.

#' OECD Revenue Statistics comparison
#'
#' Fetches OECD Revenue Statistics for cross-country tax-to-GDP
#' benchmarking. Returns tax revenue as percent of GDP by tax
#' category. Use to contextualise Australian ATO aggregates in
#' cross-country policy arguments (e.g. OECD average corporate
#' tax-to-GDP, international ranks for personal income tax).
#'
#' Thin wrapper pointing users to `readoecd::` for full OECD API
#' access; returns a minimal tax-to-GDP slice here for convenience.
#'
#' @param country Country ISO code or name (default `"AUS"`).
#' @param year Four-digit year or `"latest"`.
#'
#' @return An `ato_tbl` with columns `country`, `year`, `tax`,
#'   `pct_gdp`.
#'
#' @source OECD Revenue Statistics
#'   <https://www.oecd.org/tax/tax-policy/revenue-statistics.htm>.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_international(country = "AUS"))
#' options(op)
#' }
ato_international <- function(country = "AUS", year = "latest") {
  cli::cli_inform(c(
    "i" = "For full OECD Revenue Statistics access, install {.pkg readoecd}.",
    "i" = "This wrapper returns bundled headline tax-to-GDP ratios only."
  ))
  # Bundled headline ratios for common years (OECD published values).
  headline <- data.frame(
    country = c("AUS", "AUS", "AUS", "OECD_AVG", "OECD_AVG"),
    year    = c("2021", "2022", "2023", "2022", "2023"),
    tax     = c("total", "total", "total", "total", "total"),
    pct_gdp = c(29.5, 29.4, 29.4, 33.9, 34.0),
    stringsAsFactors = FALSE
  )
  out <- headline[headline$country == country, , drop = FALSE]
  if (year != "latest") {
    out <- out[out$year == year, , drop = FALSE]
  }
  if (nrow(out) == 0L) {
    cli::cli_warn("No OECD data bundled for {.val {country}} {.val {year}}.")
  }
  new_ato_tbl(out,
              source = "https://www.oecd.org/tax/tax-policy/revenue-statistics.htm",
              licence = "OECD terms",
              title = paste0("OECD tax-to-GDP ", country, " ", year))
}

# Petroleum Resource Rent Tax (PRRT).

#' Petroleum Resource Rent Tax (PRRT) annual data
#'
#' Returns PRRT revenue and assessments. PRRT is a 40% tax on the
#' profits of offshore petroleum projects; revenues are volatile
#' and project-specific. Key dataset for resource-tax reform
#' analysis.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Taxation Office Taxation Statistics Company.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Petroleum Resource Rent Tax
#'   Assessment Act 1987}. Enabling legislation for the 40 per
#'   cent rent tax on offshore petroleum projects.
#'
#' Callaghan, M. (2017). \emph{Review of the Petroleum Resource
#'   Rent Tax}. Treasury-commissioned review; reference for
#'   PRRT-reform analysis.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_prrt(year = "2022-23"))
#' options(op)
#' }
ato_prrt <- function(year = "latest") {
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "prrt|petroleum.resource.rent")
  url <- res$url %||% ""
  df <- if (grepl("\\.csv$", url, ignore.case = TRUE)) {
    ato_fetch_csv(url)
  } else {
    ato_fetch_xlsx(url, sheet = 1)
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO PRRT ", year))
}

# Microdata bridge: remap ATO aggregate column names to the
# `taxstats` / `taxstats2` DRAT package schema, so analyses can
# move between aggregate and 2%-sample microdata views with
# consistent variable definitions.

#' Schema map: ATO aggregate columns -> taxstats microdata columns
#' @noRd
ATO_TAXSTATS_SCHEMA_MAP <- list(
  # Aggregate -> microdata canonical name
  total_income_or_loss      = "Tot_inc_amt",
  total_income              = "Tot_inc_amt",
  taxable_income            = "Taxable_Income",
  net_tax                   = "Tax_free_threshold_amt",
  tax_payable               = "Tax_assessed_amt",
  medicare_levy             = "Medicare_levy",
  salary_wages              = "Sw_amt",
  allowances                = "All_amt",
  deductions_total          = "Total_deductions_amt",
  work_related_car          = "WRE_car_amt",
  work_related_travel       = "WRE_travel_amt",
  work_related_clothing     = "WRE_uniform_amt",
  donations                 = "Gifts_amt",
  rental_income             = "Rent_inc_amt",
  rental_interest           = "Rent_int_ded_amt",
  franking_credits          = "Franking_credits_amt",
  interest_income           = "Total_IT_amt",
  dividends                 = "Div_unfranked_amt",
  cgt_net                   = "Net_CG_amt",
  hecs_repayment            = "HELP_repay_amt",
  postcode                  = "Postcode",
  age_range                 = "age_range",
  sex                       = "Gender",
  state                     = "State"
)

#' Remap an ato_tbl to the taxstats microdata column schema
#'
#' Takes an `ato_tbl` with aggregate column names (produced by any
#' `ato_*` function) and renames columns to match the `taxstats`
#' (or `taxstats2`) 2% microdata sample schema used by Hugh
#' Parsonage's DRAT package. Enables consistent variable
#' definitions when moving between aggregate views and microdata
#' prototyping.
#'
#' @details
#' The bundled schema map (\code{ato_schema_map()}) mirrors the
#' column names from Parsonage's `taxstats` and `taxstats2`
#' packages, which in turn use the ATO Individual Sample File
#' variable names. Because `taxstats` is DRAT-distributed and not
#' on CRAN, this function imposes the mapping as a static table
#' rather than programmatically introspecting the `taxstats`
#' namespace. Re-check the bundled map against the `taxstats`
#' NAMESPACE when the ATO publishes a revised Sample File schema.
#'
#' Unknown columns pass through unchanged. Use
#' \code{\link{ato_harmonise}} first if the input panel has drift
#' in source column names.
#'
#' @param df An `ato_tbl` or data frame.
#' @param direction `"to_taxstats"` (default, aggregate -> microdata)
#'   or `"from_taxstats"` (microdata -> aggregate).
#'
#' @return A data frame with renamed columns. `ato_tbl` class and
#'   provenance attributes preserved.
#'
#' @references
#' Parsonage, H. (2019). \emph{taxstats: 2 per cent Individual
#'   Sample File from the Australian Taxation Office}. R package
#'   (DRAT).
#'   \url{https://github.com/HughParsonage/taxstats}
#'
#' Parsonage, H. (2024). \emph{grattan: Perform Common Quantitative
#'   Tasks for Australian Analysts}. R package version 2026.1.1.
#'   \url{https://cran.r-project.org/package=grattan}
#'
#' Australian Taxation Office (2024). \emph{Taxation Statistics:
#'   Individual Sample File documentation}.
#'
#' @family harmonisation
#' @export
#' @examples
#' df <- data.frame(postcode = "2000", taxable_income = 80000,
#'                  medicare_levy = 1600)
#' ato_to_taxstats(df)
ato_to_taxstats <- function(df,
                             direction = c("to_taxstats",
                                           "from_taxstats")) {
  direction <- match.arg(direction)
  map <- ATO_TAXSTATS_SCHEMA_MAP
  if (direction == "from_taxstats") {
    # Invert; for many-to-one mappings, the first aggregate name wins.
    inv <- list()
    for (k in names(map)) {
      v <- map[[k]]
      if (is.null(inv[[v]])) inv[[v]] <- k
    }
    map <- inv
  }
  nm <- names(df)
  for (i in seq_along(nm)) {
    if (!is.null(map[[nm[i]]])) nm[i] <- map[[nm[i]]]
  }
  names(df) <- nm
  df
}

#' Print the ATO -> taxstats schema map
#'
#' Convenience accessor for the bundled column-name mapping.
#'
#' @return A data frame with columns `ato_aggregate` and
#'   `taxstats_microdata`.
#' @family harmonisation
#' @export
#' @examples
#' head(ato_schema_map())
ato_schema_map <- function() {
  data.frame(
    ato_aggregate       = names(ATO_TAXSTATS_SCHEMA_MAP),
    taxstats_microdata  = unlist(ATO_TAXSTATS_SCHEMA_MAP, use.names = FALSE),
    stringsAsFactors = FALSE
  )
}

test_that("ato_to_taxstats renames known columns", {
  df <- data.frame(taxable_income = 80000,
                   medicare_levy = 1600,
                   postcode = "2000")
  out <- ato_to_taxstats(df)
  expect_true("Taxable_Income" %in% names(out))
  expect_true("Medicare_levy" %in% names(out))
  expect_true("Postcode" %in% names(out))
})

test_that("ato_to_taxstats leaves unknown columns alone", {
  df <- data.frame(foo = 1, bar = 2)
  out <- ato_to_taxstats(df)
  expect_equal(names(out), c("foo", "bar"))
})

test_that("ato_to_taxstats direction from_taxstats inverts", {
  df <- data.frame(Taxable_Income = 80000, Postcode = "2000")
  out <- ato_to_taxstats(df, direction = "from_taxstats")
  expect_true("taxable_income" %in% names(out))
  expect_true("postcode" %in% names(out))
})

test_that("ato_schema_map returns a data frame with 2 columns", {
  map <- ato_schema_map()
  expect_s3_class(map, "data.frame")
  expect_equal(ncol(map), 2L)
  expect_gt(nrow(map), 10L)
})

# Level 3 invariant: bijective columns roundtrip cleanly.
# Only test columns where the aggregate -> microdata -> aggregate
# map recovers the original name (multi-to-one maps won't invert).
test_that("ato_to_taxstats roundtrip preserves one-to-one columns", {
  bijective <- c("taxable_income", "postcode", "rental_income",
                 "medicare_levy", "salary_wages", "sex")
  df <- as.data.frame(
    stats::setNames(
      replicate(length(bijective), 1, simplify = FALSE),
      bijective
    ),
    stringsAsFactors = FALSE
  )
  forward <- ato_to_taxstats(df, direction = "to_taxstats")
  back    <- ato_to_taxstats(forward, direction = "from_taxstats")
  expect_true(all(bijective %in% names(back)))
})

# PAYG Withholding

#' PAYG withholding data
#'
#' Returns the ATO's Pay As You Go (PAYG) withholding data:
#' employer counts, total withholding amounts, and employee counts
#' by industry and state. Used by researchers studying labour
#' market taxation, wage growth, and employer compliance.
#'
#' @param year Income year in `"YYYY-YY"` form (e.g. `"2022-23"`)
#'   or `"latest"`.
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD.
#'
#' @source Australian Taxation Office PAYG withholding data
#'   on data.gov.au. Licensed CC BY 2.5 AU.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   payg <- ato_payg(year = "2022-23")
#'   head(payg)
#' })
#' options(op)
#' }
ato_payg <- function(year = "latest") {
  result   <- ato_ckan_search(q = "pay-as-you-go", rows = 10L)
  pkgs     <- result$results
  ids      <- vapply(pkgs, function(p) p$name %||% "", character(1))
  payg_ids <- ids[grepl("pay.as.you.go|payg.withholding", ids,
                          ignore.case = TRUE)]

  if (length(payg_ids) == 0L) {
    cli::cli_abort(c(
      "Could not find a PAYG withholding package on data.gov.au.",
      "i" = "Browse: {.url https://data.gov.au/data/organization/australiantaxationoffice}"
    ))
  }
  pkg_id <- payg_ids[1L]
  ato_check_staleness(pkg_id)

  if (identical(year, "latest")) {
    pkg  <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year  <- if (length(years) > 0L) max(years) else "2022-23"
  } else {
    year <- ato_resolve_year(year)
  }

  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO PAYG withholding ", year))
}

# Complementary non-ATO sibling helpers.
# These wrap fetches from ABS, OECD, and RBA so researchers have
# the full tax-system picture without juggling four packages.

#' State and territory tax revenue (ABS 5506.0)
#'
#' Fetches the ABS Taxation Revenue collection (cat. 5506.0),
#' which gives land tax, payroll tax, stamp duty, motor vehicle
#' taxes, and other state taxes by jurisdiction. Needed for
#' complete-tax-system analysis alongside ATO Commonwealth data.
#'
#' @param year `"YYYY-YY"` or `"latest"`.
#'
#' @return An `ato_tbl`.
#'
#' @source Australian Bureau of Statistics, Taxation Revenue,
#'   catalogue 5506.0 <https://www.abs.gov.au/statistics/economy/government/taxation-revenue-australia>.
#'   Licensed CC BY 4.0.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_state_tax(year = "latest"))
#' options(op)
#' }
ato_state_tax <- function(year = "latest") {
  url <- "https://www.abs.gov.au/statistics/economy/government/taxation-revenue-australia/latest-release"
  cli::cli_warn(c(
    "ABS 5506.0 is HTML-paginated; no direct CSV endpoint.",
    "i" = "Fetch the XLSX release from {.url {url}} manually,",
    "i" = "then pass the local path to {.code ato_download()} or {.code ato_fetch_xlsx()}."
  ))
  # Return an empty stub ato_tbl so downstream code can dispatch.
  df <- data.frame(
    jurisdiction = character(0),
    tax_type = character(0),
    value_aud_million = numeric(0),
    year = character(0),
    stringsAsFactors = FALSE
  )
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 4.0",
              title = paste0("ABS 5506.0 State tax ", year))
}

# Internal helpers

#' @noRd
`%||%` <- function(x, y) if (is.null(x)) y else x

#' @noRd
ato_format_bytes <- function(x) {
  if (is.na(x) || x < 1024) return(paste0(x, " B"))
  units <- c("KB", "MB", "GB", "TB")
  for (i in seq_along(units)) {
    x <- x / 1024
    if (x < 1024 || i == length(units)) {
      return(sprintf("%.1f %s", x, units[i]))
    }
  }
}

#' Clean column names to snake_case
#' @noRd
ato_clean_names <- function(x) {
  x <- tolower(x)
  x <- gsub("[^a-z0-9]+", "_", x)
  x <- gsub("^_+|_+$", "", x)
  x <- gsub("_+", "_", x)
  x
}

#' ATO confidentiality suppression tokens
#'
#' The ATO replaces values with one of these tokens when fewer
#' than ten taxpayers (or fewer than 50 returns for postcode
#' data) fall in a cell. Passing them through `na.strings`
#' ensures numeric columns stay numeric. Seen across the
#' Taxation Statistics, Corporate Tax Transparency, Tax Gaps,
#' and Small Business Benchmarks releases.
#' @noRd
ATO_SUPPRESSION_TOKENS <- c(
  "",
  "NA", "N/A", "N.A.", "n/a", "n.a.",
  "np", "n.p.", "N.P.", "Np",
  "-", "--", ".", "..", "...",
  "*", "**",
  "\u2021", "\u2020"  # double dagger, dagger
)

#' Fetch a CSV and return a tidy data frame
#'
#' ATO CSVs use `"np"` (not published), `"*"`, `"‡"`, and other
#' tokens to suppress cells with fewer than ten taxpayers. These
#' are coerced to `NA` so numeric columns stay numeric.
#' @noRd
ato_fetch_csv <- function(url, ...) {
  file <- ato_download_cached(url)
  df <- utils::read.csv(file, stringsAsFactors = FALSE,
                        check.names = FALSE,
                        na.strings = ATO_SUPPRESSION_TOKENS,
                        ...)
  names(df) <- ato_clean_names(names(df))
  df
}

#' Fetch an XLSX and return a data frame
#'
#' ATO XLSX releases use the same confidentiality-suppression
#' tokens as CSV releases; `readxl::read_excel` is passed `na =
#' ATO_SUPPRESSION_TOKENS` so numeric columns are not silently
#' coerced to character when an `np` cell appears.
#'
#' When `skip` is `NULL` (the default), the helper auto-detects
#' the header row by reading up to 15 rows without headers and
#' finding the first row that looks like a header (three or more
#' non-empty string cells). This handles ATO workbooks that lead
#' with a title or narrative block before the data table.
#' @noRd
ato_fetch_xlsx <- function(url, sheet = 1, skip = NULL) {
  if (!requireNamespace("readxl", quietly = TRUE)) {
    cli::cli_abort(c(
      "The {.pkg readxl} package is required to parse XLSX files."
    ))
  }
  file <- ato_download_cached(url)
  if (is.null(skip)) {
    skip <- ato_detect_header_row(file, sheet = sheet)
  }
  df <- as.data.frame(
    readxl::read_excel(file, sheet = sheet, skip = skip,
                       na = ATO_SUPPRESSION_TOKENS,
                       .name_repair = "minimal"),
    stringsAsFactors = FALSE
  )
  names(df) <- ato_clean_names(names(df))
  df
}

#' Auto-detect the first row of an XLSX sheet that looks like a
#' column-header row
#'
#' Heuristic: reads up to 15 rows without treating any row as a
#' header, then finds the first row where at least three cells
#' are non-empty strings that do not contain newlines (newlines
#' typically appear in narrative-block rows, not headers).
#' Returns the number of rows to skip before the header.
#' @noRd
ato_detect_header_row <- function(file, sheet = 1, max_scan = 15L) {
  probe <- tryCatch(
    readxl::read_excel(file, sheet = sheet, col_names = FALSE,
                       n_max = max_scan, .name_repair = "minimal"),
    error = function(e) NULL
  )
  if (is.null(probe) || nrow(probe) == 0L) return(0L)

  is_string_row <- function(row) {
    vals <- unlist(row, use.names = FALSE)
    vals <- vals[!is.na(vals) & nzchar(as.character(vals))]
    if (length(vals) < 3L) return(FALSE)
    chars <- suppressWarnings(as.character(vals))
    # Header rows frequently contain newlines (e.g. "Individuals\nno.")
    # so do not reject on that. Reject rows that are all-numeric.
    looks_numeric <- suppressWarnings(!any(is.na(as.numeric(chars))))
    !looks_numeric
  }

  for (i in seq_len(nrow(probe))) {
    if (is_string_row(probe[i, , drop = FALSE])) {
      return(i - 1L)
    }
  }
  0L
}

#' Find the first matching column for a logical role
#'
#' Uses ATO_COL_VARIANTS to handle cross-year column renames. Returns
#' NA_character_ and emits a warning if no variant is found.
#' @noRd
ato_find_col <- function(df, key) {
  variants <- ATO_COL_VARIANTS[[key]]
  if (is.null(variants)) return(NA_character_)
  hit <- intersect(variants, names(df))
  if (length(hit) == 0L) {
    cli::cli_warn(
      "Could not find {.val {key}} column. Tried: {.val {variants}}."
    )
    return(NA_character_)
  }
  hit[1L]
}

#' Warn if a resolved year is at or after the classification break
#'
#' @param resolved_year Canonical "YYYY-YY" string.
#' @param type "anzsco" (occupation) or "anzsic" (industry).
#' @noRd
ato_warn_classification_break <- function(resolved_year,
                                           type = c("anzsco", "anzsic")) {
  type <- match.arg(type)
  yr <- suppressWarnings(as.integer(sub("-.*", "", resolved_year)))
  if (is.na(yr) || yr < ATO_CLASSIFICATION_BREAK_YEAR) return(invisible(NULL))
  if (type == "anzsco") {
    cli::cli_warn(c(
      "{resolved_year} uses ANZSCO 2021 occupation codes.",
      "i" = "Releases before 2022-23 use ANZSCO 2013.",
      "i" = "Cross-year occupation joins need a concordance recode."
    ))
  } else {
    cli::cli_warn(c(
      "{resolved_year} uses ANZSIC 2020 industry codes.",
      "i" = "Releases before 2022-23 use ANZSIC 2006.",
      "i" = "Cross-year industry joins need a concordance recode."
    ))
  }
  invisible(NULL)
}

#' Warn if a set of resolved years spans the classification break
#' @noRd
ato_warn_classification_span <- function(resolved_years,
                                          type = c("anzsco", "anzsic")) {
  type <- match.arg(type)
  yr_nums <- suppressWarnings(as.integer(sub("-.*", "", resolved_years)))
  spans <- any(yr_nums >= ATO_CLASSIFICATION_BREAK_YEAR, na.rm = TRUE) &&
           any(yr_nums < ATO_CLASSIFICATION_BREAK_YEAR, na.rm = TRUE)
  if (!spans) return(invisible(NULL))
  label <- if (type == "anzsco") "ANZSCO 2013 \u2192 2021 (occupation)" else
                                  "ANZSIC 2006 \u2192 2020 (industry)"
  cli::cli_warn(c(
    "Requested years span the {label} reclassification in 2022-23.",
    "i" = "Cross-year comparisons across this boundary need a concordance recode."
  ))
  invisible(NULL)
}

#' Warn if a high proportion of numeric cells are suppressed
#'
#' Checks the first numeric column in df. Fires when suppression
#' rate >= threshold (default 5%).
#' @noRd
ato_warn_suppression <- function(df, threshold = 0.05,
                                  context = "cells") {
  num_cols <- which(vapply(df, is.numeric, logical(1)))
  if (length(num_cols) == 0L || nrow(df) == 0L) return(invisible(NULL))
  col_name <- names(df)[num_cols[1L]]
  n_na <- sum(is.na(df[[col_name]]))
  pct <- n_na / nrow(df)
  if (pct >= threshold) {
    cli::cli_warn(c(
      "{round(100 * pct)}% of {context} are suppressed (np/NA) \\
       in column {.col {col_name}}.",
      "i" = "The ATO suppresses small cells (<50 returns for postcodes).",
      "i" = "Aggregates will understate totals. Use {.code na.rm = TRUE} when summing."
    ))
  }
  invisible(NULL)
}

#' Stack multi-year calls to a single-year data function
#'
#' fn must accept a year= argument; ... are NOT forwarded (use closures).
#' Returns a new ato_tbl with a year column added.
#' @noRd
ato_stack_years <- function(fn, years, title_prefix = "ATO data") {
  resolved <- vapply(as.list(years), ato_resolve_year, character(1))
  first_src <- NULL
  parts <- lapply(resolved, function(y) {
    df <- fn(year = y)
    if (is.null(first_src)) first_src <<- attr(df, "ato_source")
    df <- as.data.frame(df, stringsAsFactors = FALSE)
    df$year <- y
    df
  })
  common <- Reduce(intersect, lapply(parts, names))
  common <- common[nzchar(common)]
  if (length(common) == 0L) {
    cli::cli_abort(c(
      "No column names are common across the requested years.",
      "i" = "Try fewer years, or inspect each year individually first."
    ))
  }
  stacked <- do.call(
    rbind,
    lapply(parts, function(d) d[, common, drop = FALSE])
  )
  rownames(stacked) <- NULL
  new_ato_tbl(stacked,
              source  = first_src %||% "",
              licence = "CC BY 2.5 AU",
              title   = paste0(title_prefix, " (",
                               min(resolved), " to ", max(resolved), ")"))
}

#' Convert a user-supplied year string to the ATO Taxation Statistics slug
#'
#' Users can pass year in a few forms:
#' - "2022-23"  (canonical)
#' - "2022/23"
#' - "2022-2023"
#' - 2022 (numeric : interpreted as starting year)
#' - "latest" (resolved against the catalogue)
#'
#' Returns the canonical "YYYY-YY" slug used in CKAN package names.
#' @noRd
ato_resolve_year <- function(year) {
  if (is.numeric(year)) {
    start <- as.integer(year)
    return(sprintf("%d-%02d", start, (start + 1L) %% 100L))
  }
  if (identical(year, "latest")) return(year)
  year <- gsub("[/.]", "-", year)
  m <- regmatches(year, regexec("^([0-9]{4})-?([0-9]{2,4})$", year))[[1]]
  if (length(m) == 3L) {
    start <- as.integer(m[2])
    end <- as.integer(m[3]) %% 100L
    return(sprintf("%d-%02d", start, end))
  }
  cli::cli_abort("Could not parse year {.val {year}}. Use format like '2022-23' or 2022.")
}

# Classification-break warning tests (all offline)

test_that("ato_warn_classification_break fires for year >= 2022", {
  expect_warning(
    ato:::ato_warn_classification_break("2022-23", "anzsco"),
    "ANZSCO 2021"
  )
  expect_warning(
    ato:::ato_warn_classification_break("2023-24", "anzsco"),
    "ANZSCO 2021"
  )
  expect_warning(
    ato:::ato_warn_classification_break("2022-23", "anzsic"),
    "ANZSIC 2020"
  )
})

test_that("ato_warn_classification_break is silent for year < 2022", {
  expect_silent(ato:::ato_warn_classification_break("2021-22", "anzsco"))
  expect_silent(ato:::ato_warn_classification_break("2019-20", "anzsic"))
})

test_that("ato_warn_classification_span fires when years span the break", {
  expect_warning(
    ato:::ato_warn_classification_span(c("2021-22", "2022-23"), "anzsco"),
    "reclassification"
  )
  expect_warning(
    ato:::ato_warn_classification_span(c("2019-20", "2022-23"), "anzsic"),
    "reclassification"
  )
})

test_that("ato_warn_classification_span is silent when years don't span", {
  expect_silent(
    ato:::ato_warn_classification_span(c("2021-22", "2020-21"), "anzsco")
  )
  expect_silent(
    ato:::ato_warn_classification_span(c("2022-23", "2023-24"), "anzsic")
  )
})

test_that("ATO_CLASSIFICATION_BREAK_YEAR is 2022L", {
  expect_equal(ato:::ATO_CLASSIFICATION_BREAK_YEAR, 2022L)
})

test_that("ato_clean_names handles punctuation", {
  got <- ato:::ato_clean_names(c("Postcode", "Total Income $",
                                  "Taxable Income (Median)",
                                  "  trailing "))
  expect_equal(got, c("postcode", "total_income",
                      "taxable_income_median", "trailing"))
})

test_that("ato_resolve_year parses canonical forms", {
  expect_equal(ato:::ato_resolve_year("2022-23"), "2022-23")
  expect_equal(ato:::ato_resolve_year("2022/23"), "2022-23")
  expect_equal(ato:::ato_resolve_year("2022-2023"), "2022-23")
  expect_equal(ato:::ato_resolve_year(2022), "2022-23")
  expect_equal(ato:::ato_resolve_year("latest"), "latest")
})

test_that("ato_resolve_year errors on nonsense", {
  expect_error(ato:::ato_resolve_year("not-a-year"))
  expect_error(ato:::ato_resolve_year("20"))
})

# Filter edge cases and ato_find_col tests (all offline)

test_that("ato_find_col returns first matching variant", {
  df <- data.frame(state = "NSW", x = 1)
  expect_equal(ato:::ato_find_col(df, "state"), "state")

  df2 <- data.frame(state_territory = "NSW", x = 1)
  expect_equal(ato:::ato_find_col(df2, "state"), "state_territory")
})

test_that("ato_find_col returns NA and warns when no variant found", {
  df <- data.frame(region = "NSW", x = 1)
  expect_warning(
    result <- ato:::ato_find_col(df, "state"),
    "state"
  )
  expect_true(is.na(result))
})

test_that("ato_find_col returns NA for unknown key", {
  df <- data.frame(a = 1)
  expect_true(is.na(ato:::ato_find_col(df, "nonexistent_key")))
})

test_that("ato_warn_suppression fires when suppression >= 5%", {
  df <- data.frame(income = c(1000, NA, NA, NA, NA, NA))
  expect_warning(
    ato:::ato_warn_suppression(df, context = "test cells"),
    "suppressed"
  )
})

test_that("ato_warn_suppression is silent when suppression < 5%", {
  df <- data.frame(income = c(rep(1000, 99), NA))
  expect_silent(ato:::ato_warn_suppression(df, context = "test cells"))
})

test_that("ato_warn_suppression is silent on empty data frame", {
  df <- data.frame(income = numeric(0))
  expect_silent(ato:::ato_warn_suppression(df, context = "test cells"))
})

test_that("ato_warn_suppression is silent with no numeric columns", {
  df <- data.frame(name = c("a", NA, NA, NA, NA, NA))
  expect_silent(ato:::ato_warn_suppression(df, context = "test cells"))
})

test_that("ATO_COL_VARIANTS has expected keys", {
  keys <- names(ato:::ATO_COL_VARIANTS)
  expect_true("state" %in% keys)
  expect_true("postcode" %in% keys)
  expect_true("sex" %in% keys)
  expect_true("industry" %in% keys)
  expect_true("entity" %in% keys)
  expect_true("occupation" %in% keys)
})

test_that("ATO_PACKAGE_IDS has expected keys", {
  keys <- names(ato:::ATO_PACKAGE_IDS)
  expect_true("tax_gaps" %in% keys)
  expect_true("rdti" %in% keys)
  expect_true("ctt" %in% keys)
  expect_true("sme" %in% keys)
  expect_true("excise" %in% keys)
  expect_true("help" %in% keys)
  expect_true("smsf" %in% keys)
  expect_true("vttc" %in% keys)
})

test_that("ato_stack_years produces year column and stacks rows", {
  # Two functions returning compatible schemas (income col + year added by stack).
  fn_a <- function(year) {
    df <- data.frame(income = c(100, 200))
    ato:::new_ato_tbl(df, source = "https://example.org", title = "t1")
  }
  stacked <- ato:::ato_stack_years(fn_a, c("2021-22", "2022-23"))
  expect_s3_class(stacked, "ato_tbl")
  expect_true("year" %in% names(stacked))
  expect_equal(nrow(stacked), 4L)
  expect_equal(length(unique(stacked$year)), 2L)
})

test_that("live: ato_individuals_occupation multi-year stacks", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  expect_warning(
    panel <- ato_individuals_occupation(
      year       = c("2021-22", "2022-23"),
      occupation = "accountant"
    ),
    "reclassification"
  )
  expect_s3_class(panel, "ato_tbl")
  expect_true("year" %in% names(panel))
  expect_equal(length(unique(panel$year)), 2L)
})

test_that("live: ato_companies multi-year stacks", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  expect_warning(
    panel <- ato_companies(year = c("2021-22", "2022-23")),
    "reclassification"
  )
  expect_s3_class(panel, "ato_tbl")
  expect_true("year" %in% names(panel))
})

test_that("ATO_SUPPRESSION_TOKENS covers the main ATO suppression markers", {
  expect_true("np" %in% ato:::ATO_SUPPRESSION_TOKENS)
  expect_true("n.p." %in% ato:::ATO_SUPPRESSION_TOKENS)
  expect_true("*" %in% ato:::ATO_SUPPRESSION_TOKENS)
  expect_true("-" %in% ato:::ATO_SUPPRESSION_TOKENS)
  expect_true("" %in% ato:::ATO_SUPPRESSION_TOKENS)
})

test_that("ato_fetch_csv coerces np to NA, keeping numeric columns numeric", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)
  writeLines(c("Postcode,Tax Payable",
               "2000,1234567",
               "2001,np",
               "2002,890123"), tmp)

  # Fake the cache to point at our tempfile so ato_fetch_csv
  # returns without hitting the network.
  url <- "https://example.com/test.csv"
  cache_dir <- tempfile("ato_cache_")
  dir.create(cache_dir, recursive = TRUE)
  op <- options(ato.cache_dir = cache_dir)
  on.exit(options(op), add = TRUE)
  hash <- ato:::ato_digest_url(url)
  file.copy(tmp, file.path(cache_dir, paste0(hash, ".csv")))

  df <- ato:::ato_fetch_csv(url)
  expect_true(is.numeric(df$tax_payable))
  expect_equal(sum(!is.na(df$tax_payable)), 2L)
  expect_equal(sum(df$tax_payable, na.rm = TRUE), 2124690)
})

# ato_tbl S3 class with provenance header

#' @noRd
new_ato_tbl <- function(df, source = NULL, licence = "CC BY 2.5 AU",
                        retrieved = Sys.time(), title = NULL,
                        sha256 = NA_character_,
                        snapshot_date = ato_snapshot_str()) {
  stopifnot(is.data.frame(df))
  # If source is a URL already in cache, lift the sha256 sidecar for free.
  if (is.na(sha256) && !is.null(source) && nzchar(source)) {
    d <- tryCatch(ato_cache_dir(), error = function(e) NULL)
    if (!is.null(d)) {
      ext <- tools::file_ext(source)
      ext <- if (nzchar(ext)) paste0(".", ext) else ""
      f <- file.path(d, paste0(ato_digest_url(source), ext))
      if (file.exists(f)) sha256 <- ato_sha_read(f)
    }
  }
  attr(df, "ato_source") <- source
  attr(df, "ato_licence") <- licence
  attr(df, "ato_retrieved") <- retrieved
  attr(df, "ato_title") <- title
  attr(df, "ato_sha256") <- sha256
  attr(df, "ato_snapshot_date") <- snapshot_date
  class(df) <- c("ato_tbl", class(df))
  df
}

#' Print an ato_tbl
#'
#' Prints a provenance header (title, source, licence, retrieval
#' time, dimensions) followed by the data frame.
#'
#' @param x An `ato_tbl` object.
#' @param ... Passed to the next print method.
#' @return Invisibly returns `x`.
#' @export
#' @examples
#' x <- data.frame(postcode = "2000", taxable_income = 82000)
#' x <- structure(x, ato_title = "Demo", ato_source = "https://data.gov.au",
#'                ato_licence = "CC BY 2.5 AU", ato_retrieved = Sys.time(),
#'                class = c("ato_tbl", "data.frame"))
#' print(x)
print.ato_tbl <- function(x, ...) {
  title <- attr(x, "ato_title") %||% "ATO data"
  source <- attr(x, "ato_source") %||% "https://data.gov.au"
  licence <- attr(x, "ato_licence") %||% "CC BY 2.5 AU"
  retrieved <- attr(x, "ato_retrieved")
  retrieved_str <- if (!is.null(retrieved)) {
    format(retrieved, "%Y-%m-%d %H:%M %Z")
  } else {
    "-"
  }

  sha <- attr(x, "ato_sha256")
  snap <- attr(x, "ato_snapshot_date")
  cat("# ato_tbl: ", title, "\n", sep = "")
  cat("# Source:   ", source, "\n", sep = "")
  cat("# Licence:  ", licence, "\n", sep = "")
  cat("# Retrieved:", retrieved_str, "\n")
  if (!is.null(snap) && !is.na(snap)) {
    cat("# Snapshot: ", snap, "\n", sep = "")
  }
  if (!is.null(sha) && !is.na(sha) && nzchar(sha)) {
    cat("# SHA-256:  ", substr(sha, 1, 16), "...\n", sep = "")
  }
  cat("# Rows: ", formatC(nrow(x), big.mark = ",", format = "d"),
      "  Cols: ", ncol(x), "\n", sep = "")
  cat("\n")

  y <- x
  class(y) <- setdiff(class(y), "ato_tbl")
  print(y, ...)
  invisible(x)
}

# ato_meta tests

test_that("ato_meta rejects numeric input", {
  expect_error(ato_meta(42))
})

test_that("ato_meta rejects list input", {
  expect_error(ato_meta(list()))
})

test_that("ato_meta rejects ato_tbl with unparseable source URL", {
  df <- data.frame(a = 1)
  x  <- ato:::new_ato_tbl(df, source = "not-a-real-url", title = "t")
  expect_error(ato_meta(x), "package ID")
})

test_that("live: ato_meta returns expected structure from package ID", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  m <- ato_meta("australian-tax-gaps")
  expect_type(m, "list")
  expect_setequal(names(m), c("id", "title", "notes", "licence",
                               "metadata_modified", "n_resources",
                               "resource_urls"))
  expect_equal(m$id, "australian-tax-gaps")
  expect_gt(m$n_resources, 0L)
  expect_true(is.character(m$resource_urls))
})

test_that("live: ato_meta accepts an ato_tbl", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")
  tbl <- ato_tax_gaps()
  m   <- ato_meta(tbl)
  expect_type(m, "list")
  expect_true(nzchar(m$id))
})

test_that("new_ato_tbl attaches provenance", {
  df <- data.frame(a = 1:3)
  x <- ato:::new_ato_tbl(df, source = "https://data.gov.au",
                          licence = "CC BY 2.5 AU",
                          title = "Test")
  expect_s3_class(x, "ato_tbl")
  expect_s3_class(x, "data.frame")
  expect_equal(attr(x, "ato_title"), "Test")
})

test_that("new_ato_tbl requires a data frame", {
  expect_error(ato:::new_ato_tbl(list(a = 1)))
})

test_that("print.ato_tbl emits header", {
  df <- data.frame(a = 1)
  x <- ato:::new_ato_tbl(df, title = "Demo",
                          source = "https://example.org",
                          retrieved = as.POSIXct("2026-01-01 00:00:00", tz = "UTC"))
  out <- capture.output(print(x))
  expect_true(any(grepl("ato_tbl: Demo", out, fixed = TRUE)))
  expect_true(any(grepl("example.org", out, fixed = TRUE)))
})

test_that("print.ato_tbl returns x invisibly", {
  df <- data.frame(a = 1)
  x <- ato:::new_ato_tbl(df)
  tf <- tempfile()
  sink(tf)
  wv <- withVisible(print(x))
  sink()
  unlink(tf)
  expect_false(wv$visible)
  expect_identical(wv$value, x)
})

# Small Business Benchmarks

#' Small Business Benchmarks
#'
#' Returns the ATO's Small Business Benchmarks: industry-specific
#' performance ranges (cost of sales / turnover, total expenses /
#' turnover, labour / turnover, etc.) derived from small-business
#' income tax returns. Used by the ATO to identify outlier
#' taxpayers, by small-business advisors for comparative
#' analysis, and by tax integrity researchers.
#'
#' @param year Income year in `"YYYY-YY"` form (e.g.
#'   `"2023-24"`) or `"latest"`. Releases available from
#'   2016-17 onwards.
#'
#' @return An `ato_tbl` with one row per (industry, turnover
#'   band, ratio) combination. Ratios are percentages.
#'
#' @source Australian Taxation Office Small Business Benchmarks.
#'   Licensed CC BY 2.5 AU.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   bm <- ato_sme_benchmarks(year = "2023-24")
#'   head(bm)
#' })
#' options(op)
#' }
ato_sme_benchmarks <- function(year = "latest") {
  pkg_id <- ATO_PACKAGE_IDS$sme
  if (identical(year, "latest")) {
    pkg <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year <- if (length(years) > 0L) max(years) else "2023-24"
  } else {
    year <- ato_resolve_year(year)
  }
  ato_check_staleness(pkg_id)
  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Small Business Benchmarks ", year))
}

# Harmonise column names across multi-year ATO releases. Schema
# drifts year to year (column renames, table renumbering); this
# function normalises a stacked panel to a canonical column set
# using ATO_COL_VARIANTS.

#' Harmonise column names in a multi-year ATO panel
#'
#' ATO renames columns across annual releases; a stacked panel
#' from `ato_individuals_postcode(year = c("2020-21", "2021-22"))`
#' may have inconsistent names like `total_income` vs
#' `total_income_or_loss`. `ato_harmonise()` renames columns to
#' the first variant in `ATO_COL_VARIANTS` so panels are join-ready.
#'
#' Unknown columns are left alone. Columns that collide after
#' renaming (because two variants map to the same canonical name)
#' emit a warning; the first column wins.
#'
#' @param df A data frame (typically an `ato_tbl` with `year`
#'   column from a multi-year call).
#'
#' @return A data frame with harmonised names. `ato_tbl` class
#'   and provenance attributes are preserved.
#' @family harmonisation
#' @export
#' @examples
#' df <- data.frame(postcode = "2000",
#'                  total_income_or_loss = 100,
#'                  state_territory = "NSW")
#' ato_harmonise(df)
ato_harmonise <- function(df) {
  stopifnot(is.data.frame(df))
  new_names <- names(df)
  for (canonical in names(ATO_COL_VARIANTS)) {
    variants <- ATO_COL_VARIANTS[[canonical]]
    hits <- which(new_names %in% variants)
    if (length(hits) == 0L) next
    if (length(hits) > 1L) {
      cli::cli_warn(c(
        "Multiple columns map to {.val {canonical}}: \\
         {.val {new_names[hits]}}.",
        "i" = "Keeping first; dropping others."
      ))
      drop <- hits[-1L]
      df <- df[, -drop, drop = FALSE]
      new_names <- names(df)
      hits <- which(new_names %in% variants)
    }
    new_names[hits] <- canonical
  }
  names(df) <- new_names
  df
}

test_that("ato_harmonise renames canonical columns", {
  df <- data.frame(
    post_code = c("2000", "3000"),
    state_territory = c("NSW", "VIC"),
    gender = c("M", "F"),
    stringsAsFactors = FALSE
  )
  out <- ato_harmonise(df)
  expect_true(all(c("postcode", "state", "sex") %in% names(out)))
  expect_false(any(c("post_code", "state_territory", "gender") %in% names(out)))
})

test_that("ato_harmonise leaves unknown columns alone", {
  df <- data.frame(foo = 1, bar = 2, postcode = "2000")
  out <- ato_harmonise(df)
  expect_true(all(c("foo", "bar", "postcode") %in% names(out)))
})

test_that("ato_harmonise warns on collision", {
  df <- data.frame(postcode = "2000", post_code = "3000")
  expect_warning(ato_harmonise(df), "map to")
})

# Level 3 invariant: harmonise is idempotent.
test_that("ato_harmonise is idempotent", {
  df <- data.frame(
    post_code = c("2000", "3000"),
    state_territory = c("NSW", "VIC"),
    gender = c("M", "F"),
    foo = c(1, 2),
    stringsAsFactors = FALSE
  )
  once  <- ato_harmonise(df)
  twice <- ato_harmonise(once)
  expect_identical(once, twice)
})

# Dataset metadata

#' Fetch CKAN metadata for an ATO dataset
#'
#' Returns structured metadata for any ATO dataset on data.gov.au:
#' title, notes, licence, last-modified timestamp, resource count,
#' and all resource URLs. Useful for detecting silent updates before
#' clearing the cache, or for auditing what version of data you have.
#'
#' @param x Either an `ato_tbl` (as returned by any `ato_*` data
#'   function) or a character CKAN package ID / slug (e.g.
#'   `"taxation-statistics-2022-23"`, `"corporate-transparency"`).
#'
#' @return A list with elements:
#'   - `id`: CKAN package slug
#'   - `title`: human-readable title
#'   - `notes`: dataset description (truncated to 400 chars)
#'   - `licence`: licence title
#'   - `metadata_modified`: ISO timestamp of last CKAN update
#'   - `n_resources`: number of downloadable files
#'   - `resource_urls`: character vector of all resource URLs
#'
#' @family configuration
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   # By package ID
#'   m <- ato_meta("taxation-statistics-2022-23")
#'   m$metadata_modified
#'
#'   # From an ato_tbl
#'   tbl <- ato_individuals(year = "2022-23")
#'   ato_meta(tbl)
#' })
#' options(op)
#' }
ato_meta <- function(x) {
  if (inherits(x, "ato_tbl")) {
    src <- attr(x, "ato_source") %||% ""
    # Extract CKAN package slug from a data.gov.au resource URL.
    # URL form: .../data/dataset/<slug>/resource/<id>/download/...
    m <- regmatches(src, regexpr("dataset/([^/]+)", src))
    if (length(m) == 0L || !nzchar(m)) {
      cli::cli_abort(c(
        "Cannot determine package ID from source URL: {.val {src}}",
        "i" = "Pass the package slug directly instead."
      ))
    }
    pkg_id <- sub("dataset/", "", m, fixed = TRUE)
    pkg <- ato_ckan_package(pkg_id)
  } else if (is.character(x) && length(x) == 1L) {
    pkg <- ato_ckan_package(x)
  } else {
    cli::cli_abort(
      "{.arg x} must be an {.cls ato_tbl} or a character package ID."
    )
  }

  resources <- pkg$resources %||% list()
  urls <- vapply(resources, function(r) r$url %||% "", character(1))
  notes_raw <- pkg$notes %||% ""

  list(
    id                = pkg$name %||% "",
    title             = pkg$title %||% "",
    notes             = substr(notes_raw, 1L, 400L),
    licence           = pkg$license_title %||% pkg$license_id %||% "",
    metadata_modified = pkg$metadata_modified %||% "",
    n_resources       = length(resources),
    resource_urls     = urls
  )
}

# Individuals Taxation Statistics

#' @noRd
ato_ts_package_id <- function(year) {
  if (identical(year, "latest")) {
    result <- ato_ckan_search(q = "taxation-statistics", rows = 50L)
    ids <- vapply(result$results, function(p) p$name %||% "", character(1))
    ids <- ids[grepl("^taxation-statistics-[0-9]{4}-[0-9]{2}$", ids)]
    if (length(ids) == 0L) {
      cli::cli_abort("Could not find any Taxation Statistics packages.")
    }
    ids <- sort(ids, decreasing = TRUE)
    return(ids[1L])
  }
  yr <- ato_resolve_year(year)
  paste0("taxation-statistics-", yr)
}

#' Individual Taxation Statistics snapshot
#'
#' Returns the Individuals Table 1 snapshot: aggregate counts,
#' total income, taxable income, tax payable, and deductions
#' across all individual returns (roughly 14 million per year).
#' The snapshot is the headline table; for finer cuts use the
#' dedicated functions:
#' - [ato_individuals_postcode()] for geographic breakdowns,
#' - [ato_individuals_occupation()] for occupation × sex × income-range
#'   detail, or
#' - [ato_download()] with a custom `pattern` for specific
#'   Tables 2 to 27 (age, sex, state, industry, source of income,
#'   deductions, offsets, CGT, non-residents).
#'
#' Monetary values are nominal AUD of the reporting year. Use
#' `inflateR::inflate()` or the ABS CPI series if you need
#' real-term comparisons.
#'
#' @param year Year in `"YYYY-YY"` form (e.g. `"2022-23"`) or
#'   `"latest"`. `"latest"` resolves to the most recently
#'   published release (currently 2022-23).
#'
#' @return An `ato_tbl` with one row per aggregate line-item and
#'   columns for count and amount in nominal AUD.
#'
#' @source Australian Taxation Office Taxation Statistics
#'   <https://www.ato.gov.au/about-ato/research-and-statistics/>.
#'   Licensed CC BY 2.5 AU.
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   ind <- ato_individuals(year = "2022-23")
#'   head(ind)
#' })
#' options(op)
#' }
ato_individuals <- function(year = "latest") {
  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "individual(s)?01|individual_01|snapshot")
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO individuals snapshot ", year))
}

#' Individual tax data by postcode
#'
#' Returns the Individuals Table 6 (or standalone postcode
#' dataset): taxable income, tax payable, and return counts by
#' 4-digit postcode. Headline dataset for income-distribution
#' journalism.
#'
#' **Privacy suppression.** The ATO suppresses postcodes with
#' fewer than 50 returns; those cells are returned as `NA` after
#' parsing (the package maps `"np"`, `"*"`, and similar tokens
#' to `NA` so numeric columns stay numeric). Small or remote
#' postcodes will be silently missing from the output.
#'
#' Monetary values are nominal AUD of the reporting year. Use
#' `inflateR::inflate()` for real-term series.
#'
#' @param year `"YYYY-YY"` or `"latest"`. Pass a vector of years
#'   (e.g. `c("2020-21", "2021-22", "2022-23")` or `2018:2022`)
#'   to stack multiple years with a `year` column added to the
#'   output. Useful for time-series analysis.
#' @param state Optional character vector of state codes (e.g.
#'   `"NSW"`, `c("VIC", "QLD")`).
#' @param postcode Optional character vector of 4-digit postcodes.
#'
#' @return An `ato_tbl` with one row per postcode (or per postcode
#'   per year for multi-year queries), including state, return
#'   count, total income, taxable income, and tax payable. Schema
#'   drifts year to year (SA3/SA4 columns present from 2017
#'   onwards).
#'
#' @source Australian Taxation Office Taxation Statistics
#'   postcode release. Licensed CC BY 2.5 AU.
#'
#' @references
#' Atkinson, A.B. and Leigh, A. (2007). "The Distribution of Top
#'   Incomes in Australia." \emph{Economic Record}, 83(262),
#'   247-261. \doi{10.1111/j.1475-4932.2007.00412.x}
#'
#' Burkhauser, R.V., Hahn, M.H. and Wilkins, R. (2015). "Measuring
#'   top incomes using tax record data: a cautionary tale from
#'   Australia." \emph{Journal of Economic Inequality}, 13(2),
#'   181-205. \doi{10.1007/s10888-014-9281-z}
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   # Single year
#'   p <- ato_individuals_postcode(year = "2022-23", state = "NSW")
#'   head(p)
#'   # Multi-year stack with year column
#'   panel <- ato_individuals_postcode(year = c("2020-21", "2021-22"),
#'                                     state = "NSW")
#' })
#' options(op)
#' }
ato_individuals_postcode <- function(year = "latest", state = NULL,
                                      postcode = NULL) {
  # Multi-year path: stack results with a year column.
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    first_src <- NULL
    parts <- lapply(resolved, function(y) {
      df <- ato_individuals_postcode(year = y, state = state,
                                     postcode = postcode)
      if (is.null(first_src)) first_src <<- attr(df, "ato_source")
      df <- as.data.frame(df, stringsAsFactors = FALSE)
      df$year <- y
      df
    })
    common <- Reduce(intersect, lapply(parts, names))
    common <- common[nzchar(common)]
    if (length(common) == 0L) {
      cli::cli_abort(c(
        "No column names are common across the requested years.",
        "i" = "Try fewer years or inspect each year separately first."
      ))
    }
    stacked <- do.call(
      rbind,
      lapply(parts, function(d) d[, common, drop = FALSE])
    )
    rownames(stacked) <- NULL
    return(new_ato_tbl(stacked,
                       source  = first_src %||% "",
                       licence = "CC BY 2.5 AU",
                       title   = paste0("ATO individuals by postcode (",
                                        min(resolved), " to ", max(resolved),
                                        ")")))
  }

  id <- ato_ts_package_id(year)
  res <- ato_ckan_resolve(id, "postcode")
  url <- res$url %||% ""
  cached <- ato_download_cached(url)
  sheets <- tryCatch(readxl::excel_sheets(cached),
                     error = function(e) character(0))
  target_sheet <- if (length(sheets) > 0L &&
                      tolower(sheets[1]) %in% c("notes", "cover",
                                                 "information",
                                                 "contents")) {
    2L
  } else {
    1L
  }
  df <- ato_fetch_xlsx(url, sheet = target_sheet)

  state_col <- ato_find_col(df, "state")
  if (!is.null(state) && !is.na(state_col)) {
    df <- df[toupper(df[[state_col]]) %in% toupper(state), , drop = FALSE]
  }
  pc_col <- ato_find_col(df, "postcode")
  if (!is.null(postcode) && !is.na(pc_col)) {
    df <- df[as.character(df[[pc_col]]) %in% as.character(postcode), ,
             drop = FALSE]
  }
  rownames(df) <- NULL

  ato_warn_suppression(df, context = "postcode cells")

  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO individuals by postcode ", year))
}

#' Individual tax data by occupation
#'
#' Returns the Individuals Table 14 (occupation by sex by taxable
#' income range). Around 1,000 occupations classified by ANZSCO
#' with aggregate counts, total income, taxable income, and tax
#' payable. The ATO migrated from ANZSCO 2013 to ANZSCO 2021
#' across the 2022-23 release; cross-year joins on occupation
#' name or code must account for the recode.
#'
#' **Classification break.** Releases from 2022-23 onwards use
#' ANZSCO 2021; earlier releases use ANZSCO 2013. A warning is
#' emitted when the requested year(s) are at or after this boundary,
#' or when a multi-year request spans it.
#'
#' @param year `"YYYY-YY"`, `"latest"`, or a vector of years for
#'   a multi-year panel (e.g. `c("2020-21", "2021-22", "2022-23")`).
#' @param occupation Optional substring filter (case-insensitive)
#'   applied to the occupation description column.
#' @param sex One of `"all"` (default), `"male"`, or `"female"`.
#'   Rows with sex recorded as `"Not stated"` are dropped when
#'   filtering to male or female. Short forms `"m"`/`"f"` are
#'   accepted.
#'
#' @return An `ato_tbl` with one row per occupation-sex-income
#'   combination. Multi-year queries add a `year` column.
#'   Monetary values in nominal AUD of the reporting year.
#'
#' @source Australian Taxation Office Taxation Statistics.
#'   Licensed CC BY 2.5 AU.
#'
#' @family individuals
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   occ <- ato_individuals_occupation(year = "2022-23",
#'                                     occupation = "economist")
#'   head(occ)
#'   # Multi-year panel
#'   panel <- ato_individuals_occupation(year = c("2021-22", "2022-23"),
#'                                       occupation = "nurse")
#' })
#' options(op)
#' }
ato_individuals_occupation <- function(year = "latest", occupation = NULL,
                                        sex = c("all", "male", "female",
                                                 "m", "f")) {
  sex <- match.arg(sex)
  sex <- switch(sex, m = "male", f = "female", sex)

  # Multi-year path
  if (length(year) > 1L) {
    resolved <- vapply(as.list(year), ato_resolve_year, character(1))
    ato_warn_classification_span(resolved, "anzsco")
    fn <- function(year) {
      ato_individuals_occupation(year = year, occupation = occupation, sex = sex)
    }
    return(ato_stack_years(fn, resolved,
                           title_prefix = "ATO individuals by occupation"))
  }

  id  <- ato_ts_package_id(year)
  resolved_year <- sub("taxation-statistics-", "", id)
  ato_warn_classification_break(resolved_year, "anzsco")

  res <- ato_ckan_resolve(id, "occupation|individual(s)?14|individual_14")
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)

  occ_col <- ato_find_col(df, "occupation")
  if (!is.null(occupation) && !is.na(occ_col)) {
    pattern <- paste(tolower(occupation), collapse = "|")
    df <- df[grepl(pattern, tolower(df[[occ_col]])), , drop = FALSE]
  }
  sex_col <- ato_find_col(df, "sex")
  if (sex != "all" && !is.na(sex_col)) {
    keep <- tolower(df[[sex_col]]) == sex
    df   <- df[keep, , drop = FALSE]
  }

  if (!is.null(occupation) && nrow(df) == 0L) {
    cli::cli_warn("No occupation rows matched {.val {occupation}}.")
  }

  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO individuals by occupation ", year))
}

test_that("ato_individuals_occupation validates sex", {
  expect_error(ato_individuals_occupation(sex = "x"))
})

test_that("ato_individuals_occupation accepts long and short sex forms", {
  # match.arg succeeds without hitting the network; we only
  # check the arg validation, not the fetch.
  expect_silent(match.arg("male", c("all", "male", "female", "m", "f")))
  expect_silent(match.arg("f",    c("all", "male", "female", "m", "f")))
})

test_that("ato_individuals_postcode live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("ATO_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  p <- ato_individuals_postcode(year = "2022-23", state = "NSW")
  expect_s3_class(p, "ato_tbl")
})

#' ato: Australian Taxation Office Data
#'
#' Tidy R access to Australian Taxation Office ('ATO') Taxation
#' Statistics and related datasets via the 'data.gov.au' CKAN API.
#'
#' @section Main function families:
#' - [ato_catalog()], [ato_download()]: discovery and generic
#'   resource access.
#' - [ato_individuals()], [ato_individuals_postcode()],
#'   [ato_individuals_occupation()]: individual tax return items.
#'   `ato_individuals_postcode()` accepts a vector of years for
#'   time-series panels.
#' - [ato_companies()]: company tax across Tables 1-9 (snapshot,
#'   by size, by entity type, by industry, by industry-and-size,
#'   by sub-industry, by taxable status, by source, by expenses).
#' - [ato_super_funds()]: APRA-regulated super funds and SMSFs.
#' - [ato_top_taxpayers()]: Corporate Tax Transparency release
#'   (Income tax or PRRT sheet).
#' - [ato_gst()], [ato_industry()]: GST and industry aggregates.
#' - [ato_tax_gaps()]: annual Tax Gap estimates across tax heads.
#' - [ato_rdti()]: Research and Development Tax Incentive
#'   claimants.
#' - [ato_irpd()]: International Related Party Dealings (BEPS /
#'   transfer pricing).
#' - [ato_excise()]: excise rates, fuel tax credits, beer and
#'   spirits clearances.
#' - [ato_sme_benchmarks()]: Small Business Benchmarks.
#' - [ato_help()]: HELP / AASL / VSL student-loan statistics.
#' - [ato_cite()]: citation helper (BibTeX, APA, plain text).
#' - [ato_cache_info()], [ato_clear_cache()]: cache management.
#'
#' @section Caveats for users of ATO data:
#'
#' **Nominal AUD.** All monetary values are in nominal Australian
#' dollars of the reporting year. Use `inflateR::inflate()` or
#' ABS CPI series for real-term comparisons across years.
#'
#' **Fiscal year convention.** `"2022-23"` refers to the
#' Australian income year from 1 July 2022 to 30 June 2023.
#'
#' **Confidentiality suppression.** The ATO replaces values with
#' `"np"` (not published), `"*"`, `"\u2021"`, or similar tokens when
#' fewer than ten taxpayers (or fewer than 50 returns for
#' postcode data) fall in a cell. The package coerces these to
#' `NA` so numeric columns stay numeric. Summing a column with
#' `na.rm = TRUE` is the correct default; be explicit about what
#' you do with suppressed cells in any distributional analysis.
#'
#' **Schema drift.** Column names and table numbering shift
#' year-to-year (e.g. occupation data has been T13, T14, T15 in
#' different years). The package cleans names to snake_case on
#' ingestion but does not normalise schemas across years. A
#' cross-year join requires explicit column mapping.
#'
#' **Classification migration.** ANZSIC 2006 to ANZSIC 2020 and
#' ANZSCO 2013 to ANZSCO 2021 recodes affect industry and
#' occupation series. Users stacking across releases should
#' inspect the classification version used in each release.
#'
#' **Silent revisions.** CKAN `metadata_modified` timestamps
#' change without a version bump when the ATO republishes a
#' corrected file. Run [ato_clear_cache()] to force a refresh.
#'
#' **Microdata is out of scope.** The 2% Individual Sample File
#' is distributed separately through Hugh Parsonage's
#' [taxstats](https://github.com/HughParsonage/taxstats) DRAT
#' repo. ALife (the ATO Longitudinal Information Files) is
#' restricted-access microdata via the ATO DataLab and requires
#' a researcher application; it is not provided by this package.
#'
#' **Citation.** Use [ato_cite()] on any returned `ato_tbl` to
#' produce a Treasury-grade footnote, APA reference, or BibTeX
#' entry with the source URL, licence, retrieval date, and title.
#'
#' @section Data source:
#' Taxation Statistics are published annually by the ATO on
#' <https://www.ato.gov.au/about-ato/research-and-statistics/> and
#' mirrored at <https://data.gov.au/data/organization/australiantaxationoffice>.
#' Most datasets are licensed under Creative Commons Attribution 2.5
#' Australia; Corporate Tax Transparency and the Voluntary Tax
#' Transparency Code are CC BY 3.0 Australia.
#'
#' @keywords internal
#' @concept Australian tax
#' @concept taxation statistics
#' @concept income distribution
"_PACKAGE"

# Package-level constants

# Central registry of ATO CKAN package IDs.
# If one breaks, ato_ckan_package() errors with the slug so the fix is obvious.
#' @noRd
ATO_PACKAGE_IDS <- list(
  tax_gaps  = "australian-tax-gaps",
  rdti      = "research-and-development-tax-incentive",
  irpd_bare = "international-related-party-dealings",
  excise    = "excise-data",
  sme       = "small-business-benchmarks",
  help      = "higher-education-loan-program-help",
  smsf      = "self-managed-superannuation-funds",
  ctt       = "corporate-transparency",
  vttc      = "voluntary-tax-transparency-code"
)

# Column name variants per logical role. ATO renames columns across years;
# each vector lists all known names in priority order.
# Use ato_find_col(df, "key") to pick the first matching variant.
#' @noRd
ATO_COL_VARIANTS <- list(
  state      = c("state", "state_territory"),
  postcode   = c("postcode", "post_code"),
  sex        = c("sex", "gender"),
  industry   = c("industry", "industry_description",
                 "broad_industry", "anzsic_industry"),
  entity     = c("entity_type", "tax_entity_type", "company_type"),
  occupation = c("occupation", "occupation_description")
)

# Start year (as integer) of the ANZSCO 2013 -> 2021 and ANZSIC 2006 -> 2020
# reclassification, applied across the 2022-23 Taxation Statistics release.
#' @noRd
ATO_CLASSIFICATION_BREAK_YEAR <- 2022L

# Deflation: convert nominal AUD to real AUD of a chosen base
# financial year, using ABS CPI All Groups Australia (bundled).

#' Deflate nominal AUD to real AUD
#'
#' Converts a numeric vector of nominal AUD figures indexed by
#' financial year to real AUD of a chosen base year using the
#' bundled ABS CPI series (annual, All Groups Australia,
#' 2011-12 = 1.0). For the user's `inflateR` workflow in
#' non-Australian contexts, bundle a matching CPI series and
#' call this with a custom `cpi =` argument.
#'
#' @details
#' Uses proportional (Laspeyres-style) adjustment:
#' \eqn{real = nominal \times (CPI_{base} / CPI_{source})}. The
#' bundled CPI is the ABS annual All Groups Australia index
#' published in cat. 6401.0, rebased so that 2011-12 = 1.000.
#' This is the standard rebasing used in most Australian
#' time-series work and is consistent with ABS System of National
#' Accounts methodology (cat. 5204.0).
#'
#' The formula is exact for a chain-linked index after 1949 (when
#' the ABS CPI was introduced) and approximate for earlier values
#' that rely on Commonwealth Statistician retail-price series. Use
#' a custom `cpi =` argument if you need a different deflator
#' (e.g. GDP deflator, wage price index, or industry-specific PPI).
#'
#' @param x Numeric vector of nominal AUD values.
#' @param year Character vector of financial years for each entry
#'   in `x`, in `"YYYY-YY"` form. Must be the same length as `x`.
#' @param base Base financial year for real terms (default
#'   `"2022-23"`).
#' @param cpi Optional override: a data frame with columns
#'   `financial_year` and `cpi_all_groups_australia`. Default
#'   uses the bundled ABS series.
#'
#' @return Numeric vector of real AUD values in base-year prices.
#'
#' @references
#' Australian Bureau of Statistics (2024). \emph{Consumer Price
#'   Index, Australia: Concepts, Sources and Methods}. Catalogue
#'   6461.0.
#'
#' Australian Bureau of Statistics (2024). \emph{Consumer Price
#'   Index, Australia}. Catalogue 6401.0.
#'
#' Diewert, W.E. (1998). "Index Number Issues in the Consumer Price
#'   Index." \emph{Journal of Economic Perspectives}, 12(1),
#'   47-58. \doi{10.1257/jep.12.1.47}
#'
#' @family harmonisation
#' @export
#' @examples
#' ato_deflate(c(100, 100, 100),
#'             year = c("2012-13", "2017-18", "2022-23"),
#'             base = "2022-23")
ato_deflate <- function(x, year, base = "2022-23", cpi = NULL) {
  stopifnot(is.numeric(x), is.character(year), length(x) == length(year))
  if (is.null(cpi)) cpi <- ato_crosswalk("cpi")
  cpi_vec <- stats::setNames(cpi$cpi_all_groups_australia, cpi$financial_year)
  base_val <- cpi_vec[[base]]
  if (is.null(base_val) || is.na(base_val)) {
    cli::cli_abort(c(
      "Base year {.val {base}} not in CPI series.",
      "i" = "Available: {.val {names(cpi_vec)}}."
    ))
  }
  ratios <- base_val / cpi_vec[year]
  missing <- is.na(ratios)
  if (any(missing)) {
    cli::cli_warn(c(
      "CPI missing for {.val {unique(year[missing])}}; \\
       those entries returned as {.val NA}."
    ))
  }
  unname(x * ratios)
}

#' Express an aggregate per capita using ABS ERP
#'
#' @details
#' Divides the input by Estimated Resident Population at 30 June
#' of the financial year's end (a stock measure). For flow-style
#' measures where a mid-year-average population is preferable,
#' substitute a custom `erp =` argument. ERP is ABS's preferred
#' population-denominator concept for per-capita economic
#' statistics (see cat. 3101.0 methodology).
#'
#' @param x Numeric vector of aggregate values (same length as
#'   `year`).
#' @param year Character vector of financial years.
#' @param erp Optional override: data frame with columns
#'   `financial_year` and `erp_june_australia_thousands`.
#'
#' @return Numeric vector of per-capita values (same units as `x`
#'   per person).
#'
#' @references
#' Australian Bureau of Statistics (2024). \emph{National,
#'   State and Territory Population}. Catalogue 3101.0.
#'
#' @family harmonisation
#' @export
#' @examples
#' # Income tax per person, 2022-23 FBO headline
#' ato_per_capita(316.4e9, "2022-23")
ato_per_capita <- function(x, year, erp = NULL) {
  stopifnot(is.numeric(x), is.character(year), length(x) == length(year))
  if (is.null(erp)) erp <- ato_crosswalk("erp")
  erp_vec <- stats::setNames(erp$erp_june_australia_thousands * 1e3,
                       erp$financial_year)
  pop <- erp_vec[year]
  missing <- is.na(pop)
  if (any(missing)) {
    cli::cli_warn(c(
      "ERP missing for {.val {unique(year[missing])}}; \\
       those entries returned as {.val NA}."
    ))
  }
  unname(x / pop)
}

test_that("ato_deflate returns base-year values unchanged", {
  out <- ato_deflate(100, year = "2022-23", base = "2022-23")
  expect_equal(out, 100)
})

test_that("ato_deflate scales older values up", {
  # 2012-13 CPI = 101.7, 2022-23 CPI = 132.9
  # 100 in 2012-13 AUD should be about 130.7 in 2022-23 AUD
  out <- ato_deflate(100, year = "2012-13", base = "2022-23")
  expect_gt(out, 125)
  expect_lt(out, 135)
})

test_that("ato_deflate handles vector input", {
  out <- ato_deflate(c(100, 100, 100),
                     year = c("2012-13", "2017-18", "2022-23"),
                     base = "2022-23")
  expect_length(out, 3L)
  expect_equal(out[3], 100)
  expect_gt(out[1], out[2])
  expect_gt(out[2], out[3])
})

test_that("ato_deflate warns on missing CPI year", {
  expect_warning(
    ato_deflate(100, year = "1850-51", base = "2022-23"),
    "CPI missing"
  )
})

test_that("ato_per_capita divides by ERP correctly", {
  # AUD 316.4 billion / ~27 million = around AUD 12,000 per person
  out <- ato_per_capita(316.4e9, "2022-23")
  expect_gt(out, 10000)
  expect_lt(out, 15000)
})

test_that("ato_per_capita vector input", {
  out <- ato_per_capita(c(100e9, 200e9),
                        year = c("2020-21", "2022-23"))
  expect_length(out, 2L)
})

# RBA Commonwealth receipts helper.

#' RBA Commonwealth receipts (H1 statistical table)
#'
#' Pointer to the RBA's H1 series on Commonwealth receipts for
#' long-run time series. RBA compiles since 1959-60, filling gaps
#' in ATO Taxation Statistics which start 1994-95.
#'
#' The RBA publishes H1 as an XLSX with stable URL. This function
#' fetches it and returns a tidy tibble.
#'
#' @param series One of `"receipts"` (default, all Commonwealth
#'   receipts by category) or `"income_tax"` (income tax only).
#'
#' @return An `ato_tbl`.
#'
#' @source Reserve Bank of Australia Statistical Tables H1
#'   <https://www.rba.gov.au/statistics/tables/>.
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(ato_rba(series = "receipts"))
#' options(op)
#' }
ato_rba <- function(series = c("receipts", "income_tax")) {
  series <- match.arg(series)
  url <- "https://www.rba.gov.au/statistics/tables/xls/h01hist.xlsx"
  df <- tryCatch(ato_fetch_xlsx(url, sheet = 1),
                 error = function(e) {
                   cli::cli_warn(c(
                     "RBA fetch failed: {conditionMessage(e)}.",
                     "i" = "RBA sometimes rotates URLs; check {.url https://www.rba.gov.au/statistics/tables/}."
                   ))
                   data.frame()
                 })
  new_ato_tbl(df,
              source = url,
              licence = "RBA terms",
              title = paste0("RBA Commonwealth ", series))
}

# Reconcile ATO Taxation Statistics totals against authoritative
# Commonwealth revenue figures (Final Budget Outcome, Budget
# Paper 1). This is the single biggest credibility feature for
# PBO or Grattan-style published work: a one-row diff table
# showing what the ATO data implies vs what Treasury reports.

#' Reconcile an aggregate against Commonwealth budget totals
#'
#' Compares a scalar (or data frame total) against the published
#' Final Budget Outcome figure for the same year and revenue line.
#' Useful as a sanity check on an ATO Taxation Statistics sum
#' before reporting it in a paper or brief.
#'
#' @details
#' Discrepancies between ATO Taxation Statistics aggregates and
#' the Final Budget Outcome (FBO) are expected and meaningful:
#' \itemize{
#'   \item Taxation Statistics are based on assessments made by a
#'     cut-off date (usually October of the following calendar
#'     year) and may exclude late-lodging returns.
#'   \item FBO figures are cash-basis Commonwealth receipts;
#'     Taxation Statistics are accrual-basis tax assessed.
#'   \item GST, excise, and fuel credits have timing and refund
#'     effects that further distort the cash-vs-assessment gap.
#' }
#' A 1-3 per cent gap is consistent with the accrual-to-cash
#' reconciliation Treasury publishes in the FBO statement of
#' revenues; larger gaps warrant investigation. The bundled
#' reference totals in `inst/extdata/budget_reference_totals.csv`
#' are taken from the relevant FBO release, with the precise table
#' cited in the `source` column of each row.
#'
#' @param value Numeric; the figure to check, in AUD (not AUD
#'   billions). An `ato_tbl` can also be passed: pass `sum_column`
#'   to pick which numeric column to sum.
#' @param year Financial year, e.g. `"2022-23"`.
#' @param measure One of the measure codes in `ato_crosswalk("budget")`,
#'   for example `"individuals_income_tax_net"`,
#'   `"company_tax_net"`, `"gst_net"`, `"fuel_excise_net"`.
#' @param sum_column Column name to sum when `value` is a data
#'   frame. Default `NULL` (errors if multiple numeric columns
#'   exist).
#'
#' @return A one-row data frame: `measure`, `year`,
#'   `value_aud`, `reference_aud`, `diff_aud`, `pct_diff`,
#'   `source`. Emits a warning if `abs(pct_diff) > 0.05`.
#'
#' @references
#' Commonwealth of Australia (various years). \emph{Final Budget
#'   Outcome}. The Treasury, Canberra.
#'   \url{https://budget.gov.au/content/fbo/index.htm}
#'
#' Australian Bureau of Statistics (various years).
#'   \emph{Taxation Revenue, Australia}. Catalogue 5506.0.
#'
#' Australian Taxation Office (annual). \emph{Australian tax gaps
#'   -- overview}, methodology notes on accrual-vs-cash
#'   reconciliation.
#'
#' @family harmonisation
#' @export
#' @examples
#' ato_reconcile(value = 316.4e9,
#'               year = "2022-23",
#'               measure = "individuals_income_tax_net")
ato_reconcile <- function(value, year, measure,
                           sum_column = NULL) {
  if (is.data.frame(value)) {
    cols <- names(value)[vapply(value, is.numeric, logical(1))]
    if (is.null(sum_column)) {
      if (length(cols) == 0L) {
        cli::cli_abort("No numeric columns in {.arg value}.")
      }
      if (length(cols) > 1L) {
        cli::cli_abort(c(
          "Multiple numeric columns in {.arg value}.",
          "i" = "Pass {.arg sum_column} to pick one: {.val {cols}}."
        ))
      }
      sum_column <- cols
    }
    v <- sum(value[[sum_column]], na.rm = TRUE)
  } else {
    stopifnot(is.numeric(value), length(value) == 1L)
    v <- value
  }

  ref <- ato_crosswalk("budget")
  hit <- ref[ref$financial_year == year & ref$measure == measure, ]
  if (nrow(hit) == 0L) {
    avail <- unique(ref$measure)
    cli::cli_abort(c(
      "No budget reference for {.val {measure}} in {.val {year}}.",
      "i" = "Available measures: {.val {avail}}.",
      "i" = "Available years: {.val {unique(ref$financial_year)}}."
    ))
  }
  ref_v <- hit$value_aud_billion[1L] * 1e9
  diff <- v - ref_v
  pct <- diff / ref_v

  if (abs(pct) > 0.05) {
    cli::cli_warn(c(
      "Reconciliation diff {round(100*pct, 1)}% for {.val {measure}} \\
       ({.val {year}}).",
      "i" = "ATO data: {.val {format(v, big.mark = ',', scientific = FALSE)}}.",
      "i" = "Reference: {.val {format(ref_v, big.mark = ',', scientific = FALSE)}}.",
      "i" = "Expected 1-3% accrual-vs-cash gap; investigate larger."
    ))
  }

  data.frame(
    measure       = measure,
    year          = year,
    value_aud     = v,
    reference_aud = ref_v,
    diff_aud      = diff,
    pct_diff      = pct,
    source        = hit$source[1L],
    stringsAsFactors = FALSE
  )
}

test_that("ato_reconcile returns a 1-row diff frame", {
  res <- ato_reconcile(value = 316.4e9,
                       year = "2022-23",
                       measure = "individuals_income_tax_net")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), 1L)
  expect_true(all(c("measure", "year", "value_aud", "reference_aud",
                    "diff_aud", "pct_diff", "source") %in% names(res)))
  # Exact match -> diff = 0
  expect_equal(res$diff_aud, 0)
  expect_equal(res$pct_diff, 0)
})

test_that("ato_reconcile warns on >5% discrepancy", {
  expect_warning(
    ato_reconcile(value = 400e9, year = "2022-23",
                  measure = "individuals_income_tax_net"),
    "Reconciliation diff"
  )
})

test_that("ato_reconcile errors on unknown measure", {
  expect_error(
    ato_reconcile(value = 1e9, year = "2022-23",
                  measure = "not_a_measure"),
    "reference"
  )
})

test_that("ato_reconcile sums a data frame column", {
  df <- data.frame(tax = c(100e9, 216.4e9))
  res <- ato_reconcile(df, year = "2022-23",
                       measure = "individuals_income_tax_net")
  expect_equal(res$value_aud, 316.4e9)
})

# Level 3 invariant: pct_diff sign matches (value - reference).
test_that("ato_reconcile pct_diff sign is consistent with diff", {
  high <- suppressWarnings(ato_reconcile(400e9, "2022-23",
                                          "individuals_income_tax_net"))
  low  <- suppressWarnings(ato_reconcile(200e9, "2022-23",
                                          "individuals_income_tax_net"))
  expect_gt(high$diff_aud, 0)
  expect_gt(high$pct_diff, 0)
  expect_lt(low$diff_aud, 0)
  expect_lt(low$pct_diff, 0)
})

# Reproducibility: session-level snapshot date pin.
#
# The snapshot_date is an ISO date the user records as the intended
# vintage of the data. Fetchers record it in provenance attributes
# and in the manifest. It does not alter CKAN requests (CKAN does
# not generally serve historical URLs), but gives a human-readable
# pin that pairs with SHA-256 integrity checks.

#' Pin or inspect the session snapshot date
#'
#' Call once at the top of an analysis script to declare the vintage
#' of ATO data you intend to use. Every subsequent `ato_*` fetch
#' records this date in the `ato_tbl` provenance header, in
#' `ato_manifest()` entries, and in `ato_cite()` output. Combined
#' with SHA-256 integrity (see [ato_sha256()] and [ato_manifest()]),
#' this gives a reproducible audit trail acceptable for PBO or
#' Grattan-style published work.
#'
#' If called with no arguments, returns the current pin (or `NULL`
#' if unset).
#'
#' @param date ISO `"YYYY-MM-DD"` character, `Date`, or `POSIXct`.
#'   Pass `NULL` to clear.
#'
#' @return Invisibly, the new pinned date (as `Date`), or `NULL`.
#' @family reproducibility
#' @export
#' @examples
#' ato_snapshot("2026-04-24")
#' ato_snapshot()
#' ato_snapshot(NULL)
ato_snapshot <- function(date) {
  if (missing(date)) {
    return(.ato_env$snapshot_date %||% NULL)
  }
  if (is.null(date)) {
    .ato_env$snapshot_date <- NULL
    return(invisible(NULL))
  }
  d <- tryCatch(as.Date(date), error = function(e) NA)
  if (is.na(d)) {
    cli::cli_abort(
      "Could not parse {.arg date} as a date. Use {.val YYYY-MM-DD}."
    )
  }
  .ato_env$snapshot_date <- d
  invisible(d)
}

#' Get the current snapshot pin as a character string (or NA)
#' @noRd
ato_snapshot_str <- function() {
  d <- .ato_env$snapshot_date %||% NULL
  if (is.null(d)) NA_character_ else format(d, "%Y-%m-%d")
}

test_that("ato_snapshot gets and sets the snapshot pin", {
  on.exit(ato_snapshot(NULL), add = TRUE)
  ato_snapshot(NULL)
  expect_null(ato_snapshot())

  d <- ato_snapshot("2026-04-24")
  expect_s3_class(d, "Date")
  expect_equal(format(ato_snapshot(), "%Y-%m-%d"), "2026-04-24")
})

test_that("ato_snapshot accepts Date and POSIXct", {
  on.exit(ato_snapshot(NULL), add = TRUE)
  ato_snapshot(as.Date("2026-01-01"))
  expect_equal(format(ato_snapshot(), "%Y-%m-%d"), "2026-01-01")

  ato_snapshot(as.POSIXct("2026-06-01", tz = "UTC"))
  expect_equal(format(ato_snapshot(), "%Y-%m-%d"), "2026-06-01")
})

test_that("ato_snapshot errors on bad input", {
  on.exit(ato_snapshot(NULL), add = TRUE)
  expect_error(ato_snapshot("not-a-date"), "parse")
})

test_that("ato_snapshot(NULL) clears the pin", {
  ato_snapshot("2026-04-24")
  ato_snapshot(NULL)
  expect_null(ato_snapshot())
})

# HTTP helpers : httr2 wrapper with local caching

#' @noRd
ato_user_agent <- function() {
  paste0("ato R package/", utils::packageVersion("ato"),
         " (+https://github.com/charlescoverdale/ato)")
}

#' @noRd
ato_request <- function(url, timeout = 120) {
  httr2::request(url) |>
    httr2::req_user_agent(ato_user_agent()) |>
    httr2::req_throttle(rate = 5 / 10) |>
    httr2::req_timeout(timeout) |>
    httr2::req_retry(max_tries = 3) |>
    httr2::req_error(is_error = function(r) FALSE)
}

#' Download a file to the cache and return the local path
#' @noRd
ato_download_cached <- function(url, cache = TRUE) {
  d <- ato_cache_dir()
  ext <- tools::file_ext(url)
  ext <- if (nzchar(ext)) paste0(".", ext) else ""
  file <- file.path(d, paste0(ato_digest_url(url), ext))

  if (cache && file.exists(file)) {
    # Verify SHA sidecar; warn on drift.
    ato_sha_verify(file)
    .ato_manifest_append(url = url, file = file)
    return(file)
  }

  cli::cli_progress_step("Downloading {.url {url}}")

  resp <- tryCatch(
    ato_request(url) |>
      httr2::req_perform(path = file),
    error = function(e) {
      if (file.exists(file)) unlink(file)
      cli::cli_abort(c(
        "Download failed.",
        "x" = conditionMessage(e)
      ))
    }
  )

  if (!is.null(resp) && httr2::resp_status(resp) >= 400L) {
    unlink(file, force = TRUE)
    cli::cli_abort(c(
      "HTTP {httr2::resp_status(resp)} from {.url {url}}."
    ))
  }

  # First download: write SHA sidecar, record in manifest.
  ato_sha_write(file)
  .ato_manifest_append(url = url, file = file)
  file
}

#' @noRd
ato_digest_url <- function(url) {
  tmp <- tempfile()
  on.exit(unlink(tmp))
  con <- file(tmp, open = "wb")
  writeLines(enc2utf8(url), con = con, sep = "")
  close(con)
  unname(tools::md5sum(tmp))
}

test_that("ato_digest_url is deterministic and returns MD5 hex", {
  a <- ato:::ato_digest_url("https://data.gov.au/a")
  b <- ato:::ato_digest_url("https://data.gov.au/a")
  d <- ato:::ato_digest_url("https://data.gov.au/b")
  expect_equal(a, b)
  expect_false(identical(a, d))
  expect_match(a, "^[0-9a-f]{32}$")
})

test_that("ato_user_agent includes package version", {
  ua <- ato:::ato_user_agent()
  expect_match(ua, "^ato R package/")
})

test_that("ato_request returns a httr2_request", {
  req <- ato:::ato_request("https://data.gov.au")
  expect_s3_class(req, "httr2_request")
})

test_that("ato_download_cached reuses existing cached file", {
  tmp <- tempfile("ato_cache_")
  op <- options(ato.cache_dir = tmp)
  on.exit(options(op), add = TRUE)
  dir.create(tmp, recursive = TRUE)
  url <- "https://data.gov.au/some.csv"
  hash <- ato:::ato_digest_url(url)
  fake <- file.path(tmp, paste0(hash, ".csv"))
  writeLines("a,b\n1,2", fake)
  got <- ato:::ato_download_cached(url)
  expect_equal(normalizePath(got), normalizePath(fake))
})

# Excise and Fuel Tax Credits

#' Excise and fuel tax credit rates and clearances
#'
#' Returns ATO excise data, covering four sub-releases:
#' - **beer** : beer clearances summary (volumes by product class)
#' - **spirits** : spirits and other excisable beverages clearances
#' - **excise_rates** : historical excise rate schedule (all
#'   excise categories, quarterly indexed rates)
#' - **ftc_rates** : historical Fuel Tax Credit rates
#'
#' @param table One of `"beer"`, `"spirits"`, `"excise_rates"`
#'   (default), or `"ftc_rates"`.
#'
#' @return An `ato_tbl`. Rates are in AUD per litre (or per kg
#'   for tobacco); volumes are in megalitres or similar.
#'
#' @source Australian Taxation Office excise data.
#'   Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Excise Act 1901};
#'   \emph{Excise Tariff Act 1921}; \emph{Fuel Tax Act 2006}.
#'
#' Australian Taxation Office (annual). \emph{Excise data:
#'   methodology and indexation notes}. Excise rates are indexed
#'   to the Consumer Price Index twice a year (February and
#'   August) for most commodities.
#'
#' Productivity Commission (2016). \emph{Migrant Intake into
#'   Australia} (for tobacco excise distributional analysis);
#'   \emph{Harmful Drinking} inquiry (for alcohol excise
#'   distributional analysis).
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   rates <- ato_excise("excise_rates")
#'   head(rates)
#' })
#' options(op)
#' }
ato_excise <- function(table = c("excise_rates", "ftc_rates",
                                  "beer", "spirits")) {
  table <- match.arg(table)
  pattern <- switch(table,
    excise_rates = "historical-excise-rates|excise-rates",
    ftc_rates    = "historical-ftc-rates|ftc-rates",
    beer         = "beer-clearance",
    spirits      = "spirits|other-excisable-beverage"
  )
  res <- ato_ckan_resolve(ATO_PACKAGE_IDS$excise, pattern)
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Excise data (", table, ")"))
}

# Treasury Tax Expenditures and Insights Statement (TEIS)

#' Tax Expenditures and Insights Statement (TEIS)
#'
#' Returns the Treasury TEIS annual table of concession-by-concession
#' tax expenditure estimates in AUD millions. TEIS is the
#' authoritative cost-of-concessions dataset used in PBO and Grattan
#' tax reform costings.
#'
#' TEIS is published by Treasury, not ATO; the function attempts a
#' CKAN search on data.gov.au for the TEIS release, and falls back
#' to the Treasury web URL if not indexed.
#'
#' Key concessions covered: CGT main residence exemption, CGT 50%
#' discount, superannuation earnings tax concession, franking credit
#' refundability, work-related deductions, fuel tax credit scheme,
#' R&D tax incentive, GST food exemption, and many more.
#'
#' @param year Reference year for the TEIS release, e.g. `"2024"`
#'   or `"latest"`. Treasury publishes one TEIS per calendar year.
#'
#' @return An `ato_tbl` with one row per tax expenditure: label,
#'   category, estimated revenue forgone in AUD millions by year.
#'
#' @source Treasury Tax Expenditures and Insights Statement
#'   \url{https://treasury.gov.au/publication/p2025-721342}.
#'
#' @references
#' Commonwealth of Australia (annual). \emph{Tax Expenditures
#'   and Insights Statement}. The Treasury, Canberra.
#'   \url{https://treasury.gov.au/publication/p2025-721342}
#'
#' @family specialist
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try(head(ato_tax_expenditures("latest")))
#' options(op)
#' }
ato_tax_expenditures <- function(year = "latest") {
  results <- tryCatch(
    ato_ckan_search(q = "tax-expenditure", rows = 20L),
    error = function(e) list(results = list())
  )
  hit <- NULL
  if (!is.null(results$results) && length(results$results) > 0L) {
    for (p in results$results) {
      name <- p$name %||% ""
      if (grepl("tax.expenditure", name, ignore.case = TRUE)) {
        hit <- p
        break
      }
    }
  }
  if (is.null(hit)) {
    cli::cli_abort(c(
      "TEIS not found on data.gov.au.",
      "i" = "Fetch manually from {.url https://treasury.gov.au/publication/p2025-721342}",
      "i" = "then pass the URL to {.code ato_download()}."
    ))
  }
  res <- hit$resources[[1L]]
  url <- res$url %||% ""
  df <- if (grepl("\\.csv$", url, ignore.case = TRUE)) {
    ato_fetch_csv(url)
  } else {
    ato_fetch_xlsx(url, sheet = 1)
  }
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 4.0",
              title = paste0("Treasury TEIS ", year))
}

# SHA-256 integrity: every cached file has a sidecar `.sha256` file
# recording the hash at first download. Subsequent cache hits verify
# against the sidecar and warn on drift.

#' Compute the SHA-256 digest of a file
#'
#' Wraps [tools::md5sum()] style behaviour for SHA-256 via the
#' `digest` package when available, or falls back to a pure-R
#' implementation via [tools::md5sum()] + file length as a weaker
#' check. For integrity work PBO/Grattan-grade, install the
#' `digest` package (Suggests).
#'
#' @param file Path to a local file.
#' @return A length-1 character string (hex digest), or `NA` if the
#'   file does not exist.
#' @family reproducibility
#' @export
#' @examples
#' f <- tempfile()
#' writeLines("hello", f)
#' ato_sha256(f)
ato_sha256 <- function(file) {
  if (!file.exists(file)) return(NA_character_)
  if (requireNamespace("digest", quietly = TRUE)) {
    return(digest::digest(file = file, algo = "sha256"))
  }
  # Fallback: md5 (weaker but still ships with base R via tools).
  unname(tools::md5sum(file))
}

#' @noRd
ato_sha_sidecar <- function(file) {
  paste0(file, ".sha256")
}

#' Write the SHA-256 of a file to its sidecar
#' @noRd
ato_sha_write <- function(file) {
  sha <- ato_sha256(file)
  if (is.na(sha)) return(invisible(NA_character_))
  writeLines(sha, ato_sha_sidecar(file))
  invisible(sha)
}

#' Read the cached SHA-256 sidecar for a file
#' @noRd
ato_sha_read <- function(file) {
  side <- ato_sha_sidecar(file)
  if (!file.exists(side)) return(NA_character_)
  tryCatch(readLines(side, n = 1L, warn = FALSE),
           error = function(e) NA_character_)
}

#' Verify a cached file against its sidecar SHA; warn on mismatch.
#' @noRd
ato_sha_verify <- function(file) {
  expected <- ato_sha_read(file)
  if (is.na(expected) || !nzchar(expected)) {
    # No sidecar yet, write one.
    ato_sha_write(file)
    return(invisible(TRUE))
  }
  actual <- ato_sha256(file)
  if (!identical(expected, actual)) {
    cli::cli_warn(c(
      "SHA-256 mismatch for cached file {.path {basename(file)}}.",
      "i" = "Expected {.val {substr(expected, 1, 12)}...}",
      "i" = "Got      {.val {substr(actual, 1, 12)}...}",
      "i" = "Run {.code ato_clear_cache()} and re-fetch to resolve."
    ))
    return(invisible(FALSE))
  }
  invisible(TRUE)
}

test_that("ato_sha256 returns a hex string for a real file", {
  f <- tempfile()
  writeLines("hello, world", f)
  sha <- ato_sha256(f)
  expect_type(sha, "character")
  expect_true(nzchar(sha))
})

test_that("ato_sha256 returns NA for missing file", {
  expect_true(is.na(ato_sha256(tempfile())))
})

test_that("sha write/read roundtrip via internal helpers", {
  f <- tempfile()
  writeLines("abc", f)
  ato_ns <- asNamespace("ato")
  ato_ns$ato_sha_write(f)
  expect_true(file.exists(ato_ns$ato_sha_sidecar(f)))
  sha_read <- ato_ns$ato_sha_read(f)
  sha_direct <- ato_sha256(f)
  expect_equal(sha_read, sha_direct)
})

test_that("ato_sha_verify fires a warning on mismatch", {
  f <- tempfile()
  writeLines("original", f)
  ato_ns <- asNamespace("ato")
  ato_ns$ato_sha_write(f)
  writeLines("changed", f)
  expect_warning(ato_ns$ato_sha_verify(f), "SHA-256 mismatch")
})

# Level 2: known-value test against a published SHA-256 vector.
# SHA-256 of the empty string is defined in NIST FIPS 180-4 as
# e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855.
test_that("ato_sha256 matches NIST FIPS 180-4 empty-string vector", {
  skip_if_not_installed("digest")
  f <- tempfile()
  file.create(f)
  expect_equal(
    ato_sha256(f),
    "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
  )
})

# Level 4: cross-implementation against the openssl package.
test_that("ato_sha256 agrees with openssl::sha256", {
  skip_if_not_installed("openssl")
  skip_if_not_installed("digest")
  f <- tempfile()
  writeLines("reproducibility audit", f)
  ours   <- ato_sha256(f)
  theirs <- paste(as.character(openssl::sha256(file(f))), collapse = "")
  expect_equal(tolower(ours), tolower(theirs))
})

# Voluntary Tax Transparency Code

#' Voluntary Tax Transparency Code disclosures
#'
#' Returns the ATO's Voluntary Tax Transparency Code (VTTC)
#' disclosures: large private companies that voluntarily publish
#' tax information beyond the Corporate Tax Transparency mandate.
#' Covers total income, taxable income, tax payable, and effective
#' tax rate for each disclosing entity.
#'
#' The VTTC complements [ato_top_taxpayers()] (which covers
#' mandatory CTT disclosures for entities above AUD 100m total
#' income). VTTC signatories may be below or above the CTT
#' threshold.
#'
#' Licensed under **CC BY 3.0 Australia** (same as CTT data).
#'
#' @param year Income year in `"YYYY-YY"` form (e.g. `"2022-23"`)
#'   or `"latest"`.
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD.
#'
#' @source Australian Taxation Office Voluntary Tax Transparency
#'   Code disclosures on data.gov.au. Licensed CC BY 3.0 AU.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   vttc <- ato_vttc(year = "2022-23")
#'   head(vttc)
#' })
#' options(op)
#' }
ato_vttc <- function(year = "latest") {
  # Try the constant slug first, then search as fallback.
  pkg_id <- tryCatch({
    ato_ckan_package(ATO_PACKAGE_IDS$vttc)
    ATO_PACKAGE_IDS$vttc
  }, error = function(e) {
    result   <- ato_ckan_search(q = "voluntary tax transparency", rows = 10L)
    pkgs     <- result$results
    ids      <- vapply(pkgs, function(p) p$name %||% "", character(1))
    vttc_ids <- ids[grepl("voluntary.tax.transparency|vttc", ids,
                            ignore.case = TRUE)]
    if (length(vttc_ids) == 0L) {
      cli::cli_abort(c(
        "Could not find a Voluntary Tax Transparency Code package.",
        "i" = "Browse: {.url https://data.gov.au/data/organization/australiantaxationoffice}"
      ))
    }
    vttc_ids[1L]
  })

  ato_check_staleness(pkg_id)

  if (identical(year, "latest")) {
    pkg  <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year  <- if (length(years) > 0L) max(years) else "2022-23"
  } else {
    year <- ato_resolve_year(year)
  }

  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 3.0 AU",
              title   = paste0("ATO VTTC disclosures ", year))
}

# International Related Party Dealings

#' International Related Party Dealings (IRPD)
#'
#' Returns the ATO's International Related Party Dealings data,
#' which captures intra-group cross-border payments and
#' receivables reported by Australian corporate taxpayers. Core
#' dataset for BEPS and transfer-pricing research, transfer
#' pricing risk assessment, and multinational tax analysis.
#'
#' The IRPD data is published as a separate CKAN package per
#' income year (2019-20 through 2023-24). Each annual package
#' contains three tables:
#' - **Table 1** : IRPD totals from 2015-16 to the current year
#' - **Table 2** : IRPDs by jurisdiction
#' - **Table 3** : Index of chart data
#'
#' @param year Income year in `"YYYY-YY"` form (e.g.
#'   `"2023-24"`) or `"latest"`.
#' @param table Integer 1, 2, or 3. Default `1`.
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD.
#'
#' @source Australian Taxation Office International Related
#'   Party Dealings release. Licensed CC BY 2.5 AU.
#'
#' @references
#' Organisation for Economic Co-operation and Development (2015).
#'   \emph{Transfer Pricing Documentation and Country-by-Country
#'   Reporting, Action 13: 2015 Final Report}. OECD/G20 Base
#'   Erosion and Profit Shifting Project, Paris.
#'   \doi{10.1787/9789264241480-en}
#'
#' Commonwealth of Australia. \emph{Income Tax Assessment Act
#'   1997}, Subdivision 815-B (Transfer Pricing); \emph{Multinational
#'   Anti-Avoidance Law} (MAAL) and \emph{Diverted Profits Tax}.
#'
#' Australian Taxation Office (annual). \emph{International
#'   Dealings Schedule (IDS) instructions}. Reporting framework
#'   underlying the IRPD dataset.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   by_jurisdiction <- ato_irpd(year = "2023-24", table = 2)
#'   head(by_jurisdiction)
#' })
#' options(op)
#' }
ato_irpd <- function(year = "latest", table = 1L) {
  stopifnot(table %in% c(1L, 2L, 3L))

  if (identical(year, "latest")) {
    # Search all IRPD packages for the most recent year
    result <- ato_ckan_search(q = "international-related-party-dealings",
                              rows = 20L)
    ids <- vapply(result$results,
                  function(p) p$name %||% "", character(1))
    years <- regmatches(ids, regexpr("20[0-9]{2}-[0-9]{2}$", ids))
    year <- if (length(years) > 0L) max(years) else "2023-24"
  } else {
    year <- ato_resolve_year(year)
  }

  # The 2019-20 release uses the bare slug (no suffix); later
  # releases append the year. Try suffixed form first, then bare.
  pkg_id <- paste0(ATO_PACKAGE_IDS$irpd_bare, "-", year)
  ato_check_staleness(pkg_id)
  res <- tryCatch(
    ato_ckan_resolve(pkg_id, sprintf("table[_-]?%d", table)),
    error = function(e) {
      ato_ckan_resolve(ATO_PACKAGE_IDS$irpd_bare,
                       sprintf("table[_-]?%d", table))
    }
  )
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO IRPD ", year, " (Table ", table, ")"))
}

# Fringe Benefits Tax

#' Fringe Benefits Tax statistics
#'
#' Returns the ATO's annual Fringe Benefits Tax (FBT) Taxation
#' Statistics: employer counts, gross taxable value, FBT payable,
#' and employee benefit counts by benefit type and industry. Used
#' by Treasury, PBO, and researchers evaluating the FBT concession
#' system (electric vehicles, remote area exemptions, novated
#' leases).
#'
#' @param year Income year in `"YYYY-YY"` form (e.g. `"2022-23"`)
#'   or `"latest"`.
#'
#' @return An `ato_tbl`. Monetary values in nominal AUD.
#'
#' @source Australian Taxation Office FBT Taxation Statistics
#'   on data.gov.au. Licensed CC BY 2.5 AU.
#'
#' @references
#' Commonwealth of Australia. \emph{Fringe Benefits Tax Assessment
#'   Act 1986}. Substantive FBT law; ATO rulings (TR series)
#'   elaborate taxable-value methodology.
#'
#' Australian Taxation Office (annual). \emph{FBT explanatory
#'   notes}. Definitions of reportable benefits, gross-up factors
#'   (Type 1 and Type 2), and otherwise-deductible rule.
#'
#' Treasury (2022). \emph{Electric Car Discount Bill}.
#'   Explanatory memorandum for the EV FBT exemption introduced
#'   1 July 2022.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   fbt <- ato_fbt(year = "2022-23")
#'   head(fbt)
#' })
#' options(op)
#' }
ato_fbt <- function(year = "latest") {
  # Discover the FBT package by searching the catalogue.
  result  <- ato_ckan_search(q = "fringe-benefits-tax", rows = 10L)
  pkgs    <- result$results
  ids     <- vapply(pkgs, function(p) p$name %||% "", character(1))
  fbt_ids <- ids[grepl("fringe.benefits", ids, ignore.case = TRUE)]

  if (length(fbt_ids) == 0L) {
    cli::cli_abort(c(
      "Could not find a Fringe Benefits Tax package on data.gov.au.",
      "i" = "Browse: {.url https://data.gov.au/data/organization/australiantaxationoffice}"
    ))
  }
  pkg_id <- fbt_ids[1L]
  ato_check_staleness(pkg_id)

  if (identical(year, "latest")) {
    pkg  <- ato_ckan_package(pkg_id)
    urls <- vapply(pkg$resources %||% list(),
                   function(r) r$url %||% "", character(1))
    years <- regmatches(urls, regexpr("20[0-9]{2}-[0-9]{2}", urls))
    year  <- if (length(years) > 0L) max(years) else "2022-23"
  } else {
    year <- ato_resolve_year(year)
  }

  res <- ato_ckan_resolve(pkg_id, year)
  url <- res$url %||% ""
  df  <- ato_fetch_xlsx(url, sheet = 1)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source  = url,
              licence = "CC BY 2.5 AU",
              title   = paste0("ATO FBT Taxation Statistics ", year))
}

# Catalogue discovery + generic download

#' ATO dataset catalogue
#'
#' Returns a summary of all datasets published by the Australian
#' Taxation Office on data.gov.au. Each row is a CKAN "package"
#' with an id (slug), title, licence, modification date, and
#' resource count.
#'
#' @param q Optional free-text filter (CKAN Solr query). `NULL`
#'   returns the full ATO catalogue.
#'
#' @return An `ato_tbl` with one row per dataset.
#'
#' @source 'data.gov.au' CKAN endpoint
#'   <https://data.gov.au/data/organization/australiantaxationoffice>.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   cat <- ato_catalog()
#'   head(cat[, c("id", "title", "licence")])
#' })
#' options(op)
#' }
ato_catalog <- function(q = NULL) {
  result <- ato_ckan_search(q = q, rows = 200L)
  pkgs <- result$results
  df <- data.frame(
    id = vapply(pkgs, function(p) p$name %||% "", character(1)),
    title = vapply(pkgs, function(p) p$title %||% "", character(1)),
    notes = vapply(pkgs, function(p) substr(p$notes %||% "", 1L, 200L), character(1)),
    licence = vapply(pkgs, function(p) p$license_title %||% p$license_id %||% "", character(1)),
    n_resources = vapply(pkgs, function(p) length(p$resources %||% list()), integer(1)),
    modified = vapply(pkgs, function(p) p$metadata_modified %||% "", character(1)),
    stringsAsFactors = FALSE
  )
  df <- df[order(df$id), , drop = FALSE]
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = "https://data.gov.au/data/organization/australiantaxationoffice",
              licence = "CC BY 2.5 AU / 3.0 AU",
              title = "ATO data.gov.au catalogue")
}

#' Download a resource from an ATO dataset
#'
#' Low-level helper for arbitrary CKAN resources. Resolves the
#' package by id (slug) and picks the first resource matching
#' `pattern`, or the first resource if `pattern` is `NULL`.
#'
#' @param id CKAN package id (e.g. `"taxation-statistics-2022-23"`
#'   or `"corporate-transparency"`).
#' @param pattern Optional regex applied to the resource filename
#'   and name.
#' @param parse One of `"auto"` (default), `"csv"`, `"xlsx"`, or
#'   `"none"` (returns the cached file path).
#' @param sheet For XLSX resources: sheet index or name.
#'
#' @return Either a file path (`parse = "none"`) or an `ato_tbl`.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   cat <- ato_download("corporate-transparency",
#'                       pattern = "2023",
#'                       parse = "csv")
#' })
#' options(op)
#' }
ato_download <- function(id, pattern = NULL,
                         parse = c("auto", "csv", "xlsx", "none"),
                         sheet = 1) {
  parse <- match.arg(parse)
  res <- if (is.null(pattern)) {
    pkg <- ato_ckan_package(id)
    if (length(pkg$resources) == 0L) {
      cli::cli_abort("Package {.val {id}} has no resources.")
    }
    pkg$resources[[1L]]
  } else {
    ato_ckan_resolve(id, pattern)
  }

  url <- res$url %||% ""
  if (!nzchar(url)) cli::cli_abort("Resource has no URL.")

  format <- if (parse == "auto") {
    ext <- tolower(tools::file_ext(url))
    if (ext %in% c("csv", "tsv")) "csv" else if (ext %in% c("xls", "xlsx", "xlsm")) "xlsx" else "none"
  } else parse

  if (format == "none") return(ato_download_cached(url))

  df <- if (format == "csv") ato_fetch_csv(url) else ato_fetch_xlsx(url, sheet = sheet)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = res$license_id %||% "CC BY 2.5 AU",
              title = res$name %||% id)
}

# Australian Tax Gaps

#' Australian tax gaps estimates
#'
#' Returns the ATO's annual Tax Gap publication: estimates of
#' the difference between the tax theoretically payable under
#' current law and the tax actually collected, across each tax
#' type and taxpayer population (individuals not in business,
#' small business, large corporate, GST, excise, fuel tax
#' credits, PRRT, superannuation guarantee).
#'
#' The Tax Gap series is used by Treasury (every MYEFO), the
#' Parliamentary Budget Office, and academic researchers as the
#' headline measure of revenue integrity.
#'
#' @param sheet Optional sheet name or index. The workbook
#'   contains separate sheets for each tax-gap population (e.g.
#'   "Large corporate", "Small business", "Individuals"). Pass
#'   the sheet name to extract a specific population. `NULL`
#'   (default) returns sheet 1 (overview).
#'
#' @return An `ato_tbl`. Tax-gap estimates are in nominal AUD
#'   millions of the reporting year and typically accompanied by
#'   a percentage-gap column.
#'
#' @source Australian Taxation Office Tax Gaps publication,
#'   CC BY 2.5 AU.
#'
#' @references
#' Australian Taxation Office (annual). \emph{Australian tax gaps
#'   -- overview}. Methodology notes on bottom-up, top-down, and
#'   random-inquiry approaches to the tax-gap estimation.
#'
#' HMRC (annual). \emph{Measuring tax gaps}. Sister methodology
#'   paper applied by HM Revenue and Customs in the UK; the ATO
#'   series was partly inspired by this literature.
#'
#' Organisation for Economic Co-operation and Development (2017).
#'   \emph{Shining Light on the Shadow Economy: Opportunities and
#'   Threats}. Paris. Synthesises tax-gap measurement practice
#'   across OECD member countries.
#'
#' @family discovery
#' @export
#' @examples
#' \donttest{
#' op <- options(ato.cache_dir = tempdir())
#' try({
#'   gaps <- ato_tax_gaps()
#'   head(gaps)
#' })
#' options(op)
#' }
ato_tax_gaps <- function(sheet = 1) {
  ato_check_staleness(ATO_PACKAGE_IDS$tax_gaps)
  res <- ato_ckan_resolve(ATO_PACKAGE_IDS$tax_gaps,
                          "australian-tax-gaps.*publication|tax-gaps")
  url <- res$url %||% ""
  df <- ato_fetch_xlsx(url, sheet = sheet)
  rownames(df) <- NULL
  new_ato_tbl(df,
              source = url,
              licence = "CC BY 2.5 AU",
              title = paste0("ATO Tax Gaps (sheet: ", sheet, ")"))
}

test_that("ato_manifest_clear empties the manifest", {
  ato_manifest_clear()
  df <- ato_manifest("df")
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 0L)
})

test_that("ato_manifest df has expected columns", {
  ato_manifest_clear()
  df <- ato_manifest("df")
  expect_true(all(c("url", "title", "resource_id", "package_id",
                    "licence", "sha256", "size_bytes", "retrieved",
                    "snapshot_date", "r_version", "ato_version") %in%
                    names(df)))
})

test_that("ato_manifest json output is valid JSON", {
  ato_manifest_clear()
  j <- ato_manifest("json")
  expect_type(j, "character")
  expect_silent(jsonlite::fromJSON(j))
})

test_that("ato_manifest_write writes to disk", {
  ato_manifest_clear()
  p <- tempfile(fileext = ".csv")
  ato_manifest_write(p)
  expect_true(file.exists(p))
  df <- utils::read.csv(p)
  expect_s3_class(df, "data.frame")
})

test_that("manifest records a fetch when .ato_manifest_append is called", {
  ato_manifest_clear()
  f <- tempfile()
  writeLines("test", f)
  ato_ns <- asNamespace("ato")
  ato_ns$.ato_manifest_append(
    url = "https://example.com/x.csv",
    file = f,
    title = "test dataset"
  )
  df <- ato_manifest("df")
  expect_equal(nrow(df), 1L)
  expect_equal(df$title, "test dataset")
  expect_true(nzchar(df$sha256))
})

test_that("ato_cite includes SHA and snapshot in bibtex", {
  df <- data.frame(a = 1)
  x <- structure(df,
    ato_source = "https://data.gov.au/x.csv",
    ato_licence = "CC BY 2.5 AU",
    ato_retrieved = as.POSIXct("2026-04-24 00:00:00", tz = "UTC"),
    ato_title = "ATO test dataset",
    ato_sha256 = "abc123def456789012345678",
    ato_snapshot_date = "2026-04-24",
    class = c("ato_tbl", "data.frame")
  )
  bib <- ato_cite(x, style = "bibtex")
  expect_match(bib, "snapshot 2026-04-24")
  expect_match(bib, "sha256:abc123def456")
})

test_that("ato_cite includes DOI when provided", {
  df <- data.frame(a = 1)
  x <- structure(df,
    ato_source = "https://data.gov.au/x.csv",
    ato_title = "t",
    ato_retrieved = Sys.time(),
    class = c("ato_tbl", "data.frame")
  )
  bib <- ato_cite(x, style = "bibtex", doi = "10.5281/zenodo.1")
  expect_match(bib, "doi    = \\{10\\.5281/zenodo\\.1\\}")

  apa <- ato_cite(x, style = "apa", doi = "10.5281/zenodo.1")
  expect_match(apa, "https://doi.org/10.5281/zenodo.1")
})

test_that("ato_cite text includes snapshot and SHA", {
  df <- data.frame(a = 1)
  x <- structure(df,
    ato_source = "https://x", ato_title = "t",
    ato_retrieved = as.POSIXct("2026-04-24", tz = "UTC"),
    ato_sha256 = "abcdef123456789",
    ato_snapshot_date = "2026-04-24",
    class = c("ato_tbl", "data.frame")
  )
  txt <- ato_cite(x, style = "text")
  expect_match(txt, "Snapshot: 2026-04-24")
  expect_match(txt, "SHA-256")
})

test_that("ato_cite handles ato_tbl input in all styles", {
  x <- data.frame(a = 1)
  x <- structure(x,
    ato_source = "https://data.gov.au/data/dataset/example.xlsx",
    ato_licence = "CC BY 2.5 AU",
    ato_retrieved = as.POSIXct("2026-04-23 00:00:00", tz = "UTC"),
    ato_title = "ATO individuals 2022-23",
    class = c("ato_tbl", "data.frame"))

  txt <- ato_cite(x, "text")
  expect_match(txt, "Australian Taxation Office")
  expect_match(txt, "2026-04-23")
  expect_match(txt, "CC BY 2.5 AU")

  bib <- ato_cite(x, "bibtex")
  expect_match(bib, "^@misc\\{ato_")
  expect_match(bib, "Australian Taxation Office")

  apa <- ato_cite(x, "apa")
  expect_match(apa, "^Australian Taxation Office\\. \\(2026\\)")
})

test_that("ato_cite accepts a plain URL", {
  txt <- ato_cite("https://data.gov.au/data/foo.xlsx")
  expect_match(txt, "Australian Taxation Office")
  expect_match(txt, "foo.xlsx")
})

test_that("ato_cite rejects other inputs", {
  expect_error(ato_cite(123))
  expect_error(ato_cite(list()))
})

---
title: "Interop with grattan and taxstats: costing a hypothetical reform"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Interop with grattan and taxstats: costing a hypothetical reform}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

`ato` sits in a three-package constellation for Australian tax
research:

- `ato` : published aggregate data from ATO data.gov.au releases
- `taxstats` / `taxstats2` (Hugh Parsonage, DRAT) : 2 per cent
  individual microdata sample
- `grattan` (Hugh Parsonage, CRAN) : policy calculator
  (brackets, Medicare levy, LITO, LMITO, HELP repayments)

`ato` provides `ato_to_taxstats()` to rename aggregate columns to
the microdata schema, so analyses can move cleanly between views.

## Schema map

```{r}
library(ato)
head(ato_schema_map(), 15)
```

## Costing a hypothetical reform

Suppose we want a first-pass cost of lifting the Medicare Levy
Surcharge income threshold by AUD 10,000. Workflow:

1. Get MLS taxpayer counts and amounts by income band from ATO
   aggregates.
2. Check the total matches Budget Paper 1.
3. Hand off to `taxstats` 2 per cent sample for counterfactual
   microsimulation.
4. Use `grattan::income_tax()` for the new-regime liability.

```{r}
library(ato)

ato_snapshot("2026-04-24")

mls <- ato_medicare_levy(year = "2022-23", component = "surcharge")

# Reconcile the published MLS total against FBO where available
# (MLS does not have a direct FBO line; rolled into individuals
# income tax net).
ind_total <- sum(ato_individuals(year = "2022-23")$tax_payable,
                 na.rm = TRUE)
ato_reconcile(ind_total, "2022-23", "individuals_income_tax_net")
```

## Bridge to microdata

```{r}
# Rename columns to taxstats schema
mls_ts <- ato_to_taxstats(mls)

# Now these columns are consistent with what `taxstats::taxstats1819`
# uses. You can write analysis code that works on both.

# Example: use grattan to compute the new regime's tax for the 2% sample
# library(taxstats)
# library(grattan)
# sample_2pc <- taxstats1819
# sample_2pc$new_tax <- income_tax(
#   income = sample_2pc$Taxable_Income,
#   fy.year = "2023-24",
#   ...
# )
# reform_cost <- sum(sample_2pc$new_tax - sample_2pc$Tax_assessed_amt,
#                    na.rm = TRUE) * 50  # 2% -> 100%
```

## Summary

The three-package stack covers the full tax-research pipeline:
`ato` for authoritative aggregates and provenance, `taxstats` for
distributional microdata, `grattan` for policy simulation. Use
`ato_reconcile()` at the boundary to confirm the microdata
estimate lines up with the population total.

---
title: "Panels, harmonisation, reconciliation, real terms, per-capita"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Panels, harmonisation, reconciliation, real terms, per-capita}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

This vignette walks through the four panel-ready transformations
that take raw ATO fetches to a defensible longitudinal analysis:

1. Stack multiple years with `year =` vector input.
2. Harmonise column names across releases with `ato_harmonise()`.
3. Reconcile totals against Final Budget Outcome with
   `ato_reconcile()`.
4. Express in real terms and per capita with `ato_deflate()` and
   `ato_per_capita()`.

## Build a multi-year panel

```{r}
library(ato)

pc <- ato_individuals_postcode(
  year = c("2018-19", "2019-20", "2020-21",
           "2021-22", "2022-23"),
  state = "NSW"
)

nrow(pc)
unique(pc$year)
```

## Harmonise column names

Column names drift: `total_income` in some years, `total_income_or_loss`
in others; `state` vs `state_territory`. `ato_harmonise()` renames
columns to canonical names from `ATO_COL_VARIANTS`.

```{r}
pc <- ato_harmonise(pc)
names(pc)
```

## Reconcile against Commonwealth totals

Before reporting a panel sum in a paper, check it against the Final
Budget Outcome. A 1-3 per cent accrual-vs-cash gap is expected;
larger gaps warrant investigation.

```{r}
ind_2223 <- ato_individuals(year = "2022-23")
total_tax <- sum(ind_2223$tax_payable, na.rm = TRUE)

ato_reconcile(
  value   = total_tax,
  year    = "2022-23",
  measure = "individuals_income_tax_net"
)
```

## Real-terms comparison

ATO values are nominal AUD of the reporting year. For time-series
comparison, deflate to a common base year using the bundled ABS
CPI series.

```{r}
panel_annual <- aggregate(taxable_income ~ year, data = pc, FUN = sum,
                          na.rm = TRUE)
panel_annual$real_2022_23 <- ato_deflate(
  panel_annual$taxable_income,
  year = panel_annual$year,
  base = "2022-23"
)
panel_annual
```

## Per-capita normalisation

```{r}
panel_annual$per_capita <- ato_per_capita(
  panel_annual$real_2022_23,
  year = panel_annual$year
)
panel_annual
```

The resulting four-column data frame (year, nominal, real, per
capita) is the canonical shape for distributional and time-series
tax papers.

---
title: "Methodology: hand-computed worked examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Methodology: hand-computed worked examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ato)
```

Every formula function in `ato` is verifiable with pen and paper.
This vignette walks through the three arithmetic methods and
confirms each against a hand calculation. A referee checking a
published result against the package can use this page as the
reference implementation.

## 1. `ato_deflate()`

### Formula

Proportional (Laspeyres-style) adjustment:

\[
\mathrm{real}_t \;=\; \mathrm{nominal}_t \times \frac{CPI_{\text{base}}}{CPI_t}
\]

Source: Diewert (1998), \doi{10.1257/jep.12.1.47}; ABS cat. 6461.0,
Concepts, Sources and Methods.

### Hand calculation

Take AUD 100 in 2012-13 nominal and deflate to 2022-23 real AUD.

From `ato_crosswalk("cpi")`:

```{r}
cpi <- ato_crosswalk("cpi")
cpi[cpi$financial_year %in% c("2012-13", "2022-23"), ]
```

Reading off the table:
- \(CPI_{2012\text{-}13} = 101.7\)
- \(CPI_{2022\text{-}23} = 132.9\)

Hand calculation:

\[
100 \times \frac{132.9}{101.7} \;=\; 100 \times 1.30678 \;=\; 130.68
\]

Package output:

```{r}
ato_deflate(100, year = "2012-13", base = "2022-23")
```

The package returns exactly 130.6785 (to four decimals), which
matches the hand calculation to within rounding.

### Sanity: identity at the base year

```{r}
ato_deflate(100, year = "2022-23", base = "2022-23")
```

As expected: when the source year equals the base year the index
ratio is 1 and the real-term value equals the nominal value.

## 2. `ato_reconcile()`

### Formula

\[
\Delta \;=\; V - R, \qquad \pi \;=\; \frac{V - R}{R}
\]

where \(V\) is the value from Taxation Statistics and \(R\) is the
Final Budget Outcome reference total for the same year and
measure.

### Hand calculation

From `ato_crosswalk("budget")`:

```{r}
bud <- ato_crosswalk("budget")
bud[bud$financial_year == "2022-23" &
    bud$measure == "individuals_income_tax_net", ]
```

Reference: \(R = 316.4 \times 10^9\) AUD.

Suppose we fetch the ATO 2022-23 Individuals snapshot and sum the
`tax_payable` column. Say our sum is \(V = 310.0 \times 10^9\)
(hypothetical; real sums typically fall 1 to 3 per cent below the
cash-basis FBO).

Hand:

\[
\Delta = 310.0 - 316.4 = -6.4 \text{ (AUD billion)}
\]

\[
\pi = \frac{-6.4}{316.4} = -0.02023 = -2.02\%
\]

Package output:

```{r}
res <- ato_reconcile(310.0e9, "2022-23", "individuals_income_tax_net")
res[, c("diff_aud", "pct_diff")]
```

Agrees with hand calculation.

### Warning threshold

The package warns when `abs(pct_diff) > 0.05`. Test:

```{r}
res <- tryCatch(
  ato_reconcile(400e9, "2022-23", "individuals_income_tax_net"),
  warning = function(w) {
    message("Captured warning: ", conditionMessage(w))
    NULL
  }
)
```

At \(V = 400 \text{ AUD billion}\), the diff is +83.6 billion and
\(\pi = 26.4\%\) which exceeds the 5% threshold, so the warning
fires.

## 3. `ato_per_capita()`

### Formula

\[
\mathrm{per\_capita}_t \;=\; \frac{x_t}{\mathrm{ERP}_t}
\]

where \(\mathrm{ERP}_t\) is the ABS Estimated Resident Population
at 30 June of the financial year end (cat. 3101.0).

### Hand calculation

From `ato_crosswalk("erp")`:

```{r}
erp <- ato_crosswalk("erp")
erp[erp$financial_year == "2022-23", ]
```

ERP 2022-23 = 26,638 thousand = 26,638,000 persons.

Commonwealth individual income tax for 2022-23 = AUD 316.4 billion
(from `ato_reconcile()` reference).

Hand:

\[
\frac{316.4 \times 10^9}{26.638 \times 10^6} \;=\; 11{,}878 \text{ AUD per person}
\]

Package output:

```{r}
ato_per_capita(316.4e9, "2022-23")
```

Agrees.

## 4. `ato_sha256()`

### Specification

SHA-256 as defined in NIST FIPS 180-4. For the empty string, the
published digest is:

```
e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855
```

### Verification

```{r}
f <- tempfile()
file.create(f)
ato_sha256(f)
```

Matches the NIST FIPS 180-4 reference digest exactly. The
package-level test suite also cross-checks against
`openssl::sha256` for further independence.

## Summary

| Function | Formula | Reference | Status |
|---|---|---|---|
| `ato_deflate` | Laspeyres ratio | Diewert 1998 + ABS 6461.0 | Verified |
| `ato_reconcile` | (V - R) / R | FBO + ATO tax gap methodology | Verified |
| `ato_per_capita` | x / ERP | ABS 3101.0 | Verified |
| `ato_sha256` | NIST FIPS 180-4 | `digest` + `openssl` | Verified |

A reviewer can reproduce each worked example on paper and confirm
the package output. Any disagreement is a bug in the package;
please file an issue.

---
title: "Canonical replications: top 1% share, corporate ETR, tax gap trend"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Canonical replications: top 1% share, corporate ETR, tax gap trend}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

Three canonical tax-research exercises, each in around 20 lines.

## 1. Top 1 per cent income share (Atkinson-Leigh style)

The approach below loosely mirrors Atkinson and Leigh (2007),
"The Distribution of Top Incomes in Australia," *Economic Record*,
83(262), 247-261 (<https://doi.org/10.1111/j.1475-4932.2007.00412.x>).
The Atkinson-Leigh method reconstructs top-income shares from ATO
published tabulations; a postcode-based approximation is a cruder
but data-only proxy. For rigorous top-share work use Pareto
interpolation on the ATO top-percentile table or apply for ALife
microdata access. See also Burkhauser, Hahn and Wilkins (2015)
for caveats (<https://doi.org/10.1007/s10888-014-9281-z>).

```{r}
library(ato)
ato_snapshot("2026-04-24")

pc_panel <- ato_individuals_postcode(
  year = c("2015-16", "2016-17", "2017-18", "2018-19",
           "2019-20", "2020-21", "2021-22", "2022-23")
)
pc_panel <- ato_harmonise(pc_panel)

# For each year, rank postcodes by mean taxable income per return,
# take top 1% of returns, compute their share of total income.
top1 <- function(df) {
  df <- df[order(-df$taxable_income / df$number_of_individuals), ]
  cum_returns <- cumsum(df$number_of_individuals)
  total_returns <- sum(df$number_of_individuals, na.rm = TRUE)
  cutoff <- which(cum_returns >= 0.01 * total_returns)[1]
  sum(df$taxable_income[seq_len(cutoff)], na.rm = TRUE) /
    sum(df$taxable_income, na.rm = TRUE)
}

shares <- by(pc_panel, pc_panel$year, top1)
shares
```

## 2. Corporate effective tax rate by industry (transparency data)

```{r}
ctt <- ato_top_taxpayers(year = "2022-23")

# Effective tax rate = tax payable / taxable income, for entities
# with positive taxable income. Drop zero-taxable rows (they bias
# the ratio; rely on loss-makers analysis separately).
ctt <- ctt[!is.na(ctt$taxable_income) & ctt$taxable_income > 0, ]
ctt$etr <- ctt$tax_payable / ctt$taxable_income

by_industry <- aggregate(etr ~ entity_type, data = ctt, FUN = median)
by_industry[order(-by_industry$etr), ]
```

## 3. Tax gap trend and confidence context

```{r}
tg <- ato_tax_gaps()

library(ggplot2)
ggplot(tg, aes(x = year, y = tax_gap_estimate,
               colour = tax_gap_type)) +
  geom_line() +
  labs(title = "ATO estimated tax gaps over time",
       x = NULL, y = "Estimated tax gap (AUD million)",
       colour = "Gap type",
       caption = "Source: ATO Taxation Statistics. Retrieved via ato package.") +
  theme_minimal()
```

## 4. HELP debt by age cohort

```{r}
help_data <- ato_help()

# Bucketed by age range; real-terms deflation to 2022-23
help_data$real <- ato_deflate(help_data$total_debt,
                               year = help_data$year,
                               base = "2022-23")
head(help_data)
```

Each of these replications takes an ATO published release, a
`harmonise/deflate/reconcile` transformation, and a small
computation. The provenance header (snapshot pin + SHA-256)
makes the result fully auditable.

---
title: "Reproducibility workflow: snapshot, manifest, SHA-256, Zenodo"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reproducibility workflow: snapshot, manifest, SHA-256, Zenodo}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

Published tax research (PBO costings, Grattan reform papers, Tax
Institute briefs) has a reproducibility bar that goes beyond "I
called `ato_individuals()` and summed column X." Reviewers need to
verify that the data you used is exactly the data you say you used.
`ato` provides four features to meet that bar:

1. **Snapshot pin** : declare the intended vintage of the data.
2. **SHA-256 integrity** : every cached file is hashed; drift warns.
3. **Session manifest** : every fetch is recorded with URL, SHA,
   retrieval time, and snapshot pin.
4. **Zenodo DOI** : mint a DOI for the manifest so a paper can cite
   the exact data snapshot.

## Setup

```{r}
library(ato)

ato_snapshot("2026-04-24")
ato_manifest_clear()
```

## Fetch your datasets

```{r}
ind <- ato_individuals_postcode(
  year = c("2020-21", "2021-22", "2022-23"),
  state = "NSW"
)

companies <- ato_companies(year = "2022-23", table = "industry")
tax_gap   <- ato_tax_gaps()
```

Each `ato_tbl` prints with the snapshot pin and SHA-256 digest in
its provenance header.

## Inspect the session manifest

```{r}
man <- ato_manifest()
man[, c("title", "sha256", "retrieved", "snapshot_date")]
```

## Export the manifest for your paper appendix

```{r}
ato_manifest_write("appendix/ato_manifest.csv")
ato_manifest_write("appendix/ato_manifest.yaml")
```

## Mint a DOI via Zenodo

A DOI makes "retrieved from data.gov.au on 2026-04-24" citable and
immutable. Your paper then cites `doi:10.5281/zenodo.XXXXXXXX`
instead of a URL that might rotate.

```{r}
dep <- ato_deposit_zenodo(
  title = "ATO data snapshot for working paper v1",
  creators = list(list(name = "Author, A.", orcid = "0000-0000-0000-0000")),
  upload = FALSE  # dry run; inspect payload first
)
dep$payload$metadata$title

# When ready to actually deposit:
# Sys.setenv(ZENODO_TOKEN = "...your token...")
# dep <- ato_deposit_zenodo(upload = TRUE)
# dep$doi_prereserve
```

## Citing a dataset with full provenance

```{r}
ato_cite(ind, style = "bibtex", doi = "10.5281/zenodo.XXXXXXXX")
```

The BibTeX `note` field includes the snapshot date and first 12
hex characters of the SHA-256. That is the verifiable audit trail
a reviewer would ask for.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflate.R
\name{ato_per_capita}
\alias{ato_per_capita}
\title{Express an aggregate per capita using ABS ERP}
\usage{
ato_per_capita(x, year, erp = NULL)
}
\arguments{
\item{x}{Numeric vector of aggregate values (same length as
\code{year}).}

\item{year}{Character vector of financial years.}

\item{erp}{Optional override: data frame with columns
\code{financial_year} and \code{erp_june_australia_thousands}.}
}
\value{
Numeric vector of per-capita values (same units as \code{x}
per person).
}
\description{
Express an aggregate per capita using ABS ERP
}
\details{
Divides the input by Estimated Resident Population at 30 June
of the financial year's end (a stock measure). For flow-style
measures where a mid-year-average population is preferable,
substitute a custom \verb{erp =} argument. ERP is ABS's preferred
population-denominator concept for per-capita economic
statistics (see cat. 3101.0 methodology).
}
\examples{
# Income tax per person, 2022-23 FBO headline
ato_per_capita(316.4e9, "2022-23")
}
\references{
Australian Bureau of Statistics (2024). \emph{National,
State and Territory Population}. Catalogue 3101.0.
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_deflate}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_schema_map}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/state_tax.R
\name{ato_state_tax}
\alias{ato_state_tax}
\title{State and territory tax revenue (ABS 5506.0)}
\source{
Australian Bureau of Statistics, Taxation Revenue,
catalogue 5506.0 \url{https://www.abs.gov.au/statistics/economy/government/taxation-revenue-australia}.
Licensed CC BY 4.0.
}
\usage{
ato_state_tax(year = "latest")
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Fetches the ABS Taxation Revenue collection (cat. 5506.0),
which gives land tax, payroll tax, stamp duty, motor vehicle
taxes, and other state taxes by jurisdiction. Needed for
complete-tax-system analysis alongside ATO Commonwealth data.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_state_tax(year = "latest"))
options(op)
}
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconcile.R
\name{ato_reconcile}
\alias{ato_reconcile}
\title{Reconcile an aggregate against Commonwealth budget totals}
\usage{
ato_reconcile(value, year, measure, sum_column = NULL)
}
\arguments{
\item{value}{Numeric; the figure to check, in AUD (not AUD
billions). An \code{ato_tbl} can also be passed: pass \code{sum_column}
to pick which numeric column to sum.}

\item{year}{Financial year, e.g. \code{"2022-23"}.}

\item{measure}{One of the measure codes in \code{ato_crosswalk("budget")},
for example \code{"individuals_income_tax_net"},
\code{"company_tax_net"}, \code{"gst_net"}, \code{"fuel_excise_net"}.}

\item{sum_column}{Column name to sum when \code{value} is a data
frame. Default \code{NULL} (errors if multiple numeric columns
exist).}
}
\value{
A one-row data frame: \code{measure}, \code{year},
\code{value_aud}, \code{reference_aud}, \code{diff_aud}, \code{pct_diff},
\code{source}. Emits a warning if \code{abs(pct_diff) > 0.05}.
}
\description{
Compares a scalar (or data frame total) against the published
Final Budget Outcome figure for the same year and revenue line.
Useful as a sanity check on an ATO Taxation Statistics sum
before reporting it in a paper or brief.
}
\details{
Discrepancies between ATO Taxation Statistics aggregates and
the Final Budget Outcome (FBO) are expected and meaningful:
\itemize{
\item Taxation Statistics are based on assessments made by a
cut-off date (usually October of the following calendar
year) and may exclude late-lodging returns.
\item FBO figures are cash-basis Commonwealth receipts;
Taxation Statistics are accrual-basis tax assessed.
\item GST, excise, and fuel credits have timing and refund
effects that further distort the cash-vs-assessment gap.
}
A 1-3 per cent gap is consistent with the accrual-to-cash
reconciliation Treasury publishes in the FBO statement of
revenues; larger gaps warrant investigation. The bundled
reference totals in \code{inst/extdata/budget_reference_totals.csv}
are taken from the relevant FBO release, with the precise table
cited in the \code{source} column of each row.
}
\examples{
ato_reconcile(value = 316.4e9,
              year = "2022-23",
              measure = "individuals_income_tax_net")
}
\references{
Commonwealth of Australia (various years). \emph{Final Budget
Outcome}. The Treasury, Canberra.
\url{https://budget.gov.au/content/fbo/index.htm}

Australian Bureau of Statistics (various years).
\emph{Taxation Revenue, Australia}. Catalogue 5506.0.

Australian Taxation Office (annual). \emph{Australian tax gaps
-- overview}, methodology notes on accrual-vs-cash
reconciliation.
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_deflate}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_schema_map}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/payg.R
\name{ato_payg}
\alias{ato_payg}
\title{PAYG withholding data}
\source{
Australian Taxation Office PAYG withholding data
on data.gov.au. Licensed CC BY 2.5 AU.
}
\usage{
ato_payg(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g. \code{"2022-23"})
or \code{"latest"}.}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD.
}
\description{
Returns the ATO's Pay As You Go (PAYG) withholding data:
employer counts, total withholding amounts, and employee counts
by industry and state. Used by researchers studying labour
market taxation, wage growth, and employer compliance.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  payg <- ato_payg(year = "2022-23")
  head(payg)
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{ato_help}
\alias{ato_help}
\title{Study and Training Support Loan data (HELP, AASL, VSL)}
\source{
Australian Taxation Office Study and Training Support
Loans statistics. Licensed CC BY 2.5 AU.
}
\usage{
ato_help(scheme = c("help", "aasl", "vsl"))
}
\arguments{
\item{scheme}{One of \code{"help"} (default), \code{"aasl"}, or \code{"vsl"}.}
}
\value{
An \code{ato_tbl}. All dollar values in nominal AUD.
}
\description{
Returns aggregate statistics on Australia's three main
education-loan schemes:
\itemize{
\item \strong{HELP} (Higher Education Loan Program, ~3m borrowers,
AUD 80bn+ outstanding debt)
\item \strong{AASL} (Australian Apprenticeship Support Loans,
previously Trade Support Loans)
\item \strong{VSL} (VET Student Loans, vocational education loans)
}
}
\details{
Headline covers: new loans by income range, outstanding debt
by age and gender, repayment rates, median debt on entry.
Used by Treasury (PBO costings of HELP indexation changes)
and education policy researchers.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  help <- ato_help(scheme = "help")
  head(help)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Higher Education Support Act
2003}; \emph{VET Student Loans Act 2016}.

Australian Department of Education (annual). \emph{Higher
Education Statistics: HELP statistics collection}.

Norton, A. and Cherastidtham, I. (2018). \emph{Mapping
Australian higher education}. Grattan Institute.
Methodology reference for HELP repayment projections.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ato_tbl.R
\name{print.ato_tbl}
\alias{print.ato_tbl}
\title{Print an ato_tbl}
\usage{
\method{print}{ato_tbl}(x, ...)
}
\arguments{
\item{x}{An \code{ato_tbl} object.}

\item{...}{Passed to the next print method.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints a provenance header (title, source, licence, retrieval
time, dimensions) followed by the data frame.
}
\examples{
x <- data.frame(postcode = "2000", taxable_income = 82000)
x <- structure(x, ato_title = "Demo", ato_source = "https://data.gov.au",
               ato_licence = "CC BY 2.5 AU", ato_retrieved = Sys.time(),
               class = c("ato_tbl", "data.frame"))
print(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdti.R
\name{ato_rdti}
\alias{ato_rdti}
\title{R&D Tax Incentive claimants}
\source{
Australian Taxation Office Research and Development
Tax Incentive report. Licensed CC BY 2.5 AU.
}
\usage{
ato_rdti(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g.
\code{"2022-23"}) or \code{"latest"}. Current releases cover 2021-22
and 2022-23.}
}
\value{
An \code{ato_tbl} with one row per entity (or aggregated
cell, depending on the release schema). Monetary values in
nominal AUD.
}
\description{
Returns the annual "Report of data about Research and
Development Tax Incentive entities": claimants, claimed
expenditure, refundable and non-refundable tax offsets by
industry and company size. Treasury and DISR use this series
to evaluate the R&D Tax Incentive programme, which is the
largest single element of Australia's business innovation
policy (AUD 2 billion+ per year).
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  rdti <- ato_rdti(year = "2022-23")
  head(rdti)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Income Tax Assessment Act
1997}, Division 355 (Research and Development Tax Incentive).

Department of Industry, Science and Resources and Australian
Taxation Office (annual). \emph{R&DTI Transparency Report}.
Jointly administered programme methodology.

Ferris, B., Finkel, A. and Fraser, J. (2016). \emph{Review of
the R&D Tax Incentive}. Australian Government review
(the "Three Fs review") informing subsequent programme design.

Organisation for Economic Co-operation and Development (annual).
\emph{R&D Tax Incentives Database}. International comparator
data for R&D tax expenditures.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalog.R
\name{ato_download}
\alias{ato_download}
\title{Download a resource from an ATO dataset}
\usage{
ato_download(
  id,
  pattern = NULL,
  parse = c("auto", "csv", "xlsx", "none"),
  sheet = 1
)
}
\arguments{
\item{id}{CKAN package id (e.g. \code{"taxation-statistics-2022-23"}
or \code{"corporate-transparency"}).}

\item{pattern}{Optional regex applied to the resource filename
and name.}

\item{parse}{One of \code{"auto"} (default), \code{"csv"}, \code{"xlsx"}, or
\code{"none"} (returns the cached file path).}

\item{sheet}{For XLSX resources: sheet index or name.}
}
\value{
Either a file path (\code{parse = "none"}) or an \code{ato_tbl}.
}
\description{
Low-level helper for arbitrary CKAN resources. Resolves the
package by id (slug) and picks the first resource matching
\code{pattern}, or the first resource if \code{pattern} is \code{NULL}.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  cat <- ato_download("corporate-transparency",
                      pattern = "2023",
                      parse = "csv")
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compliance.R
\name{ato_compliance}
\alias{ato_compliance}
\title{ATO compliance program outcomes}
\source{
Australian Taxation Office annual report data.
Licensed CC BY 3.0 AU.
}
\usage{
ato_compliance(year = "latest", metric = c("overview", "debt", "audit"))
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{metric}{One of \code{"overview"} (default), \code{"debt"}
(collectable vs insolvency vs disputed), or \code{"audit"}
(liabilities raised by program area).}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns the ATO's annual compliance program outcomes: audit
yield (tax raised from audits), settled disputes, collectable
debt, and compliance cost recovery. These appear in the ATO
annual report and related data.gov.au releases.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_compliance(year = "2022-23", metric = "debt"))
options(op)
}
}
\seealso{
Other specialist: 
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individuals.R
\name{ato_individuals_postcode}
\alias{ato_individuals_postcode}
\title{Individual tax data by postcode}
\source{
Australian Taxation Office Taxation Statistics
postcode release. Licensed CC BY 2.5 AU.
}
\usage{
ato_individuals_postcode(year = "latest", state = NULL, postcode = NULL)
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}. Pass a vector of years
(e.g. \code{c("2020-21", "2021-22", "2022-23")} or \code{2018:2022})
to stack multiple years with a \code{year} column added to the
output. Useful for time-series analysis.}

\item{state}{Optional character vector of state codes (e.g.
\code{"NSW"}, \code{c("VIC", "QLD")}).}

\item{postcode}{Optional character vector of 4-digit postcodes.}
}
\value{
An \code{ato_tbl} with one row per postcode (or per postcode
per year for multi-year queries), including state, return
count, total income, taxable income, and tax payable. Schema
drifts year to year (SA3/SA4 columns present from 2017
onwards).
}
\description{
Returns the Individuals Table 6 (or standalone postcode
dataset): taxable income, tax payable, and return counts by
4-digit postcode. Headline dataset for income-distribution
journalism.
}
\details{
\strong{Privacy suppression.} The ATO suppresses postcodes with
fewer than 50 returns; those cells are returned as \code{NA} after
parsing (the package maps \code{"np"}, \code{"*"}, and similar tokens
to \code{NA} so numeric columns stay numeric). Small or remote
postcodes will be silently missing from the output.

Monetary values are nominal AUD of the reporting year. Use
\code{inflateR::inflate()} for real-term series.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  # Single year
  p <- ato_individuals_postcode(year = "2022-23", state = "NSW")
  head(p)
  # Multi-year stack with year column
  panel <- ato_individuals_postcode(year = c("2020-21", "2021-22"),
                                    state = "NSW")
})
options(op)
}
}
\references{
Atkinson, A.B. and Leigh, A. (2007). "The Distribution of Top
Incomes in Australia." \emph{Economic Record}, 83(262),
247-261. \doi{10.1111/j.1475-4932.2007.00412.x}

Burkhauser, R.V., Hahn, M.H. and Wilkins, R. (2015). "Measuring
top incomes using tax record data: a cautionary tale from
Australia." \emph{Journal of Economic Inequality}, 13(2),
181-205. \doi{10.1007/s10888-014-9281-z}
}
\seealso{
Other individuals: 
\code{\link{ato_individuals}()},
\code{\link{ato_individuals_age}()},
\code{\link{ato_individuals_occupation}()},
\code{\link{ato_individuals_sex}()},
\code{\link{ato_individuals_state}()}
}
\concept{individuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individuals.R
\name{ato_individuals_occupation}
\alias{ato_individuals_occupation}
\title{Individual tax data by occupation}
\source{
Australian Taxation Office Taxation Statistics.
Licensed CC BY 2.5 AU.
}
\usage{
ato_individuals_occupation(
  year = "latest",
  occupation = NULL,
  sex = c("all", "male", "female", "m", "f")
)
}
\arguments{
\item{year}{\code{"YYYY-YY"}, \code{"latest"}, or a vector of years for
a multi-year panel (e.g. \code{c("2020-21", "2021-22", "2022-23")}).}

\item{occupation}{Optional substring filter (case-insensitive)
applied to the occupation description column.}

\item{sex}{One of \code{"all"} (default), \code{"male"}, or \code{"female"}.
Rows with sex recorded as \code{"Not stated"} are dropped when
filtering to male or female. Short forms \code{"m"}/\code{"f"} are
accepted.}
}
\value{
An \code{ato_tbl} with one row per occupation-sex-income
combination. Multi-year queries add a \code{year} column.
Monetary values in nominal AUD of the reporting year.
}
\description{
Returns the Individuals Table 14 (occupation by sex by taxable
income range). Around 1,000 occupations classified by ANZSCO
with aggregate counts, total income, taxable income, and tax
payable. The ATO migrated from ANZSCO 2013 to ANZSCO 2021
across the 2022-23 release; cross-year joins on occupation
name or code must account for the recode.
}
\details{
\strong{Classification break.} Releases from 2022-23 onwards use
ANZSCO 2021; earlier releases use ANZSCO 2013. A warning is
emitted when the requested year(s) are at or after this boundary,
or when a multi-year request spans it.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  occ <- ato_individuals_occupation(year = "2022-23",
                                    occupation = "economist")
  head(occ)
  # Multi-year panel
  panel <- ato_individuals_occupation(year = c("2021-22", "2022-23"),
                                      occupation = "nurse")
})
options(op)
}
}
\seealso{
Other individuals: 
\code{\link{ato_individuals}()},
\code{\link{ato_individuals_age}()},
\code{\link{ato_individuals_postcode}()},
\code{\link{ato_individuals_sex}()},
\code{\link{ato_individuals_state}()}
}
\concept{individuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{ato_manifest_write}
\alias{ato_manifest_write}
\title{Write the session manifest to a file}
\usage{
ato_manifest_write(path, format = c("auto", "csv", "yaml", "json"))
}
\arguments{
\item{path}{Output file path. Extension determines format if
\code{format = "auto"}: \code{.csv} to CSV, \code{.yaml}/\code{.yml} to YAML,
\code{.json} to JSON.}

\item{format}{One of \code{"auto"} (infer from extension), \code{"csv"},
\code{"yaml"}, or \code{"json"}.}
}
\value{
Invisibly, the absolute path to the written file.
}
\description{
Writes the manifest to a file in the requested format. Call
at the end of an analysis script; commit the manifest
alongside the paper for full reproducibility.
}
\examples{
\donttest{
p <- tempfile(fileext = ".csv")
ato_manifest_clear()
ato_manifest_write(p)
}
}
\seealso{
Other reproducibility: 
\code{\link{ato_deposit_zenodo}()},
\code{\link{ato_manifest}()},
\code{\link{ato_manifest_clear}()},
\code{\link{ato_sha256}()},
\code{\link{ato_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tax_gaps.R
\name{ato_tax_gaps}
\alias{ato_tax_gaps}
\title{Australian tax gaps estimates}
\source{
Australian Taxation Office Tax Gaps publication,
CC BY 2.5 AU.
}
\usage{
ato_tax_gaps(sheet = 1)
}
\arguments{
\item{sheet}{Optional sheet name or index. The workbook
contains separate sheets for each tax-gap population (e.g.
"Large corporate", "Small business", "Individuals"). Pass
the sheet name to extract a specific population. \code{NULL}
(default) returns sheet 1 (overview).}
}
\value{
An \code{ato_tbl}. Tax-gap estimates are in nominal AUD
millions of the reporting year and typically accompanied by
a percentage-gap column.
}
\description{
Returns the ATO's annual Tax Gap publication: estimates of
the difference between the tax theoretically payable under
current law and the tax actually collected, across each tax
type and taxpayer population (individuals not in business,
small business, large corporate, GST, excise, fuel tax
credits, PRRT, superannuation guarantee).
}
\details{
The Tax Gap series is used by Treasury (every MYEFO), the
Parliamentary Budget Office, and academic researchers as the
headline measure of revenue integrity.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  gaps <- ato_tax_gaps()
  head(gaps)
})
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Australian tax gaps
-- overview}. Methodology notes on bottom-up, top-down, and
random-inquiry approaches to the tax-gap estimation.

HMRC (annual). \emph{Measuring tax gaps}. Sister methodology
paper applied by HM Revenue and Customs in the UK; the ATO
series was partly inspired by this literature.

Organisation for Economic Co-operation and Development (2017).
\emph{Shining Light on the Shadow Economy: Opportunities and
Threats}. Paris. Synthesises tax-gap measurement practice
across OECD member countries.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/whm.R
\name{ato_whm}
\alias{ato_whm}
\title{Working Holiday Maker tax data}
\source{
Australian Taxation Office Taxation Statistics.
Licensed CC BY 2.5 AU.
}
\usage{
ato_whm(year = "latest")
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns aggregate Working Holiday Maker tax data: number of
backpackers, total earnings, tax paid. Relevant for migration
and labour-market policy analysis.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_whm(year = "2022-23"))
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Migration Act 1958}, visa
subclasses 417 and 462; \emph{Working Holiday Maker
Reform Act 2016}. Establishes the 15 per cent flat
tax rate from the first dollar of WHM earnings.

Productivity Commission (2016). \emph{Migrant Intake into
Australia}. Includes WHM labour-market analysis.
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to_taxstats.R
\name{ato_to_taxstats}
\alias{ato_to_taxstats}
\title{Remap an ato_tbl to the taxstats microdata column schema}
\usage{
ato_to_taxstats(df, direction = c("to_taxstats", "from_taxstats"))
}
\arguments{
\item{df}{An \code{ato_tbl} or data frame.}

\item{direction}{\code{"to_taxstats"} (default, aggregate -> microdata)
or \code{"from_taxstats"} (microdata -> aggregate).}
}
\value{
A data frame with renamed columns. \code{ato_tbl} class and
provenance attributes preserved.
}
\description{
Takes an \code{ato_tbl} with aggregate column names (produced by any
\verb{ato_*} function) and renames columns to match the \code{taxstats}
(or \code{taxstats2}) 2\% microdata sample schema used by Hugh
Parsonage's DRAT package. Enables consistent variable
definitions when moving between aggregate views and microdata
prototyping.
}
\details{
The bundled schema map (\code{ato_schema_map()}) mirrors the
column names from Parsonage's \code{taxstats} and \code{taxstats2}
packages, which in turn use the ATO Individual Sample File
variable names. Because \code{taxstats} is DRAT-distributed and not
on CRAN, this function imposes the mapping as a static table
rather than programmatically introspecting the \code{taxstats}
namespace. Re-check the bundled map against the \code{taxstats}
NAMESPACE when the ATO publishes a revised Sample File schema.

Unknown columns pass through unchanged. Use
\code{\link{ato_harmonise}} first if the input panel has drift
in source column names.
}
\examples{
df <- data.frame(postcode = "2000", taxable_income = 80000,
                 medicare_levy = 1600)
ato_to_taxstats(df)
}
\references{
Parsonage, H. (2019). \emph{taxstats: 2 per cent Individual
Sample File from the Australian Taxation Office}. R package
(DRAT).
\url{https://github.com/HughParsonage/taxstats}

Parsonage, H. (2024). \emph{grattan: Perform Common Quantitative
Tasks for Australian Analysts}. R package version 2026.1.1.
\url{https://cran.r-project.org/package=grattan}

Australian Taxation Office (2024). \emph{Taxation Statistics:
Individual Sample File documentation}.
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_deflate}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_schema_map}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gst.R
\name{ato_industry}
\alias{ato_industry}
\title{Industry aggregates across entity types}
\source{
Australian Taxation Office Taxation Statistics.
Licensed CC BY 2.5 AU.
}
\usage{
ato_industry(
  year = "latest",
  entity = c("company", "individual", "all"),
  anzsic = NULL
)
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{entity}{One of \code{"individual"}, \code{"company"} (default), or
\code{"all"}.}

\item{anzsic}{Optional substring filter on industry name.}
}
\value{
An \code{ato_tbl}.
}
\description{
Derived helper that returns an ANZSIC industry breakdown based
on either individual, company, or all entities for the year.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  i <- ato_industry(year = "2022-23", entity = "company",
                    anzsic = "manufacturing")
  head(i)
})
options(op)
}
}
\seealso{
Other gst: 
\code{\link{ato_gst}()}
}
\concept{gst}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/charities.R
\name{ato_charities}
\alias{ato_charities}
\title{Charity and deductible gift recipient data}
\source{
Australian Taxation Office charity statistics on
data.gov.au. Licensed CC BY 2.5 AU.
}
\usage{
ato_charities(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g. \code{"2021-22"})
or \code{"latest"}.}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD.
}
\description{
Returns the ATO's data on income tax-exempt entities and
Deductible Gift Recipients (DGRs): entity counts, income,
expenditure, and gift deductions by charity subtype and
state. Covers public benevolent institutions, health promotion
charities, environmental organisations, and other DGR
categories.
}
\details{
Used by Treasury (charity tax expenditure estimates),
researchers studying the non-profit sector, and civil society
policy analysts.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  ch <- ato_charities(year = "2021-22")
  head(ch)
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fuel_tax_credits.R
\name{ato_fuel_tax_credits}
\alias{ato_fuel_tax_credits}
\title{Fuel Tax Credits by industry and claim period}
\source{
Australian Taxation Office Excise and Fuel Tax Credit
data. Licensed CC BY 3.0 AU.
}
\usage{
ato_fuel_tax_credits(year = "latest", by = c("industry", "fuel", "period"))
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{by}{One of \code{"industry"} (default, by ANZSIC division),
\code{"fuel"} (by fuel type), or \code{"period"} (quarterly rates).}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns the Fuel Tax Credits scheme data: entitlement rates
by fuel type, claim totals by industry. FTC is a major
implicit fossil-fuel subsidy and is a key lens for
decarbonisation policy cost-benefit analysis.
}
\details{
The ATO publishes FTC data as part of the Excise Data release
and in standalone FTC tables.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(head(ato_fuel_tax_credits(year = "latest", by = "industry")))
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Fuel Tax Act 2006}; \emph{Fuel
Tax (Consequential and Transitional Provisions) Act 2006}.

Denniss, R. and Grudnoff, M. (2021). \emph{Fossil fuel
subsidies in Australia}. The Australia Institute. FTC-as-
subsidy framing used in decarbonisation policy analysis.

Intergovernmental Panel on Climate Change (2022). \emph{Climate
Change 2022: Mitigation of Climate Change}. Chapter 13
covers fossil-fuel subsidy reform.
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosswalk.R
\name{ato_crosswalk}
\alias{ato_crosswalk}
\title{Load a bundled ATO crosswalk table}
\usage{
ato_crosswalk(name = c("anzsic", "anzsco", "postcode", "cpi", "erp", "budget"))
}
\arguments{
\item{name}{One of \code{"anzsic"}, \code{"anzsco"}, \code{"postcode"},
\code{"cpi"}, \code{"erp"}, \code{"budget"}.}
}
\value{
A data frame.
}
\description{
Returns one of the bundled classification crosswalks. Used
internally by \code{\link[=ato_harmonise]{ato_harmonise()}} and available for user-level
panel work.
}
\details{
Bundled crosswalks (at division/major-group level):
\itemize{
\item \code{"anzsic"}: ANZSIC 2006 to 2020 (19 divisions, complete)
\item \code{"anzsco"}: ANZSCO 2013 to 2021 (8 major groups, complete)
\item \code{"postcode"}: postcode first-digit to state anchors
\item \code{"cpi"}: ABS CPI annual, base 2011-12 = 1.0
\item \code{"erp"}: ABS Estimated Resident Population, June 30 annual
\item \code{"budget"}: Final Budget Outcome reference totals
}

For 4-digit ANZSIC, 6-digit ANZSCO, or postcode-to-SA2/LGA/CED
crosswalks, fetch the full tables from ABS. The bundled
division/major-group level covers cross-year ATO Taxation
Statistics joins at the industry headings used in all ATO
tables.
}
\examples{
ato_crosswalk("anzsic")
ato_crosswalk("cpi")
}
\references{
Australian Bureau of Statistics (2006). \emph{Australian and
New Zealand Standard Industrial Classification (ANZSIC)}.
Catalogue 1292.0.

Australian Bureau of Statistics (2020). \emph{ANZSIC 2006
Update}, cat. 1292.0, divisional structure. Used by ATO
Taxation Statistics from 2022-23.

Australian Bureau of Statistics (2013). \emph{Australian and
New Zealand Standard Classification of Occupations
(ANZSCO)}. Catalogue 1220.0.

Australian Bureau of Statistics (2022). \emph{ANZSCO Revised
Edition}, cat. 1220.0. Used by ATO Taxation Statistics from
2022-23 onward.
}
\seealso{
Other harmonisation: 
\code{\link{ato_deflate}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_schema_map}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrity.R
\name{ato_sha256}
\alias{ato_sha256}
\title{Compute the SHA-256 digest of a file}
\usage{
ato_sha256(file)
}
\arguments{
\item{file}{Path to a local file.}
}
\value{
A length-1 character string (hex digest), or \code{NA} if the
file does not exist.
}
\description{
Wraps \code{\link[tools:md5sum]{tools::md5sum()}} style behaviour for SHA-256 via the
\code{digest} package when available, or falls back to a pure-R
implementation via \code{\link[tools:md5sum]{tools::md5sum()}} + file length as a weaker
check. For integrity work PBO/Grattan-grade, install the
\code{digest} package (Suggests).
}
\examples{
f <- tempfile()
writeLines("hello", f)
ato_sha256(f)
}
\seealso{
Other reproducibility: 
\code{\link{ato_deposit_zenodo}()},
\code{\link{ato_manifest}()},
\code{\link{ato_manifest_clear}()},
\code{\link{ato_manifest_write}()},
\code{\link{ato_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cite.R
\name{ato_cite}
\alias{ato_cite}
\title{Cite an ato_tbl (or URL) in BibTeX and plain-text form}
\usage{
ato_cite(x, style = c("text", "bibtex", "apa"), doi = NULL)
}
\arguments{
\item{x}{Either an \code{ato_tbl} (as returned by any \verb{ato_*} data
function) or a character URL pointing to an ATO data.gov.au
resource.}

\item{style}{One of \code{"text"} (default, plain-text footnote),
\code{"bibtex"}, or \code{"apa"}.}

\item{doi}{Optional DOI (e.g. from \code{\link[=ato_deposit_zenodo]{ato_deposit_zenodo()}}) to
include in BibTeX output as a \code{doi} field and APA suffix.}
}
\value{
A character string. For \code{style = "bibtex"}, a complete
\verb{@misc\{\}} entry.
}
\description{
Returns a citation suitable for footnotes, papers, and
Treasury-grade briefs. Uses the provenance attributes
attached to every \code{ato_tbl}: source URL, licence, retrieval
date, title, snapshot pin, and SHA-256 digest.
}
\details{
BibTeX output includes the SHA-256 digest (first 12 hex chars)
and snapshot pin (when set via \code{\link[=ato_snapshot]{ato_snapshot()}}) in the
\code{note} field, which is what research reviewers need to verify
the provenance of a downstream result.
}
\examples{
x <- data.frame(a = 1)
x <- structure(x,
  ato_source = "https://data.gov.au/data/dataset/example.xlsx",
  ato_licence = "CC BY 2.5 AU",
  ato_retrieved = as.POSIXct("2026-04-23 00:00:00", tz = "UTC"),
  ato_title = "ATO individuals 2022-23",
  ato_sha256 = "abc123def456",
  ato_snapshot_date = "2026-04-23",
  class = c("ato_tbl", "data.frame"))

ato_cite(x)
ato_cite(x, style = "bibtex")
# DOI style: supply any minted DOI (Zenodo, DataCite, etc.).
# The placeholder below is illustrative only.
ato_cite(x, style = "apa", doi = "10.5281/zenodo.XXXXXXXX")
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ato-package.R
\docType{package}
\name{ato-package}
\alias{ato}
\alias{ato-package}
\title{ato: Australian Taxation Office Data}
\description{
Tidy R access to Australian Taxation Office ('ATO') Taxation
Statistics and related datasets via the 'data.gov.au' CKAN API.
}
\section{Main function families}{

\itemize{
\item \code{\link[=ato_catalog]{ato_catalog()}}, \code{\link[=ato_download]{ato_download()}}: discovery and generic
resource access.
\item \code{\link[=ato_individuals]{ato_individuals()}}, \code{\link[=ato_individuals_postcode]{ato_individuals_postcode()}},
\code{\link[=ato_individuals_occupation]{ato_individuals_occupation()}}: individual tax return items.
\code{ato_individuals_postcode()} accepts a vector of years for
time-series panels.
\item \code{\link[=ato_companies]{ato_companies()}}: company tax across Tables 1-9 (snapshot,
by size, by entity type, by industry, by industry-and-size,
by sub-industry, by taxable status, by source, by expenses).
\item \code{\link[=ato_super_funds]{ato_super_funds()}}: APRA-regulated super funds and SMSFs.
\item \code{\link[=ato_top_taxpayers]{ato_top_taxpayers()}}: Corporate Tax Transparency release
(Income tax or PRRT sheet).
\item \code{\link[=ato_gst]{ato_gst()}}, \code{\link[=ato_industry]{ato_industry()}}: GST and industry aggregates.
\item \code{\link[=ato_tax_gaps]{ato_tax_gaps()}}: annual Tax Gap estimates across tax heads.
\item \code{\link[=ato_rdti]{ato_rdti()}}: Research and Development Tax Incentive
claimants.
\item \code{\link[=ato_irpd]{ato_irpd()}}: International Related Party Dealings (BEPS /
transfer pricing).
\item \code{\link[=ato_excise]{ato_excise()}}: excise rates, fuel tax credits, beer and
spirits clearances.
\item \code{\link[=ato_sme_benchmarks]{ato_sme_benchmarks()}}: Small Business Benchmarks.
\item \code{\link[=ato_help]{ato_help()}}: HELP / AASL / VSL student-loan statistics.
\item \code{\link[=ato_cite]{ato_cite()}}: citation helper (BibTeX, APA, plain text).
\item \code{\link[=ato_cache_info]{ato_cache_info()}}, \code{\link[=ato_clear_cache]{ato_clear_cache()}}: cache management.
}
}

\section{Caveats for users of ATO data}{


\strong{Nominal AUD.} All monetary values are in nominal Australian
dollars of the reporting year. Use \code{inflateR::inflate()} or
ABS CPI series for real-term comparisons across years.

\strong{Fiscal year convention.} \code{"2022-23"} refers to the
Australian income year from 1 July 2022 to 30 June 2023.

\strong{Confidentiality suppression.} The ATO replaces values with
\code{"np"} (not published), \code{"*"}, \code{"\\u2021"}, or similar tokens when
fewer than ten taxpayers (or fewer than 50 returns for
postcode data) fall in a cell. The package coerces these to
\code{NA} so numeric columns stay numeric. Summing a column with
\code{na.rm = TRUE} is the correct default; be explicit about what
you do with suppressed cells in any distributional analysis.

\strong{Schema drift.} Column names and table numbering shift
year-to-year (e.g. occupation data has been T13, T14, T15 in
different years). The package cleans names to snake_case on
ingestion but does not normalise schemas across years. A
cross-year join requires explicit column mapping.

\strong{Classification migration.} ANZSIC 2006 to ANZSIC 2020 and
ANZSCO 2013 to ANZSCO 2021 recodes affect industry and
occupation series. Users stacking across releases should
inspect the classification version used in each release.

\strong{Silent revisions.} CKAN \code{metadata_modified} timestamps
change without a version bump when the ATO republishes a
corrected file. Run \code{\link[=ato_clear_cache]{ato_clear_cache()}} to force a refresh.

\strong{Microdata is out of scope.} The 2\% Individual Sample File
is distributed separately through Hugh Parsonage's
\href{https://github.com/HughParsonage/taxstats}{taxstats} DRAT
repo. ALife (the ATO Longitudinal Information Files) is
restricted-access microdata via the ATO DataLab and requires
a researcher application; it is not provided by this package.

\strong{Citation.} Use \code{\link[=ato_cite]{ato_cite()}} on any returned \code{ato_tbl} to
produce a Treasury-grade footnote, APA reference, or BibTeX
entry with the source URL, licence, retrieval date, and title.
}

\section{Data source}{

Taxation Statistics are published annually by the ATO on
\url{https://www.ato.gov.au/about-ato/research-and-statistics/} and
mirrored at \url{https://data.gov.au/data/organization/australiantaxationoffice}.
Most datasets are licensed under Creative Commons Attribution 2.5
Australia; Corporate Tax Transparency and the Voluntary Tax
Transparency Code are CC BY 3.0 Australia.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://charlescoverdale.github.io/ato/}
  \item \url{https://github.com/charlescoverdale/ato}
  \item Report bugs at \url{https://github.com/charlescoverdale/ato/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charlesfcoverdale@gmail.com}

}
\concept{Australian tax}
\concept{income distribution}
\concept{taxation statistics}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/medicare_levy.R
\name{ato_medicare_levy}
\alias{ato_medicare_levy}
\title{Medicare Levy and Medicare Levy Surcharge}
\source{
Australian Taxation Office Taxation Statistics
Individuals. Licensed CC BY 2.5 AU.
}
\usage{
ato_medicare_levy(year = "latest", component = c("levy", "surcharge"))
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{component}{One of \code{"levy"} (default, standard Medicare
Levy) or \code{"surcharge"} (MLS).}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns aggregate Medicare Levy and MLS data from Taxation
Statistics Individuals. The 2\% Medicare Levy is on most
taxable income; MLS is an additional 1.0 to 1.5\% on high-income
earners without adequate private hospital cover. Used in
private health insurance reform analysis.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_medicare_levy(year = "2022-23", component = "surcharge"))
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Medicare Levy Act 1986};
\emph{A New Tax System (Medicare Levy Surcharge -- Fringe
Benefits) Act 1999}.

Productivity Commission (2015). \emph{Efficiency in Health}.
Analysis of Medicare Levy and MLS distributional effects.
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/harmonise.R
\name{ato_harmonise}
\alias{ato_harmonise}
\title{Harmonise column names in a multi-year ATO panel}
\usage{
ato_harmonise(df)
}
\arguments{
\item{df}{A data frame (typically an \code{ato_tbl} with \code{year}
column from a multi-year call).}
}
\value{
A data frame with harmonised names. \code{ato_tbl} class
and provenance attributes are preserved.
}
\description{
ATO renames columns across annual releases; a stacked panel
from \code{ato_individuals_postcode(year = c("2020-21", "2021-22"))}
may have inconsistent names like \code{total_income} vs
\code{total_income_or_loss}. \code{ato_harmonise()} renames columns to
the first variant in \code{ATO_COL_VARIANTS} so panels are join-ready.
}
\details{
Unknown columns are left alone. Columns that collide after
renaming (because two variants map to the same canonical name)
emit a warning; the first column wins.
}
\examples{
df <- data.frame(postcode = "2000",
                 total_income_or_loss = 100,
                 state_territory = "NSW")
ato_harmonise(df)
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_deflate}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_schema_map}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deflate.R
\name{ato_deflate}
\alias{ato_deflate}
\title{Deflate nominal AUD to real AUD}
\usage{
ato_deflate(x, year, base = "2022-23", cpi = NULL)
}
\arguments{
\item{x}{Numeric vector of nominal AUD values.}

\item{year}{Character vector of financial years for each entry
in \code{x}, in \code{"YYYY-YY"} form. Must be the same length as \code{x}.}

\item{base}{Base financial year for real terms (default
\code{"2022-23"}).}

\item{cpi}{Optional override: a data frame with columns
\code{financial_year} and \code{cpi_all_groups_australia}. Default
uses the bundled ABS series.}
}
\value{
Numeric vector of real AUD values in base-year prices.
}
\description{
Converts a numeric vector of nominal AUD figures indexed by
financial year to real AUD of a chosen base year using the
bundled ABS CPI series (annual, All Groups Australia,
2011-12 = 1.0). For the user's \code{inflateR} workflow in
non-Australian contexts, bundle a matching CPI series and
call this with a custom \verb{cpi =} argument.
}
\details{
Uses proportional (Laspeyres-style) adjustment:
\eqn{real = nominal \times (CPI_{base} / CPI_{source})}. The
bundled CPI is the ABS annual All Groups Australia index
published in cat. 6401.0, rebased so that 2011-12 = 1.000.
This is the standard rebasing used in most Australian
time-series work and is consistent with ABS System of National
Accounts methodology (cat. 5204.0).

The formula is exact for a chain-linked index after 1949 (when
the ABS CPI was introduced) and approximate for earlier values
that rely on Commonwealth Statistician retail-price series. Use
a custom \verb{cpi =} argument if you need a different deflator
(e.g. GDP deflator, wage price index, or industry-specific PPI).
}
\examples{
ato_deflate(c(100, 100, 100),
            year = c("2012-13", "2017-18", "2022-23"),
            base = "2022-23")
}
\references{
Australian Bureau of Statistics (2024). \emph{Consumer Price
Index, Australia: Concepts, Sources and Methods}. Catalogue
6461.0.

Australian Bureau of Statistics (2024). \emph{Consumer Price
Index, Australia}. Catalogue 6401.0.

Diewert, W.E. (1998). "Index Number Issues in the Consumer Price
Index." \emph{Journal of Economic Perspectives}, 12(1),
47-58. \doi{10.1257/jep.12.1.47}
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_schema_map}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapshot.R
\name{ato_snapshot}
\alias{ato_snapshot}
\title{Pin or inspect the session snapshot date}
\usage{
ato_snapshot(date)
}
\arguments{
\item{date}{ISO \code{"YYYY-MM-DD"} character, \code{Date}, or \code{POSIXct}.
Pass \code{NULL} to clear.}
}
\value{
Invisibly, the new pinned date (as \code{Date}), or \code{NULL}.
}
\description{
Call once at the top of an analysis script to declare the vintage
of ATO data you intend to use. Every subsequent \verb{ato_*} fetch
records this date in the \code{ato_tbl} provenance header, in
\code{ato_manifest()} entries, and in \code{ato_cite()} output. Combined
with SHA-256 integrity (see \code{\link[=ato_sha256]{ato_sha256()}} and \code{\link[=ato_manifest]{ato_manifest()}}),
this gives a reproducible audit trail acceptable for PBO or
Grattan-style published work.
}
\details{
If called with no arguments, returns the current pin (or \code{NULL}
if unset).
}
\examples{
ato_snapshot("2026-04-24")
ato_snapshot()
ato_snapshot(NULL)
}
\seealso{
Other reproducibility: 
\code{\link{ato_deposit_zenodo}()},
\code{\link{ato_manifest}()},
\code{\link{ato_manifest_clear}()},
\code{\link{ato_manifest_write}()},
\code{\link{ato_sha256}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\name{ato_meta}
\alias{ato_meta}
\title{Fetch CKAN metadata for an ATO dataset}
\usage{
ato_meta(x)
}
\arguments{
\item{x}{Either an \code{ato_tbl} (as returned by any \verb{ato_*} data
function) or a character CKAN package ID / slug (e.g.
\code{"taxation-statistics-2022-23"}, \code{"corporate-transparency"}).}
}
\value{
A list with elements:
\itemize{
\item \code{id}: CKAN package slug
\item \code{title}: human-readable title
\item \code{notes}: dataset description (truncated to 400 chars)
\item \code{licence}: licence title
\item \code{metadata_modified}: ISO timestamp of last CKAN update
\item \code{n_resources}: number of downloadable files
\item \code{resource_urls}: character vector of all resource URLs
}
}
\description{
Returns structured metadata for any ATO dataset on data.gov.au:
title, notes, licence, last-modified timestamp, resource count,
and all resource URLs. Useful for detecting silent updates before
clearing the cache, or for auditing what version of data you have.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  # By package ID
  m <- ato_meta("taxation-statistics-2022-23")
  m$metadata_modified

  # From an ato_tbl
  tbl <- ato_individuals(year = "2022-23")
  ato_meta(tbl)
})
options(op)
}
}
\seealso{
Other configuration: 
\code{\link{ato_cache_info}()},
\code{\link{ato_clear_cache}()}
}
\concept{configuration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/top_taxpayers.R
\name{ato_top_taxpayers}
\alias{ato_top_taxpayers}
\title{Corporate Tax Transparency}
\source{
Australian Taxation Office Corporate Tax Transparency
release. Licensed CC BY 3.0 AU.
}
\usage{
ato_top_taxpayers(
  year = "latest",
  entity_type = c("all", "public", "private", "foreign"),
  sheet = c("income_tax", "prrt")
)
}
\arguments{
\item{year}{\code{"YYYY-YY"} (e.g. \code{"2023-24"}) or \code{"latest"}.}

\item{entity_type}{One of \code{"all"} (default), \code{"public"},
\code{"private"}, or \code{"foreign"}. Matches the CTT \verb{Entity type}
column values \code{"Australian public"}, \code{"Australian private"},
\code{"Foreign-owned"}.}

\item{sheet}{One of \code{"income_tax"} (default, the ~4,000-entity
income-tax sheet) or \code{"prrt"} (petroleum resource rent tax
filers, typically 10-20 entities).}
}
\value{
An \code{ato_tbl} with one row per disclosed entity. All
monetary values are nominal AUD of the reporting year.
}
\description{
Returns the ATO's annual Corporate Tax Transparency release,
mandated by Part 5-25 of the \emph{Taxation Administration Act
1953}. Covers every Australian public company, foreign-owned
company, or Australian-owned private company above the
AUD 100 million total-income threshold (the private-company
threshold was lowered from AUD 200 million to AUD 100 million
for the 2022-23 income year onwards, making all three
categories uniform). The 2023-24 release was published 1
October 2025 and covered 4,110 entities.
}
\details{
The underlying XLSX has three sheets:
\itemize{
\item \strong{Information} (cover/metadata, ~7 rows).
\item \strong{Income tax details} (the headline dataset, ~4,000
entities: total income, taxable income, tax payable).
\item \strong{PRRT details} (petroleum resource rent tax filers,
typically 10-20 entities).
}

Licensed under \strong{CC BY 3.0 Australia} (the Corporate Tax
Transparency and Voluntary Tax Transparency Code releases use
CC BY 3.0 AU; most other Taxation Statistics use CC BY 2.5 AU).
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  top <- ato_top_taxpayers(year = "2023-24")
  head(top)
  # Petroleum resource rent tax sheet
  prrt <- ato_top_taxpayers(year = "2023-24", sheet = "prrt")
  head(prrt)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Taxation Administration Act
1953}, Part 5-25 (Corporate Tax Transparency).

Australian Taxation Office (annual). \emph{Report of entity
tax information}. The statutory Corporate Tax Transparency
release.

Commonwealth Treasury (2013). \emph{Improving the transparency
of Australia's business tax system: Exposure draft
explanatory memorandum}. Rationale for the Part 5-25 regime.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vttc.R
\name{ato_vttc}
\alias{ato_vttc}
\title{Voluntary Tax Transparency Code disclosures}
\source{
Australian Taxation Office Voluntary Tax Transparency
Code disclosures on data.gov.au. Licensed CC BY 3.0 AU.
}
\usage{
ato_vttc(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g. \code{"2022-23"})
or \code{"latest"}.}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD.
}
\description{
Returns the ATO's Voluntary Tax Transparency Code (VTTC)
disclosures: large private companies that voluntarily publish
tax information beyond the Corporate Tax Transparency mandate.
Covers total income, taxable income, tax payable, and effective
tax rate for each disclosing entity.
}
\details{
The VTTC complements \code{\link[=ato_top_taxpayers]{ato_top_taxpayers()}} (which covers
mandatory CTT disclosures for entities above AUD 100m total
income). VTTC signatories may be below or above the CTT
threshold.

Licensed under \strong{CC BY 3.0 Australia} (same as CTT data).
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  vttc <- ato_vttc(year = "2022-23")
  head(vttc)
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individuals.R
\name{ato_individuals}
\alias{ato_individuals}
\title{Individual Taxation Statistics snapshot}
\source{
Australian Taxation Office Taxation Statistics
\url{https://www.ato.gov.au/about-ato/research-and-statistics/}.
Licensed CC BY 2.5 AU.
}
\usage{
ato_individuals(year = "latest")
}
\arguments{
\item{year}{Year in \code{"YYYY-YY"} form (e.g. \code{"2022-23"}) or
\code{"latest"}. \code{"latest"} resolves to the most recently
published release (currently 2022-23).}
}
\value{
An \code{ato_tbl} with one row per aggregate line-item and
columns for count and amount in nominal AUD.
}
\description{
Returns the Individuals Table 1 snapshot: aggregate counts,
total income, taxable income, tax payable, and deductions
across all individual returns (roughly 14 million per year).
The snapshot is the headline table; for finer cuts use the
dedicated functions:
\itemize{
\item \code{\link[=ato_individuals_postcode]{ato_individuals_postcode()}} for geographic breakdowns,
\item \code{\link[=ato_individuals_occupation]{ato_individuals_occupation()}} for occupation × sex × income-range
detail, or
\item \code{\link[=ato_download]{ato_download()}} with a custom \code{pattern} for specific
Tables 2 to 27 (age, sex, state, industry, source of income,
deductions, offsets, CGT, non-residents).
}
}
\details{
Monetary values are nominal AUD of the reporting year. Use
\code{inflateR::inflate()} or the ABS CPI series if you need
real-term comparisons.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  ind <- ato_individuals(year = "2022-23")
  head(ind)
})
options(op)
}
}
\seealso{
Other individuals: 
\code{\link{ato_individuals_age}()},
\code{\link{ato_individuals_occupation}()},
\code{\link{ato_individuals_postcode}()},
\code{\link{ato_individuals_sex}()},
\code{\link{ato_individuals_state}()}
}
\concept{individuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zenodo.R
\name{ato_deposit_zenodo}
\alias{ato_deposit_zenodo}
\title{Prepare a Zenodo deposit payload for the session manifest}
\usage{
ato_deposit_zenodo(
  title = NULL,
  description = NULL,
  creators = list(list(name = "Anonymous")),
  keywords = c("ATO", "taxation", "Australia", "reproducibility"),
  upload = FALSE,
  sandbox = FALSE,
  token = Sys.getenv("ZENODO_TOKEN")
)
}
\arguments{
\item{title}{Deposit title. Defaults to "ATO data snapshot
YYYY-MM-DD" using the current snapshot pin.}

\item{description}{Free-text description. Defaults to a short
auto-generated note listing the datasets fetched.}

\item{creators}{List of creator records. Each should be a list
with \code{name}, optional \code{affiliation}, \code{orcid}. Defaults to a
single anonymous entry; override for published work.}

\item{keywords}{Character vector of keywords. Defaults to
\code{c("ATO", "taxation", "Australia", "reproducibility")}.}

\item{upload}{Logical; if \code{TRUE}, POSTs the deposit to Zenodo
and uploads the manifest CSV. Default \code{FALSE} (dry run).}

\item{sandbox}{Logical; if \code{TRUE}, uses Zenodo Sandbox
(sandbox.zenodo.org) for testing. Default \code{FALSE}.}

\item{token}{Zenodo personal access token. Defaults to
\code{Sys.getenv("ZENODO_TOKEN")}.}
}
\value{
A list with \code{payload} (the JSON metadata), \code{manifest_path}
(where the CSV manifest was staged), and if \code{upload = TRUE},
\code{deposit_id}, \code{doi_prereserve}, and \code{url}.
}
\description{
Builds the JSON metadata payload Zenodo expects for a data
deposit, using the current \code{ato_manifest()} and the snapshot
pin set via \code{\link[=ato_snapshot]{ato_snapshot()}}. The function does NOT upload by
default; it returns the payload and saved manifest path so
you can inspect before calling with \code{upload = TRUE}.
}
\details{
To upload, supply a Zenodo personal access token via the
\code{ZENODO_TOKEN} environment variable (or the \code{token} argument).
Tokens can be generated at
\url{https://zenodo.org/account/settings/applications/}.
}
\examples{
\donttest{
ato_snapshot("2026-04-24")
ato_deposit_zenodo(
  title = "ATO data snapshot for working paper v1",
  creators = list(list(name = "Coverdale, Charles")),
  upload = FALSE
)
}
}
\seealso{
Other reproducibility: 
\code{\link{ato_manifest}()},
\code{\link{ato_manifest_clear}()},
\code{\link{ato_manifest_write}()},
\code{\link{ato_sha256}()},
\code{\link{ato_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/super.R
\name{ato_super_funds}
\alias{ato_super_funds}
\title{Superannuation fund aggregates}
\source{
Australian Taxation Office Taxation Statistics Super
Funds tables + SMSF statistical overview. Licensed CC BY 2.5 AU.
}
\usage{
ato_super_funds(year = "latest", type = c("apra", "smsf", "all"))
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{type}{One of \code{"apra"} (APRA-regulated funds, default),
\code{"smsf"} (SMSF statistical overview), or \code{"all"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns Taxation Statistics Super Funds tables or Self-Managed
Superannuation Fund ('SMSF') aggregates, depending on \code{type}.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  s <- ato_super_funds(year = "2022-23", type = "apra")
  head(s)
})
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Taxation Statistics:
Super funds and SMSF explanatory notes}. Distinguishes
reporting populations: APRA-regulated large funds, SMSFs,
and Pooled Superannuation Trusts.

Australian Prudential Regulation Authority (annual).
\emph{Annual Superannuation Bulletin}. Complementary
APRA-regulated fund statistics.

Commonwealth of Australia. \emph{Superannuation Industry
(Supervision) Act 1993} (SIS Act); \emph{Superannuation
Guarantee (Administration) Act 1992} (SGAA).

Productivity Commission (2018). \emph{Superannuation:
Assessing Efficiency and Competitiveness}. Inquiry report.
}
\concept{super}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individuals_demographic.R
\name{ato_individuals_sex}
\alias{ato_individuals_sex}
\title{Individual tax data by sex}
\source{
Australian Taxation Office Taxation Statistics Individuals.
Licensed CC BY 2.5 AU.
}
\usage{
ato_individuals_sex(year = "latest")
}
\arguments{
\item{year}{\code{"YYYY-YY"}, \code{"latest"}, or a vector of years.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns counts and aggregates split by sex. Thin wrapper around
the ATO "Selected items by sex" table.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_individuals_sex(year = "2022-23"))
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Taxation Statistics:
Individuals explanatory notes}. Age-range breakdowns use the
taxpayer's reported date of birth at lodgement; sex is
self-reported on the return.
}
\seealso{
Other individuals: 
\code{\link{ato_individuals}()},
\code{\link{ato_individuals_age}()},
\code{\link{ato_individuals_occupation}()},
\code{\link{ato_individuals_postcode}()},
\code{\link{ato_individuals_state}()}
}
\concept{individuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sme_benchmarks.R
\name{ato_sme_benchmarks}
\alias{ato_sme_benchmarks}
\title{Small Business Benchmarks}
\source{
Australian Taxation Office Small Business Benchmarks.
Licensed CC BY 2.5 AU.
}
\usage{
ato_sme_benchmarks(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g.
\code{"2023-24"}) or \code{"latest"}. Releases available from
2016-17 onwards.}
}
\value{
An \code{ato_tbl} with one row per (industry, turnover
band, ratio) combination. Ratios are percentages.
}
\description{
Returns the ATO's Small Business Benchmarks: industry-specific
performance ranges (cost of sales / turnover, total expenses /
turnover, labour / turnover, etc.) derived from small-business
income tax returns. Used by the ATO to identify outlier
taxpayers, by small-business advisors for comparative
analysis, and by tax integrity researchers.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  bm <- ato_sme_benchmarks(year = "2023-24")
  head(bm)
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/companies.R
\name{ato_companies}
\alias{ato_companies}
\title{Company Taxation Statistics}
\source{
Australian Taxation Office Taxation Statistics Company
Tables. Licensed CC BY 2.5 AU.
}
\usage{
ato_companies(
  year = "latest",
  table = c("industry", "snapshot", "key_items_by_size", "entity_type",
    "industry_by_size", "sub_industry", "taxable_status", "source", "expenses"),
  industry = NULL
)
}
\arguments{
\item{year}{\code{"YYYY-YY"}, \code{"latest"}, or a vector of years for
a multi-year panel. Multi-year requests add a \code{year} column.}

\item{table}{One of \code{"snapshot"}, \code{"key_items_by_size"},
\code{"entity_type"}, \code{"industry"} (default), \code{"industry_by_size"},
\code{"sub_industry"}, \code{"taxable_status"}, \code{"source"}, or
\code{"expenses"}.}

\item{industry}{Optional substring filter on industry name
(applied only when the fetched table has an industry column).}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD of the
reporting year.
}
\description{
Returns the annual Company Taxation Statistics tables. The
Company release ships tables covering entity type, turnover
band, industry, taxable status, source of income, and
expense deductions. Pick the table that matches your
question:
}
\details{
\itemize{
\item \strong{snapshot} (T1): aggregate counts, total income, net tax
across all companies (~1m entities)
\item \strong{key_items_by_size} (T2): net tax by company size band
\item \strong{entity_type} (T3): split by public/private/co-operative
\item \strong{industry} (T4, default): key items by 2-digit ANZSIC
subdivision
\item \strong{industry_by_size} (T5): industry x turnover band
\item \strong{sub_industry} (T6): 4-digit ANZSIC class detail
\item \strong{taxable_status} (T7): items by taxable status
\item \strong{source} (T8): source of income
\item \strong{expenses} (T9): expense and deduction categories
}

\strong{Classification break.} Releases from 2022-23 onwards use
ANZSIC 2020; earlier releases use ANZSIC 2006. A warning is
emitted when the requested year(s) are at or after this boundary,
or when a multi-year request spans it.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  s <- ato_companies(year = "2022-23", table = "snapshot")
  head(s)
  m <- ato_companies(year = "2022-23", industry = "mining")
  head(m)
  # Multi-year industry panel
  panel <- ato_companies(year = c("2021-22", "2022-23"))
})
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Taxation Statistics:
Company tables explanatory notes}. Methodology notes on
lodgement cut-off, entity-type definitions, and turnover-band
thresholds. Accessible from
\url{https://www.ato.gov.au/about-ato/research-and-statistics/in-detail/taxation-statistics/}.

Australian Bureau of Statistics (2020). \emph{Australian and
New Zealand Standard Industrial Classification (ANZSIC)},
2006 revision with 2020 update. Catalogue 1292.0.
}
\concept{companies}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fbt.R
\name{ato_fbt}
\alias{ato_fbt}
\title{Fringe Benefits Tax statistics}
\source{
Australian Taxation Office FBT Taxation Statistics
on data.gov.au. Licensed CC BY 2.5 AU.
}
\usage{
ato_fbt(year = "latest")
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g. \code{"2022-23"})
or \code{"latest"}.}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD.
}
\description{
Returns the ATO's annual Fringe Benefits Tax (FBT) Taxation
Statistics: employer counts, gross taxable value, FBT payable,
and employee benefit counts by benefit type and industry. Used
by Treasury, PBO, and researchers evaluating the FBT concession
system (electric vehicles, remote area exemptions, novated
leases).
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  fbt <- ato_fbt(year = "2022-23")
  head(fbt)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Fringe Benefits Tax Assessment
Act 1986}. Substantive FBT law; ATO rulings (TR series)
elaborate taxable-value methodology.

Australian Taxation Office (annual). \emph{FBT explanatory
notes}. Definitions of reportable benefits, gross-up factors
(Type 1 and Type 2), and otherwise-deductible rule.

Treasury (2022). \emph{Electric Car Discount Bill}.
Explanatory memorandum for the EV FBT exemption introduced
1 July 2022.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rba.R
\name{ato_rba}
\alias{ato_rba}
\title{RBA Commonwealth receipts (H1 statistical table)}
\source{
Reserve Bank of Australia Statistical Tables H1
\url{https://www.rba.gov.au/statistics/tables/}.
}
\usage{
ato_rba(series = c("receipts", "income_tax"))
}
\arguments{
\item{series}{One of \code{"receipts"} (default, all Commonwealth
receipts by category) or \code{"income_tax"} (income tax only).}
}
\value{
An \code{ato_tbl}.
}
\description{
Pointer to the RBA's H1 series on Commonwealth receipts for
long-run time series. RBA compiles since 1959-60, filling gaps
in ATO Taxation Statistics which start 1994-95.
}
\details{
The RBA publishes H1 as an XLSX with stable URL. This function
fetches it and returns a tidy tibble.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_rba(series = "receipts"))
options(op)
}
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catalog.R
\name{ato_catalog}
\alias{ato_catalog}
\title{ATO dataset catalogue}
\source{
'data.gov.au' CKAN endpoint
\url{https://data.gov.au/data/organization/australiantaxationoffice}.
}
\usage{
ato_catalog(q = NULL)
}
\arguments{
\item{q}{Optional free-text filter (CKAN Solr query). \code{NULL}
returns the full ATO catalogue.}
}
\value{
An \code{ato_tbl} with one row per dataset.
}
\description{
Returns a summary of all datasets published by the Australian
Taxation Office on data.gov.au. Each row is a CKAN "package"
with an id (slug), title, licence, modification date, and
resource count.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  cat <- ato_catalog()
  head(cat[, c("id", "title", "licence")])
})
options(op)
}
}
\seealso{
Other discovery: 
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/individuals_demographic.R
\name{ato_individuals_age}
\alias{ato_individuals_age}
\title{Individual tax data by age range}
\source{
Australian Taxation Office Taxation Statistics Individuals.
Licensed CC BY 2.5 AU.
}
\usage{
ato_individuals_age(year = "latest", sex = c("all", "male", "female"))
}
\arguments{
\item{year}{\code{"YYYY-YY"}, \code{"latest"}, or a vector of years.}

\item{sex}{One of \code{"all"} (default), \code{"male"}, or \code{"female"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns Taxation Statistics Individuals Table 2 (approximately):
counts, total income, taxable income, and tax payable by age
range and (usually) sex. Age ranges are 5-year bands for most
of working life plus wider bands at the tails.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_individuals_age(year = "2022-23", sex = "female"))
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Taxation Statistics:
Individuals explanatory notes}. Age-range breakdowns use the
taxpayer's reported date of birth at lodgement; sex is
self-reported on the return.
}
\seealso{
Other individuals: 
\code{\link{ato_individuals}()},
\code{\link{ato_individuals_occupation}()},
\code{\link{ato_individuals_postcode}()},
\code{\link{ato_individuals_sex}()},
\code{\link{ato_individuals_state}()}
}
\concept{individuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/excise.R
\name{ato_excise}
\alias{ato_excise}
\title{Excise and fuel tax credit rates and clearances}
\source{
Australian Taxation Office excise data.
Licensed CC BY 2.5 AU.
}
\usage{
ato_excise(table = c("excise_rates", "ftc_rates", "beer", "spirits"))
}
\arguments{
\item{table}{One of \code{"beer"}, \code{"spirits"}, \code{"excise_rates"}
(default), or \code{"ftc_rates"}.}
}
\value{
An \code{ato_tbl}. Rates are in AUD per litre (or per kg
for tobacco); volumes are in megalitres or similar.
}
\description{
Returns ATO excise data, covering four sub-releases:
\itemize{
\item \strong{beer} : beer clearances summary (volumes by product class)
\item \strong{spirits} : spirits and other excisable beverages clearances
\item \strong{excise_rates} : historical excise rate schedule (all
excise categories, quarterly indexed rates)
\item \strong{ftc_rates} : historical Fuel Tax Credit rates
}
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  rates <- ato_excise("excise_rates")
  head(rates)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Excise Act 1901};
\emph{Excise Tariff Act 1921}; \emph{Fuel Tax Act 2006}.

Australian Taxation Office (annual). \emph{Excise data:
methodology and indexation notes}. Excise rates are indexed
to the Consumer Price Index twice a year (February and
August) for most commodities.

Productivity Commission (2016). \emph{Migrant Intake into
Australia} (for tobacco excise distributional analysis);
\emph{Harmful Drinking} inquiry (for alcohol excise
distributional analysis).
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_irpd}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/division293.R
\name{ato_division293}
\alias{ato_division293}
\title{Division 293 tax assessments (high-income super contributions)}
\source{
Australian Taxation Office Taxation Statistics
Individuals. Licensed CC BY 2.5 AU.
}
\usage{
ato_division293(year = "latest")
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns Division 293 tax data: number of assessments, average
Division 293 liability, and distribution by income band. Division
293 applies an extra 15\% tax on concessional super contributions
for individuals with combined income plus low-tax super
contributions above AUD 250,000. Central to retirement-income
reform analysis (e.g. Grattan's "Better Super" proposals).
}
\details{
Published as part of the Individuals Taxation Statistics
(Table 3b in recent releases).
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_division293(year = "2022-23"))
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Income Tax Assessment Act
1997}, Division 293. Extra 15 per cent tax on concessional
super contributions for high-income earners.

Daley, J., Coates, B. and Wood, D. (2018). \emph{Money in
retirement: more than enough}. Grattan Institute. Uses
Division 293 distributional data in reform analysis.
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{ato_manifest}
\alias{ato_manifest}
\title{Return the session manifest of fetched ATO datasets}
\usage{
ato_manifest(format = c("df", "yaml", "json"))
}
\arguments{
\item{format}{One of \code{"df"} (default, tidy data frame),
\code{"yaml"}, or \code{"json"}.}
}
\value{
A data frame, YAML string, or JSON string depending on
\code{format}.
}
\description{
Every call to a data function (\code{ato_individuals()},
\code{ato_companies()}, etc.) appends one row to the session
manifest, recording URL, dataset title, CKAN resource and
package IDs where resolvable, SHA-256 of the cached file,
size, retrieval timestamp, and the snapshot pin set via
\code{\link[=ato_snapshot]{ato_snapshot()}}. Duplicate URLs within a session are
deduplicated (last fetch wins).
}
\details{
Attach the output to your paper's appendix, deposit it to
Zenodo with \code{\link[=ato_deposit_zenodo]{ato_deposit_zenodo()}} to mint a DOI, or export
with \code{\link[=ato_manifest_write]{ato_manifest_write()}} for CI artefacts.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
ato_manifest_clear()
ato_snapshot("2026-04-24")
try(ato_individuals(year = "2022-23"))
ato_manifest()
options(op)
}
}
\seealso{
Other reproducibility: 
\code{\link{ato_deposit_zenodo}()},
\code{\link{ato_manifest_clear}()},
\code{\link{ato_manifest_write}()},
\code{\link{ato_sha256}()},
\code{\link{ato_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/irpd.R
\name{ato_irpd}
\alias{ato_irpd}
\title{International Related Party Dealings (IRPD)}
\source{
Australian Taxation Office International Related
Party Dealings release. Licensed CC BY 2.5 AU.
}
\usage{
ato_irpd(year = "latest", table = 1L)
}
\arguments{
\item{year}{Income year in \code{"YYYY-YY"} form (e.g.
\code{"2023-24"}) or \code{"latest"}.}

\item{table}{Integer 1, 2, or 3. Default \code{1}.}
}
\value{
An \code{ato_tbl}. Monetary values in nominal AUD.
}
\description{
Returns the ATO's International Related Party Dealings data,
which captures intra-group cross-border payments and
receivables reported by Australian corporate taxpayers. Core
dataset for BEPS and transfer-pricing research, transfer
pricing risk assessment, and multinational tax analysis.
}
\details{
The IRPD data is published as a separate CKAN package per
income year (2019-20 through 2023-24). Each annual package
contains three tables:
\itemize{
\item \strong{Table 1} : IRPD totals from 2015-16 to the current year
\item \strong{Table 2} : IRPDs by jurisdiction
\item \strong{Table 3} : Index of chart data
}
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  by_jurisdiction <- ato_irpd(year = "2023-24", table = 2)
  head(by_jurisdiction)
})
options(op)
}
}
\references{
Organisation for Economic Co-operation and Development (2015).
\emph{Transfer Pricing Documentation and Country-by-Country
Reporting, Action 13: 2015 Final Report}. OECD/G20 Base
Erosion and Profit Shifting Project, Paris.
\doi{10.1787/9789264241480-en}

Commonwealth of Australia. \emph{Income Tax Assessment Act
1997}, Subdivision 815-B (Transfer Pricing); \emph{Multinational
Anti-Avoidance Law} (MAAL) and \emph{Diverted Profits Tax}.

Australian Taxation Office (annual). \emph{International
Dealings Schedule (IDS) instructions}. Reporting framework
underlying the IRPD dataset.
}
\seealso{
Other discovery: 
\code{\link{ato_catalog}()},
\code{\link{ato_charities}()},
\code{\link{ato_cite}()},
\code{\link{ato_download}()},
\code{\link{ato_excise}()},
\code{\link{ato_fbt}()},
\code{\link{ato_help}()},
\code{\link{ato_payg}()},
\code{\link{ato_rdti}()},
\code{\link{ato_sme_benchmarks}()},
\code{\link{ato_tax_gaps}()},
\code{\link{ato_top_taxpayers}()},
\code{\link{ato_vttc}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tax_expenditures.R
\name{ato_tax_expenditures}
\alias{ato_tax_expenditures}
\title{Tax Expenditures and Insights Statement (TEIS)}
\source{
Treasury Tax Expenditures and Insights Statement
\url{https://treasury.gov.au/publication/p2025-721342}.
}
\usage{
ato_tax_expenditures(year = "latest")
}
\arguments{
\item{year}{Reference year for the TEIS release, e.g. \code{"2024"}
or \code{"latest"}. Treasury publishes one TEIS per calendar year.}
}
\value{
An \code{ato_tbl} with one row per tax expenditure: label,
category, estimated revenue forgone in AUD millions by year.
}
\description{
Returns the Treasury TEIS annual table of concession-by-concession
tax expenditure estimates in AUD millions. TEIS is the
authoritative cost-of-concessions dataset used in PBO and Grattan
tax reform costings.
}
\details{
TEIS is published by Treasury, not ATO; the function attempts a
CKAN search on data.gov.au for the TEIS release, and falls back
to the Treasury web URL if not indexed.

Key concessions covered: CGT main residence exemption, CGT 50\%
discount, superannuation earnings tax concession, franking credit
refundability, work-related deductions, fuel tax credit scheme,
R&D tax incentive, GST food exemption, and many more.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(head(ato_tax_expenditures("latest")))
options(op)
}
}
\references{
Commonwealth of Australia (annual). \emph{Tax Expenditures
and Insights Statement}. The Treasury, Canberra.
\url{https://treasury.gov.au/publication/p2025-721342}
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{ato_cache_info}
\alias{ato_cache_info}
\title{Inspect the local ato cache}
\usage{
ato_cache_info()
}
\value{
A list with \code{dir}, \code{n_files}, \code{size_bytes},
\code{size_human}, and \code{files}.
}
\description{
Inspect the local ato cache
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
ato_cache_info()
options(op)
}
}
\seealso{
Other configuration: 
\code{\link{ato_clear_cache}()},
\code{\link{ato_meta}()}
}
\concept{configuration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prrt.R
\name{ato_prrt}
\alias{ato_prrt}
\title{Petroleum Resource Rent Tax (PRRT) annual data}
\source{
Australian Taxation Office Taxation Statistics Company.
Licensed CC BY 2.5 AU.
}
\usage{
ato_prrt(year = "latest")
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns PRRT revenue and assessments. PRRT is a 40\% tax on the
profits of offshore petroleum projects; revenues are volatile
and project-specific. Key dataset for resource-tax reform
analysis.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_prrt(year = "2022-23"))
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Petroleum Resource Rent Tax
Assessment Act 1987}. Enabling legislation for the 40 per
cent rent tax on offshore petroleum projects.

Callaghan, M. (2017). \emph{Review of the Petroleum Resource
Rent Tax}. Treasury-commissioned review; reference for
PRRT-reform analysis.
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_international}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to_taxstats.R
\name{ato_schema_map}
\alias{ato_schema_map}
\title{Print the ATO -> taxstats schema map}
\usage{
ato_schema_map()
}
\value{
A data frame with columns \code{ato_aggregate} and
\code{taxstats_microdata}.
}
\description{
Convenience accessor for the bundled column-name mapping.
}
\examples{
head(ato_schema_map())
}
\seealso{
Other harmonisation: 
\code{\link{ato_crosswalk}()},
\code{\link{ato_deflate}()},
\code{\link{ato_harmonise}()},
\code{\link{ato_per_capita}()},
\code{\link{ato_reconcile}()},
\code{\link{ato_to_taxstats}()}
}
\concept{harmonisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{ato_manifest_clear}
\alias{ato_manifest_clear}
\title{Clear the session manifest}
\usage{
ato_manifest_clear()
}
\value{
Invisibly \code{NULL}. Useful at the top of a script when
running repeatedly.
}
\description{
Clear the session manifest
}
\examples{
ato_manifest_clear()
}
\seealso{
Other reproducibility: 
\code{\link{ato_deposit_zenodo}()},
\code{\link{ato_manifest}()},
\code{\link{ato_manifest_write}()},
\code{\link{ato_sha256}()},
\code{\link{ato_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gst.R
\name{ato_gst}
\alias{ato_gst}
\title{GST and activity statement ratios}
\source{
Australian Taxation Office Taxation Statistics.
Licensed CC BY 2.5 AU.
}
\usage{
ato_gst(year = "latest", table = c("overview", "state", "industry", "ratios"))
}
\arguments{
\item{year}{\code{"YYYY-YY"} or \code{"latest"}.}

\item{table}{One of \code{"overview"} (default, GST T1), \code{"state"}
(GST by state), \code{"industry"} (GST by ANZSIC), or
\code{"ratios"} (Activity Statement Ratios).}
}
\value{
An \code{ato_tbl}.
}
\description{
Returns the Taxation Statistics GST tables (T1-T5) or the
Activity Statement Ratios (A1-A5) for the requested year.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try({
  g <- ato_gst(year = "2022-23", table = "industry")
  head(g)
})
options(op)
}
}
\references{
Australian Taxation Office (annual). \emph{Taxation Statistics:
GST and Activity Statement Ratios explanatory notes}.

Commonwealth of Australia. \emph{A New Tax System (Goods and
Services Tax) Act 1999}. Enabling legislation for the 10 per
cent value-added tax introduced 1 July 2000.

Productivity Commission (2018). \emph{Horizontal Fiscal
Equalisation}. Background reference on the GST distribution
formula across states.
}
\seealso{
Other gst: 
\code{\link{ato_industry}()}
}
\concept{gst}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/international.R
\name{ato_international}
\alias{ato_international}
\title{OECD Revenue Statistics comparison}
\source{
OECD Revenue Statistics
\url{https://www.oecd.org/tax/tax-policy/revenue-statistics.htm}.
}
\usage{
ato_international(country = "AUS", year = "latest")
}
\arguments{
\item{country}{Country ISO code or name (default \code{"AUS"}).}

\item{year}{Four-digit year or \code{"latest"}.}
}
\value{
An \code{ato_tbl} with columns \code{country}, \code{year}, \code{tax},
\code{pct_gdp}.
}
\description{
Fetches OECD Revenue Statistics for cross-country tax-to-GDP
benchmarking. Returns tax revenue as percent of GDP by tax
category. Use to contextualise Australian ATO aggregates in
cross-country policy arguments (e.g. OECD average corporate
tax-to-GDP, international ranks for personal income tax).
}
\details{
Thin wrapper pointing users to \verb{readoecd::} for full OECD API
access; returns a minimal tax-to-GDP slice here for convenience.
}
\examples{
\donttest{
op <- options(ato.cache_dir = tempdir())
try(ato_international(country = "AUS"))
options(op)
}
}
\seealso{
Other specialist: 
\code{\link{ato_compliance}()},
\code{\link{ato_division293}()},
\code{\link{ato_fuel_tax_credits}()},
\code{\link{ato_medicare_levy}()},
\code{\link{ato_prrt}()},
\code{\link{ato_rba}()},
\code{\link{ato_state_tax}()},
\code{\link{ato_tax_expenditures}()},
\code{\link{ato_whm}()}
}
\concept{specialist}
